/*
 * FILE:   ui_vwin.c
 * AUTHOR: Colin Perkins <csp@csperkins.org>
 *
 * Copyright (c) 2003 University of Glasgow
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, is permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 
 * 3. All advertising materials mentioning features or use of this software
 *    must display the following acknowledgement:
 * 
 *      This product includes software developed by the University of Glasgow
 * 
 * 4. The name of the University may not be used to endorse or promote 
 *    products derived from this software without specific prior written 
 *    permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE AUTHORS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESSED OR IMPLIED WARRANTIES, INCLUDING,
 * BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY
 * AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO
 * EVENT SHALL THE AUTHORS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT,
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * $Revision: 1.2 $
 * $Date: 2004/10/13 10:25:24 $
 *
 */

#include "config.h"
#include "config_unix.h"
#include "config_win32.h"
#include "debug.h"
#include "video_display.h"
#include "ui/ui_vwin.h"

#ifdef HAVE_TCL_TK
#include "tcl.h"
#include "tk.h"

extern Tcl_Interp      *interp;		/* FIXME */

struct vwin {
	Tk_Window		 window;
	Display			*display;
	Visual			*visual;
	int			 depth;
	display_size_t		 size;
	display_colour_t	 colour;
};
#endif

struct vwin *
ui_create_vwin(display_size_t ds, display_colour_t dc)
{
	UNUSED(ds);
	UNUSED(dc);
	return NULL;
#ifdef NDEF
	/* Create a window of the specified size and colour mode, to    */
	/* display an active video source. Return an opaque identifier. */
	struct vwin 	*w;
	unsigned	 desired_width, desired_height;

	w = malloc(sizeof(struct vwin));
	if (w != NULL) {
		w->size   = ds;
		w->colour = dc;

		/* Create a bare window to draw into... */
		w->window = Tk_CreateWindowFromPath(interp, Tk_MainWindow(interp), ".vw", 0);
		if (w->window == 0) {
			return NULL;
		};

		w->display = Tk_Display(w->window);
		w->visual  = Tk_Visual(w->window);
		w->depth   = Tk_Depth(w->window);

		if (w->depth != 24) {
			printf("Unable to create window: not 24 bit colour\n");
			return NULL;
		}
		if (w->visual->class != TrueColor) {
			printf("Unable to create window: not TrueColor visual\n");
			return NULL;
		}

		switch (ds) {
		case DS_176x144:   
			desired_width  =  176;
			desired_height =  144;
			break;
		case DS_352x288:   
			desired_width  =  352;
			desired_height =  288;
			break;
		case DS_702x576:   
			desired_width  =  702;
			desired_height =  576;
			break;
		case DS_1280x720:  
			desired_width  = 1280;
			desired_height =  720;
			break;
		case DS_1920x1080: 
			desired_width  = 1920;
			desired_height = 1080;
			break;
		default: 	   
			printf("Unknown window size requested\n"); 
			abort();
		}
		Tk_GeometryRequest(w->window, desired_width, desired_height);
		Tk_MapWindow(w->window);

		switch (dc) {
		case DC_RGB :
			/* Attach a shared memory segment, to improve performance */
			w->shm_image = XShmCreateImage(w->display,  w->visual, w->depth, ZPixmap, NULL, &s->vw_shm_segment, 
			                               desired_width, desired_height);
			if ((w->shm_image->width != desired_width) || (w->shm_image->width != desired_height)) {
				debug_msg("Display doesn't support %ux%u shared memory images\n", desired_width, desired_height);
				free(w);
				return NULL;
			}
		
			s->vw_shm_segment.shmid    = shmget(IPC_PRIVATE, w->shm_image->bytes_per_line * w->shm_image->height, IPC_CREAT|0777);
			s->vw_shm_segment.shmaddr  = shmat(s->vw_shm_segment.shmid, 0, 0);
			s->vw_shm_segment.readOnly = False;
		
			s->vw_image->data = s->vw_shm_segment.shmaddr;
		
			if (XShmAttach(w->display, &s->vw_shm_segment) == 0) {
				printf("Cannot attach shared memory segment\n");
				abort();
			}
			break;
		case DC_YUV :
			break;
		default :
			printf("Unknown colour mode requested\n");
			abort();
		}
	}
	return w;
#endif
}


