/*
 * decim_3528kHz_8kHz.h --
 *
 * The following file declares coefficients for a Parks-McClellan
 * decimate-by-441 FIR filter. The input data is at 3528kHz. The
 * passband cutoff is 3.7kHz with 1.5dB ripple. The stopband begins at
 * 4.3kHz with 60dB attenuation.
 *
 * The coefficients were calculated using Matlab 5.3 with the following
 * statements:
 *
 * rp = 1.5;        % Passband ripple
 * rs = 60;         % Stopband attenuation
 * fs = 80 * 44100; % Sampling frequency
 * f = [3700 4300]; % Cutoff frequencies
 * a = [1 0];       % Desired amplitudes
 * % Compute deviations
 * dev = [(10^(rp/20)-1)/(10^(rp/20)+1)  10^(-rs/20)];
 * [n,fo,ao,w] = remezord(f,a,dev,fs);
 * n = 10879;        % remez returns n+1 coefficients
 * b = remez(n,fo,ao,w);
 * save <filename> b -ascii;
 * 
 * The remez function creates n+1 coefficients. 10880 is a multiple of
 * 80. We want a multiple of 80 only in this case specifically because
 * we will upsample by zero-stuffing 79 out of 80 samples.
 *
 * Copyright (c) 2000-2002 The Regents of the University of California.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 * A. Redistributions of source code must retain the above copyright notice,
 *    this list of conditions and the following disclaimer.
 * B. Redistributions in binary form must reproduce the above copyright notice,
 *    this list of conditions and the following disclaimer in the documentation
 *    and/or other materials provided with the distribution.
 * C. Neither the names of the copyright holders nor the names of its
 *    contributors may be used to endorse or promote products derived from this
 *    software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS ``AS
 * IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
 * THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */

#define DECIM441_LENGTH 10880

double decim441[DECIM441_LENGTH] = {
-5.6931991e-04,
-7.5431544e-05,
-7.4324210e-05,
-7.6394920e-05,
-7.5250027e-05,
-7.7274005e-05,
-7.6098940e-05,
-7.8076167e-05,
-7.6878719e-05,
-7.8806089e-05,
-7.7595289e-05,
-7.9462951e-05,
-7.8254896e-05,
-8.0034419e-05,
-7.8877499e-05,
-8.0401456e-05,
-7.9418760e-05,
-8.1816816e-05,
-7.9348578e-05,
-8.1407417e-05,
-7.9861450e-05,
-8.1921510e-05,
-8.0451000e-05,
-8.2405598e-05,
-8.0893543e-05,
-8.2747052e-05,
-8.1185934e-05,
-8.2969287e-05,
-8.1372797e-05,
-8.3119961e-05,
-8.1495185e-05,
-8.3255531e-05,
-8.1507153e-05,
-8.3418104e-05,
-8.1821812e-05,
-8.3293519e-05,
-8.1769647e-05,
-8.3420935e-05,
-8.1921990e-05,
-8.3558005e-05,
-8.2009317e-05,
-8.3597202e-05,
-8.2010098e-05,
-8.3569990e-05,
-8.1969287e-05,
-8.3519772e-05,
-8.1917644e-05,
-8.3465103e-05,
-8.1872369e-05,
-8.3338424e-05,
-8.1861297e-05,
-8.3307477e-05,
-8.1641049e-05,
-8.3139843e-05,
-8.1518937e-05,
-8.3016848e-05,
-8.1364058e-05,
-8.2822996e-05,
-8.1140777e-05,
-8.2584033e-05,
-8.0900202e-05,
-8.2347859e-05,
-8.0674935e-05,
-8.2123319e-05,
-8.0464845e-05,
-8.1898032e-05,
-8.0192915e-05,
-8.1711571e-05,
-7.9948454e-05,
-8.1391304e-05,
-7.9664053e-05,
-8.1131448e-05,
-7.9403419e-05,
-8.0852096e-05,
-7.9101842e-05,
-8.0539969e-05,
-7.8784212e-05,
-8.0228176e-05,
-7.8470847e-05,
-7.9918247e-05,
-7.8143707e-05,
-7.9595592e-05,
-7.7779545e-05,
-7.9219838e-05,
-7.7447527e-05,
-7.8834901e-05,
-7.7029953e-05,
-7.8444223e-05,
-7.6652472e-05,
-7.8068171e-05,
-7.6260426e-05,
-7.7664931e-05,
-7.5849188e-05,
-7.7254079e-05,
-7.5437464e-05,
-7.6841388e-05,
-7.5016652e-05,
-7.6408310e-05,
-7.4592949e-05,
-7.5949237e-05,
-7.4143081e-05,
-7.5524209e-05,
-7.3675058e-05,
-7.5048908e-05,
-7.3214474e-05,
-7.4597922e-05,
-7.2757866e-05,
-7.4125949e-05,
-7.2274574e-05,
-7.3633276e-05,
-7.1782022e-05,
-7.3136423e-05,
-7.1287282e-05,
-7.2634637e-05,
-7.0792612e-05,
-7.2137726e-05,
-7.0269766e-05,
-7.1633177e-05,
-6.9774339e-05,
-7.1109166e-05,
-6.9252187e-05,
-7.0596144e-05,
-6.8745823e-05,
-7.0080171e-05,
-6.8218697e-05,
-6.9541129e-05,
-6.7677717e-05,
-6.8999221e-05,
-6.7139161e-05,
-6.8464831e-05,
-6.6596798e-05,
-6.7919373e-05,
-6.6043909e-05,
-6.7342008e-05,
-6.5489152e-05,
-6.6780482e-05,
-6.4911178e-05,
-6.6200549e-05,
-6.4337568e-05,
-6.5625919e-05,
-6.3753891e-05,
-6.5031468e-05,
-6.3154212e-05,
-6.4432321e-05,
-6.2559967e-05,
-6.3840955e-05,
-6.1962026e-05,
-6.3230417e-05,
-6.1357784e-05,
-6.2611362e-05,
-6.0732265e-05,
-6.2003190e-05,
-6.0116183e-05,
-6.1375613e-05,
-5.9487095e-05,
-6.0750281e-05,
-5.8858680e-05,
-6.0115307e-05,
-5.8216137e-05,
-5.9471074e-05,
-5.7573908e-05,
-5.8826476e-05,
-5.6924826e-05,
-5.8174698e-05,
-5.6267675e-05,
-5.7524855e-05,
-5.5602788e-05,
-5.6861646e-05,
-5.4949895e-05,
-5.6200342e-05,
-5.4279589e-05,
-5.5530658e-05,
-5.3612409e-05,
-5.4863460e-05,
-5.2939353e-05,
-5.4187088e-05,
-5.2259637e-05,
-5.3504719e-05,
-5.1577256e-05,
-5.2828302e-05,
-5.0893393e-05,
-5.2143019e-05,
-5.0208915e-05,
-5.1441662e-05,
-4.9512508e-05,
-5.0748311e-05,
-4.8811754e-05,
-5.0040449e-05,
-4.8105587e-05,
-4.9335911e-05,
-4.7399253e-05,
-4.8623507e-05,
-4.6680341e-05,
-4.7901952e-05,
-4.5965251e-05,
-4.7187732e-05,
-4.5250924e-05,
-4.6461609e-05,
-4.4526946e-05,
-4.5731414e-05,
-4.3788906e-05,
-4.4998650e-05,
-4.3061791e-05,
-4.4263896e-05,
-4.2325787e-05,
-4.3526292e-05,
-4.1590594e-05,
-4.2784722e-05,
-4.0844239e-05,
-4.2036072e-05,
-4.0100665e-05,
-4.1288018e-05,
-3.9351892e-05,
-4.0534325e-05,
-3.8590361e-05,
-3.9776100e-05,
-3.7828688e-05,
-3.9009114e-05,
-3.7071665e-05,
-3.8251089e-05,
-3.6309763e-05,
-3.7482418e-05,
-3.5542043e-05,
-3.6711453e-05,
-3.4770626e-05,
-3.5938971e-05,
-3.3996071e-05,
-3.5160780e-05,
-3.3216858e-05,
-3.4383511e-05,
-3.2435441e-05,
-3.3599037e-05,
-3.1655869e-05,
-3.2812985e-05,
-3.0868817e-05,
-3.2029858e-05,
-3.0083208e-05,
-3.1236491e-05,
-2.9285577e-05,
-3.0438469e-05,
-2.8490381e-05,
-2.9641959e-05,
-2.7688003e-05,
-2.8837431e-05,
-2.6886197e-05,
-2.8035954e-05,
-2.6087473e-05,
-2.7231724e-05,
-2.5281903e-05,
-2.6428554e-05,
-2.4471578e-05,
-2.5618644e-05,
-2.3664472e-05,
-2.4809359e-05,
-2.2850841e-05,
-2.3995714e-05,
-2.2040793e-05,
-2.3184294e-05,
-2.1226687e-05,
-2.2369743e-05,
-2.0414802e-05,
-2.1556686e-05,
-1.9602296e-05,
-2.0743640e-05,
-1.8781995e-05,
-1.9924355e-05,
-1.7962811e-05,
-1.9099872e-05,
-1.7141709e-05,
-1.8281166e-05,
-1.6323450e-05,
-1.7459738e-05,
-1.5502106e-05,
-1.6636240e-05,
-1.4681064e-05,
-1.5815249e-05,
-1.3859287e-05,
-1.4991443e-05,
-1.3035108e-05,
-1.4166821e-05,
-1.2210597e-05,
-1.3336272e-05,
-1.1385080e-05,
-1.2508924e-05,
-1.0557569e-05,
-1.1683135e-05,
-9.7365164e-06,
-1.0857331e-05,
-8.9073043e-06,
-1.0024612e-05,
-8.0796492e-06,
-9.1957158e-06,
-7.2497252e-06,
-8.3620792e-06,
-6.4189236e-06,
-7.5281327e-06,
-5.5902621e-06,
-6.6954987e-06,
-4.7571214e-06,
-5.8653830e-06,
-3.9270621e-06,
-5.0336636e-06,
-3.0992246e-06,
-4.2039246e-06,
-2.2672616e-06,
-3.3689832e-06,
-1.4360208e-06,
-2.5369632e-06,
-6.0604069e-07,
-1.7055410e-06,
2.2276490e-07,
-8.7550631e-07,
1.0512499e-06,
-4.8334197e-08,
1.8809428e-06,
7.8203613e-07,
2.7077838e-06,
1.6108134e-06,
3.5372801e-06,
2.4395267e-06,
4.3646473e-06,
3.2680707e-06,
5.1936395e-06,
4.0969429e-06,
6.0189344e-06,
4.9202234e-06,
6.8426839e-06,
5.7440164e-06,
7.6669472e-06,
6.5682104e-06,
8.4879929e-06,
7.3929788e-06,
9.3097692e-06,
8.2129703e-06,
1.0131264e-05,
9.0337465e-06,
1.0948224e-05,
9.8498125e-06,
1.1765798e-05,
1.0668059e-05,
1.2580490e-05,
1.1480920e-05,
1.3393108e-05,
1.2295196e-05,
1.4205717e-05,
1.3109504e-05,
1.5015280e-05,
1.3918709e-05,
1.5824794e-05,
1.4726658e-05,
1.6630042e-05,
1.5532430e-05,
1.7432676e-05,
1.6333661e-05,
1.8232449e-05,
1.7135218e-05,
1.9031224e-05,
1.7934114e-05,
1.9826110e-05,
1.8729512e-05,
2.0618636e-05,
1.9521733e-05,
2.1408684e-05,
2.0309438e-05,
2.2194650e-05,
2.1098283e-05,
2.2977814e-05,
2.1881261e-05,
2.3758937e-05,
2.2662424e-05,
2.4536354e-05,
2.3440861e-05,
2.5313289e-05,
2.4218850e-05,
2.6085609e-05,
2.4989215e-05,
2.6853011e-05,
2.5756739e-05,
2.7618335e-05,
2.6523638e-05,
2.8379553e-05,
2.7287180e-05,
2.9138932e-05,
2.8044557e-05,
2.9892864e-05,
2.8799480e-05,
3.0644213e-05,
2.9549356e-05,
3.1391937e-05,
3.0297192e-05,
3.2134889e-05,
3.1037862e-05,
3.2871595e-05,
3.1776086e-05,
3.3606902e-05,
3.2513371e-05,
3.4339729e-05,
3.3242847e-05,
3.5066475e-05,
3.3968410e-05,
3.5786907e-05,
3.4689194e-05,
3.6505287e-05,
3.5405240e-05,
3.7217527e-05,
3.6116069e-05,
3.7925386e-05,
3.6822652e-05,
3.8627737e-05,
3.7523888e-05,
3.9326881e-05,
3.8220873e-05,
4.0021267e-05,
3.8910599e-05,
4.0706101e-05,
3.9595810e-05,
4.1387611e-05,
4.0275971e-05,
4.2064560e-05,
4.0951762e-05,
4.2736097e-05,
4.1620937e-05,
4.3403060e-05,
4.2287308e-05,
4.4065484e-05,
4.2946129e-05,
4.4720980e-05,
4.3599143e-05,
4.5369900e-05,
4.4247565e-05,
4.6012549e-05,
4.4889402e-05,
4.6652403e-05,
4.5527132e-05,
4.7284188e-05,
4.6158490e-05,
4.7911425e-05,
4.6783411e-05,
4.8532042e-05,
4.7403050e-05,
4.9146672e-05,
4.8015201e-05,
4.9752310e-05,
4.8620289e-05,
5.0352528e-05,
4.9221381e-05,
5.0949895e-05,
4.9816060e-05,
5.1539928e-05,
5.0404596e-05,
5.2121634e-05,
5.0985600e-05,
5.2698042e-05,
5.1560914e-05,
5.3267211e-05,
5.2128168e-05,
5.3829047e-05,
5.2689299e-05,
5.4383632e-05,
5.3242466e-05,
5.4933114e-05,
5.3790774e-05,
5.5477700e-05,
5.4333117e-05,
5.6011144e-05,
5.4864675e-05,
5.6537928e-05,
5.5390212e-05,
5.7057533e-05,
5.5909288e-05,
5.7570887e-05,
5.6419886e-05,
5.8076515e-05,
5.6924376e-05,
5.8576328e-05,
5.7421404e-05,
5.9068732e-05,
5.7911848e-05,
5.9553407e-05,
5.8394686e-05,
6.0028345e-05,
5.8866306e-05,
6.0496814e-05,
5.9333195e-05,
6.0957274e-05,
5.9792103e-05,
6.1411215e-05,
6.0243269e-05,
6.1856683e-05,
6.0685854e-05,
6.2294298e-05,
6.1121028e-05,
6.2722627e-05,
6.1546801e-05,
6.3142474e-05,
6.1963092e-05,
6.3554877e-05,
6.2372291e-05,
6.3958987e-05,
6.2773843e-05,
6.4355276e-05,
6.3167445e-05,
6.4743287e-05,
6.3553235e-05,
6.5122529e-05,
6.3928686e-05,
6.5492137e-05,
6.4296375e-05,
6.5853637e-05,
6.4654328e-05,
6.6206310e-05,
6.5003527e-05,
6.6551868e-05,
6.5347539e-05,
6.6887846e-05,
6.5679450e-05,
6.7215245e-05,
6.6004248e-05,
6.7533065e-05,
6.6319771e-05,
6.7842005e-05,
6.6625504e-05,
6.8141560e-05,
6.6922799e-05,
6.8433675e-05,
6.7212001e-05,
6.8715968e-05,
6.7493111e-05,
6.8990732e-05,
6.7766106e-05,
6.9255806e-05,
6.8027216e-05,
6.9511109e-05,
6.8280609e-05,
6.9757396e-05,
6.8524432e-05,
6.9994578e-05,
6.8759902e-05,
7.0223057e-05,
6.8985641e-05,
7.0442115e-05,
6.9202831e-05,
7.0651362e-05,
6.9410657e-05,
7.0851844e-05,
6.9607215e-05,
7.1042161e-05,
6.9795334e-05,
7.1222459e-05,
6.9972903e-05,
7.1394138e-05,
7.0142325e-05,
7.1556638e-05,
7.0303286e-05,
7.1710028e-05,
7.0453398e-05,
7.1852307e-05,
7.0593349e-05,
7.1986444e-05,
7.0724176e-05,
7.2110339e-05,
7.0844179e-05,
7.2224394e-05,
7.0956766e-05,
7.2329112e-05,
7.1057081e-05,
7.2424192e-05,
7.1149794e-05,
7.2510267e-05,
7.1232387e-05,
7.2585533e-05,
7.1303850e-05,
7.2649791e-05,
7.1365070e-05,
7.2706245e-05,
7.1418214e-05,
7.2751860e-05,
7.1461357e-05,
7.2788336e-05,
7.1494597e-05,
7.2815055e-05,
7.1517285e-05,
7.2831535e-05,
7.1530742e-05,
7.2839032e-05,
7.1533724e-05,
7.2835021e-05,
7.1526451e-05,
7.2821382e-05,
7.1510175e-05,
7.2798425e-05,
7.1484561e-05,
7.2765305e-05,
7.1448501e-05,
7.2722170e-05,
7.1400766e-05,
7.2668046e-05,
7.1344794e-05,
7.2604456e-05,
7.1277387e-05,
7.2530767e-05,
7.1200503e-05,
7.2446858e-05,
7.1114112e-05,
7.2354027e-05,
7.1018700e-05,
7.2250328e-05,
7.0911846e-05,
7.2137224e-05,
7.0795394e-05,
7.2013267e-05,
7.0668425e-05,
7.1879150e-05,
7.0533249e-05,
7.1736264e-05,
7.0386663e-05,
7.1582265e-05,
7.0230858e-05,
7.1419664e-05,
7.0065644e-05,
7.1247064e-05,
6.9889672e-05,
7.1062843e-05,
6.9702340e-05,
7.0869073e-05,
6.9506664e-05,
7.0665467e-05,
6.9301421e-05,
7.0452887e-05,
6.9085949e-05,
7.0230102e-05,
6.8860628e-05,
6.9996922e-05,
6.8625164e-05,
6.9755136e-05,
6.8380519e-05,
6.9502424e-05,
6.8124297e-05,
6.9238935e-05,
6.7858897e-05,
6.8966325e-05,
6.7584921e-05,
6.8685380e-05,
6.7301211e-05,
6.8394441e-05,
6.7005981e-05,
6.8091801e-05,
6.6701779e-05,
6.7780590e-05,
6.6387169e-05,
6.7459535e-05,
6.6063284e-05,
6.7128263e-05,
6.5728866e-05,
6.6787730e-05,
6.5386385e-05,
6.6438674e-05,
6.5034313e-05,
6.6080637e-05,
6.4672722e-05,
6.5711559e-05,
6.4300349e-05,
6.5332250e-05,
6.3919327e-05,
6.4945313e-05,
6.3529506e-05,
6.4548467e-05,
6.3129795e-05,
6.4142741e-05,
6.2721383e-05,
6.3728387e-05,
6.2304041e-05,
6.3304831e-05,
6.1876773e-05,
6.2870780e-05,
6.1440352e-05,
6.2427096e-05,
6.0994543e-05,
6.1975293e-05,
6.0540198e-05,
6.1514678e-05,
6.0078191e-05,
6.1045256e-05,
5.9605935e-05,
6.0567026e-05,
5.9125895e-05,
6.0080142e-05,
5.8636223e-05,
5.9583404e-05,
5.8137329e-05,
5.9077313e-05,
5.7629975e-05,
5.8563903e-05,
5.7114750e-05,
5.8042552e-05,
5.6590856e-05,
5.7511392e-05,
5.6058743e-05,
5.6972203e-05,
5.5517485e-05,
5.6423881e-05,
5.4967912e-05,
5.5867250e-05,
5.4409385e-05,
5.5301436e-05,
5.3842517e-05,
5.4728613e-05,
5.3268222e-05,
5.4148444e-05,
5.2686737e-05,
5.3559736e-05,
5.2096187e-05,
5.2962228e-05,
5.1496994e-05,
5.2356825e-05,
5.0890904e-05,
5.1743636e-05,
5.0276187e-05,
5.1122826e-05,
4.9653911e-05,
5.0494319e-05,
4.9024193e-05,
4.9859272e-05,
4.8387433e-05,
4.9215755e-05,
4.7742936e-05,
4.8564226e-05,
4.7089743e-05,
4.7905244e-05,
4.6428660e-05,
4.7238522e-05,
4.5761852e-05,
4.6565778e-05,
4.5087304e-05,
4.5885319e-05,
4.4405596e-05,
4.5198287e-05,
4.3717302e-05,
4.4504028e-05,
4.3021571e-05,
4.3802343e-05,
4.2318469e-05,
4.3093981e-05,
4.1608694e-05,
4.2378930e-05,
4.0892322e-05,
4.1657418e-05,
4.0170153e-05,
4.0930272e-05,
3.9441819e-05,
4.0195434e-05,
3.8706030e-05,
3.9454664e-05,
3.7964363e-05,
3.8706976e-05,
3.7215602e-05,
3.7953996e-05,
3.6461206e-05,
3.7194852e-05,
3.5701577e-05,
3.6429689e-05,
3.4936042e-05,
3.5659499e-05,
3.4164653e-05,
3.4883135e-05,
3.3388086e-05,
3.4100633e-05,
3.2605163e-05,
3.3312745e-05,
3.1817057e-05,
3.2519506e-05,
3.1023902e-05,
3.1721860e-05,
3.0226054e-05,
3.0918385e-05,
2.9422817e-05,
3.0109868e-05,
2.8614458e-05,
2.9296666e-05,
2.7800687e-05,
2.8477536e-05,
2.6981928e-05,
2.7654345e-05,
2.6159207e-05,
2.6826100e-05,
2.5331430e-05,
2.5993801e-05,
2.4500390e-05,
2.5156955e-05,
2.3664145e-05,
2.4315587e-05,
2.2823182e-05,
2.3470274e-05,
2.1978695e-05,
2.2620560e-05,
2.1129685e-05,
2.1766731e-05,
2.0276651e-05,
2.0910192e-05,
1.9421355e-05,
2.0049319e-05,
1.8561159e-05,
1.9184956e-05,
1.7697948e-05,
1.8316515e-05,
1.6830140e-05,
1.7445247e-05,
1.5959723e-05,
1.6570870e-05,
1.5086500e-05,
1.5692936e-05,
1.4209675e-05,
1.4812591e-05,
1.3330513e-05,
1.3929883e-05,
1.2448843e-05,
1.3044034e-05,
1.1563777e-05,
1.2155139e-05,
1.0675667e-05,
1.1263760e-05,
9.7859098e-06,
1.0371014e-05,
8.8947277e-06,
9.4759240e-06,
8.0004530e-06,
8.5782748e-06,
7.1040185e-06,
7.6791241e-06,
6.2064152e-06,
6.7778666e-06,
5.3057945e-06,
5.8746899e-06,
4.4042062e-06,
4.9693912e-06,
3.5003006e-06,
4.0632240e-06,
2.5970747e-06,
3.1564538e-06,
1.6918336e-06,
2.2478266e-06,
7.8430335e-07,
1.3377152e-06,
-1.2344475e-07,
4.2724169e-07,
-1.0318629e-06,
-4.8434608e-07,
-1.9418418e-06,
-1.3963640e-06,
-2.8511872e-06,
-2.3088621e-06,
-3.7612887e-06,
-3.2212961e-06,
-4.6708282e-06,
-4.1342168e-06,
-5.5820311e-06,
-5.0482045e-06,
-6.4932664e-06,
-5.9620409e-06,
-7.4035374e-06,
-6.8751606e-06,
-8.3140690e-06,
-7.7882939e-06,
-9.2238737e-06,
-8.7005231e-06,
-1.0133180e-05,
-9.6120038e-06,
-1.1041562e-05,
-1.0523061e-05,
-1.1950211e-05,
-1.1434233e-05,
-1.2857882e-05,
-1.2344256e-05,
-1.3763736e-05,
-1.3252300e-05,
-1.4668278e-05,
-1.4159349e-05,
-1.5571995e-05,
-1.5065183e-05,
-1.6473605e-05,
-1.5969235e-05,
-1.7374723e-05,
-1.6871930e-05,
-1.8273656e-05,
-1.7773405e-05,
-1.9172164e-05,
-1.8673505e-05,
-2.0067152e-05,
-1.9570064e-05,
-2.0959776e-05,
-2.0464130e-05,
-2.1850824e-05,
-2.1357079e-05,
-2.2739571e-05,
-2.2247134e-05,
-2.3625535e-05,
-2.3134655e-05,
-2.4509338e-05,
-2.4019646e-05,
-2.5390152e-05,
-2.4901710e-05,
-2.6268192e-05,
-2.5780248e-05,
-2.7142004e-05,
-2.6654700e-05,
-2.8012806e-05,
-2.7526810e-05,
-2.8881340e-05,
-2.8396374e-05,
-2.9746205e-05,
-2.9261614e-05,
-3.0607376e-05,
-3.0123406e-05,
-3.1464448e-05,
-3.0981142e-05,
-3.2318112e-05,
-3.1834762e-05,
-3.3166997e-05,
-3.2683605e-05,
-3.4012046e-05,
-3.3529074e-05,
-3.4852786e-05,
-3.4371010e-05,
-3.5689923e-05,
-3.5208074e-05,
-3.6522014e-05,
-3.6040173e-05,
-3.7349104e-05,
-3.6867502e-05,
-3.8171070e-05,
-3.7689779e-05,
-3.8988769e-05,
-3.8507477e-05,
-3.9800951e-05,
-3.9319849e-05,
-4.0608929e-05,
-4.0128168e-05,
-4.1411430e-05,
-4.0930828e-05,
-4.2208394e-05,
-4.1727328e-05,
-4.2999432e-05,
-4.2518591e-05,
-4.3784662e-05,
-4.3303883e-05,
-4.4564189e-05,
-4.4083665e-05,
-4.5338057e-05,
-4.4857890e-05,
-4.6106049e-05,
-4.5625924e-05,
-4.6868064e-05,
-4.6387520e-05,
-4.7623784e-05,
-4.7142311e-05,
-4.8372104e-05,
-4.7890580e-05,
-4.9114316e-05,
-4.8632777e-05,
-4.9850274e-05,
-4.9368244e-05,
-5.0579510e-05,
-5.0097346e-05,
-5.1301897e-05,
-5.0819355e-05,
-5.2017569e-05,
-5.1534392e-05,
-5.2725602e-05,
-5.2241409e-05,
-5.3426778e-05,
-5.2941540e-05,
-5.4119932e-05,
-5.3634440e-05,
-5.4806508e-05,
-5.4320067e-05,
-5.5485855e-05,
-5.4998142e-05,
-5.6157105e-05,
-5.5668267e-05,
-5.6820458e-05,
-5.6330097e-05,
-5.7475849e-05,
-5.6984376e-05,
-5.8123052e-05,
-5.7630379e-05,
-5.8762497e-05,
-5.8268516e-05,
-5.9393944e-05,
-5.8898392e-05,
-6.0017244e-05,
-5.9520083e-05,
-6.0632085e-05,
-6.0133149e-05,
-6.1238039e-05,
-6.0737134e-05,
-6.1835071e-05,
-6.1332437e-05,
-6.2423512e-05,
-6.1919461e-05,
-6.3003639e-05,
-6.2497861e-05,
-6.3574456e-05,
-6.3066456e-05,
-6.4136634e-05,
-6.3626244e-05,
-6.4689122e-05,
-6.4176930e-05,
-6.5232292e-05,
-6.4718055e-05,
-6.5766260e-05,
-6.5249757e-05,
-6.6290422e-05,
-6.5772445e-05,
-6.6805817e-05,
-6.6285915e-05,
-6.7311941e-05,
-6.6789758e-05,
-6.7807441e-05,
-6.7282937e-05,
-6.8293453e-05,
-6.7767102e-05,
-6.8769687e-05,
-6.8241179e-05,
-6.9235935e-05,
-6.8705339e-05,
-6.9692465e-05,
-6.9159572e-05,
-7.0138652e-05,
-6.9603949e-05,
-7.0575139e-05,
-7.0038052e-05,
-7.1000768e-05,
-7.0461392e-05,
-7.1415549e-05,
-7.0874507e-05,
-7.1820321e-05,
-7.1277585e-05,
-7.2215309e-05,
-7.1669871e-05,
-7.2599366e-05,
-7.2051886e-05,
-7.2972870e-05,
-7.2423011e-05,
-7.3335881e-05,
-7.2783488e-05,
-7.3687769e-05,
-7.3132834e-05,
-7.4028291e-05,
-7.3471128e-05,
-7.4358533e-05,
-7.3799518e-05,
-7.4678088e-05,
-7.4116286e-05,
-7.4986559e-05,
-7.4421977e-05,
-7.5283636e-05,
-7.4716538e-05,
-7.5569349e-05,
-7.4999633e-05,
-7.5844357e-05,
-7.5271532e-05,
-7.6107387e-05,
-7.5531764e-05,
-7.6359555e-05,
-7.5781323e-05,
-7.6601278e-05,
-7.6020102e-05,
-7.6831102e-05,
-7.6246347e-05,
-7.7048651e-05,
-7.6461396e-05,
-7.7255386e-05,
-7.6664745e-05,
-7.7450311e-05,
-7.6856862e-05,
-7.7634014e-05,
-7.7037177e-05,
-7.7805714e-05,
-7.7205782e-05,
-7.7966511e-05,
-7.7363529e-05,
-7.8115596e-05,
-7.7508911e-05,
-7.8251823e-05,
-7.7641910e-05,
-7.8376043e-05,
-7.7763460e-05,
-7.8489527e-05,
-7.7873351e-05,
-7.8591040e-05,
-7.7971843e-05,
-7.8680467e-05,
-7.8057865e-05,
-7.8758097e-05,
-7.8132456e-05,
-7.8824024e-05,
-7.8195122e-05,
-7.8877408e-05,
-7.8244872e-05,
-7.8918255e-05,
-7.8283149e-05,
-7.8947928e-05,
-7.8309758e-05,
-7.8965684e-05,
-7.8324685e-05,
-7.8971649e-05,
-7.8327240e-05,
-7.8964718e-05,
-7.8317208e-05,
-7.8946088e-05,
-7.8295710e-05,
-7.8915135e-05,
-7.8261375e-05,
-7.8871816e-05,
-7.8214896e-05,
-7.8816837e-05,
-7.8157381e-05,
-7.8750368e-05,
-7.8087734e-05,
-7.8670936e-05,
-7.8005923e-05,
-7.8579907e-05,
-7.7911312e-05,
-7.8475971e-05,
-7.7804757e-05,
-7.8360469e-05,
-7.7686298e-05,
-7.8232660e-05,
-7.7555314e-05,
-7.8092906e-05,
-7.7412820e-05,
-7.7941783e-05,
-7.7258528e-05,
-7.7777999e-05,
-7.7091493e-05,
-7.7601535e-05,
-7.6912064e-05,
-7.7413110e-05,
-7.6720470e-05,
-7.7212667e-05,
-7.6517373e-05,
-7.7000462e-05,
-7.6301863e-05,
-7.6776145e-05,
-7.6074713e-05,
-7.6540000e-05,
-7.5835735e-05,
-7.6292206e-05,
-7.5584153e-05,
-7.6031488e-05,
-7.5320174e-05,
-7.5759178e-05,
-7.5044453e-05,
-7.5474624e-05,
-7.4757333e-05,
-7.5179163e-05,
-7.4458590e-05,
-7.4871373e-05,
-7.4147360e-05,
-7.4551724e-05,
-7.3824797e-05,
-7.4220310e-05,
-7.3490145e-05,
-7.3876946e-05,
-7.3143158e-05,
-7.3521626e-05,
-7.2784857e-05,
-7.3155167e-05,
-7.2415321e-05,
-7.2776867e-05,
-7.2034487e-05,
-7.2387815e-05,
-7.1641932e-05,
-7.1986288e-05,
-7.1237429e-05,
-7.1573397e-05,
-7.0821528e-05,
-7.1149193e-05,
-7.0394356e-05,
-7.0713280e-05,
-6.9955737e-05,
-7.0266560e-05,
-6.9506319e-05,
-6.9808449e-05,
-6.9045530e-05,
-6.9339235e-05,
-6.8573537e-05,
-6.8858581e-05,
-6.8090089e-05,
-6.8366344e-05,
-6.7595541e-05,
-6.7863175e-05,
-6.7089849e-05,
-6.7349360e-05,
-6.6573835e-05,
-6.6824495e-05,
-6.6046887e-05,
-6.6289203e-05,
-6.5508955e-05,
-6.5742668e-05,
-6.4959981e-05,
-6.5185703e-05,
-6.4400564e-05,
-6.4617397e-05,
-6.3830265e-05,
-6.4038993e-05,
-6.3249837e-05,
-6.3449993e-05,
-6.2658964e-05,
-6.2850921e-05,
-6.2057887e-05,
-6.2241433e-05,
-6.1446329e-05,
-6.1621416e-05,
-6.0823995e-05,
-6.0991163e-05,
-6.0191700e-05,
-6.0351015e-05,
-5.9549584e-05,
-5.9700645e-05,
-5.8897277e-05,
-5.9040927e-05,
-5.8235527e-05,
-5.8371181e-05,
-5.7564006e-05,
-5.7691646e-05,
-5.6882238e-05,
-5.7002379e-05,
-5.6191112e-05,
-5.6303190e-05,
-5.5490312e-05,
-5.5595046e-05,
-5.4780466e-05,
-5.4877576e-05,
-5.4061022e-05,
-5.4150916e-05,
-5.3332434e-05,
-5.3415031e-05,
-5.2594734e-05,
-5.2670106e-05,
-5.1847863e-05,
-5.1915646e-05,
-5.1091537e-05,
-5.1152543e-05,
-5.0327081e-05,
-5.0380883e-05,
-4.9554133e-05,
-4.9600802e-05,
-4.8772423e-05,
-4.8812013e-05,
-4.7981769e-05,
-4.8014768e-05,
-4.7183293e-05,
-4.7209088e-05,
-4.6376280e-05,
-4.6395418e-05,
-4.5561040e-05,
-4.5573158e-05,
-4.4737935e-05,
-4.4743588e-05,
-4.3907351e-05,
-4.3906529e-05,
-4.3069066e-05,
-4.3061094e-05,
-4.2222462e-05,
-4.2207808e-05,
-4.1368269e-05,
-4.1347239e-05,
-4.0507195e-05,
-4.0479547e-05,
-3.9638588e-05,
-3.9604391e-05,
-3.8762757e-05,
-3.8722194e-05,
-3.7880461e-05,
-3.7833414e-05,
-3.6990984e-05,
-3.6937586e-05,
-3.6094570e-05,
-3.6034320e-05,
-3.5191183e-05,
-3.5124522e-05,
-3.4281716e-05,
-3.4209031e-05,
-3.3365975e-05,
-3.3287267e-05,
-3.2443855e-05,
-3.2358693e-05,
-3.1515418e-05,
-3.1424587e-05,
-3.0581406e-05,
-3.0484176e-05,
-2.9640904e-05,
-2.9537543e-05,
-2.8694594e-05,
-2.8585473e-05,
-2.7743058e-05,
-2.7628084e-05,
-2.6786490e-05,
-2.6665897e-05,
-2.5824310e-05,
-2.5697959e-05,
-2.4856780e-05,
-2.4724800e-05,
-2.3884174e-05,
-2.3747136e-05,
-2.2906775e-05,
-2.2764118e-05,
-2.1924371e-05,
-2.1776609e-05,
-2.0937651e-05,
-2.0785363e-05,
-1.9946991e-05,
-1.9789636e-05,
-1.8951989e-05,
-1.8789318e-05,
-1.7952324e-05,
-1.7784527e-05,
-1.6948547e-05,
-1.6776395e-05,
-1.5941243e-05,
-1.5764364e-05,
-1.4929925e-05,
-1.4748626e-05,
-1.3915377e-05,
-1.3729947e-05,
-1.2897736e-05,
-1.2708100e-05,
-1.1876788e-05,
-1.1682816e-05,
-1.0852379e-05,
-1.0653684e-05,
-9.8247614e-06,
-9.6221959e-06,
-8.7948516e-06,
-8.5885872e-06,
-7.7626473e-06,
-7.5519848e-06,
-6.7274410e-06,
-6.5132407e-06,
-5.6905222e-06,
-5.4724334e-06,
-4.6513034e-06,
-4.4289845e-06,
-3.6096304e-06,
-3.3836681e-06,
-2.5658958e-06,
-2.3361468e-06,
-1.5207505e-06,
-1.2879197e-06,
-4.7485926e-07,
-2.3818570e-07,
5.7273108e-07,
8.1315100e-07,
1.6218618e-06,
1.8654739e-06,
2.6716836e-06,
2.9189341e-06,
3.7227602e-06,
3.9734809e-06,
4.7745841e-06,
5.0283863e-06,
5.8268660e-06,
6.0836124e-06,
6.8789663e-06,
7.1388620e-06,
7.9312261e-06,
8.1947379e-06,
8.9842765e-06,
9.2507374e-06,
1.0037144e-05,
1.0306579e-05,
1.1089969e-05,
1.1362230e-05,
1.2142381e-05,
1.2417052e-05,
1.3193882e-05,
1.3471340e-05,
1.4244712e-05,
1.4524430e-05,
1.5294340e-05,
1.5577025e-05,
1.6343628e-05,
1.6628730e-05,
1.7391735e-05,
1.7679095e-05,
1.8438129e-05,
1.8727848e-05,
1.9483390e-05,
1.9774939e-05,
2.0526425e-05,
2.0819805e-05,
2.1567663e-05,
2.1863241e-05,
2.2607207e-05,
2.2904239e-05,
2.3644645e-05,
2.3943587e-05,
2.4680255e-05,
2.4980370e-05,
2.5712715e-05,
2.6014363e-05,
2.6742416e-05,
2.7045521e-05,
2.7769518e-05,
2.8073737e-05,
2.8793289e-05,
2.9098908e-05,
2.9814439e-05,
3.0121122e-05,
3.0832263e-05,
3.1139773e-05,
3.1846820e-05,
3.2155050e-05,
3.2857495e-05,
3.3166377e-05,
3.3864080e-05,
3.4173612e-05,
3.4866860e-05,
3.5177190e-05,
3.5865616e-05,
3.6176593e-05,
3.6860444e-05,
3.7172105e-05,
3.7851003e-05,
3.8162812e-05,
3.8836537e-05,
3.9149058e-05,
3.9817766e-05,
4.0130355e-05,
4.0793668e-05,
4.1106709e-05,
4.1764894e-05,
4.2077955e-05,
4.2730945e-05,
4.3044457e-05,
4.3691723e-05,
4.4005478e-05,
4.4647497e-05,
4.4961153e-05,
4.5597014e-05,
4.5910493e-05,
4.6540568e-05,
4.6854529e-05,
4.7478786e-05,
4.7792426e-05,
4.8410805e-05,
4.8724333e-05,
4.9336813e-05,
4.9650120e-05,
5.0256448e-05,
5.0569595e-05,
5.1169596e-05,
5.1482568e-05,
5.2076191e-05,
5.2388504e-05,
5.2975471e-05,
5.3287400e-05,
5.3868339e-05,
5.4179944e-05,
5.4754389e-05,
5.5065267e-05,
5.5633309e-05,
5.5943177e-05,
5.6504533e-05,
5.6813784e-05,
5.7368600e-05,
5.7676631e-05,
5.8224904e-05,
5.8532135e-05,
5.9073281e-05,
5.9379375e-05,
5.9914028e-05,
6.0219221e-05,
6.0747373e-05,
6.1051189e-05,
6.1572181e-05,
6.1874819e-05,
6.2388841e-05,
6.2689766e-05,
6.3196882e-05,
6.3496427e-05,
6.3996879e-05,
6.4294440e-05,
6.4788086e-05,
6.5083972e-05,
6.5570395e-05,
6.5864643e-05,
6.6344349e-05,
6.6636907e-05,
6.7109253e-05,
6.7399518e-05,
6.7864386e-05,
6.8152877e-05,
6.8610447e-05,
6.8896950e-05,
6.9347511e-05,
6.9631854e-05,
7.0075066e-05,
7.0357266e-05,
7.0793173e-05,
7.1072870e-05,
7.1501526e-05,
7.1779169e-05,
7.2200256e-05,
7.2475480e-05,
7.2888625e-05,
7.3161130e-05,
7.3566841e-05,
7.3837086e-05,
7.4235227e-05,
7.4503432e-05,
7.4893898e-05,
7.5159381e-05,
7.5541584e-05,
7.5804786e-05,
7.6179165e-05,
7.6439503e-05,
7.6806153e-05,
7.7064047e-05,
7.7422354e-05,
7.7677439e-05,
7.8027536e-05,
7.8280068e-05,
7.8622395e-05,
7.8872418e-05,
7.9206606e-05,
7.9454233e-05,
7.9779805e-05,
8.0024359e-05,
8.0341359e-05,
8.0583239e-05,
8.0891964e-05,
8.1131109e-05,
8.1431478e-05,
8.1667678e-05,
8.1959482e-05,
8.2192754e-05,
8.2476033e-05,
8.2706846e-05,
8.2981623e-05,
8.3209490e-05,
8.3475544e-05,
8.3700225e-05,
8.3957218e-05,
8.4178747e-05,
8.4427293e-05,
8.4645738e-05,
8.4885567e-05,
8.5101210e-05,
8.5332419e-05,
8.5544542e-05,
8.5767214e-05,
8.5976106e-05,
8.6190074e-05,
8.6395872e-05,
8.6600998e-05,
8.6803182e-05,
8.6999453e-05,
8.7197879e-05,
8.7385330e-05,
8.7580695e-05,
8.7759658e-05,
8.7951556e-05,
8.8121568e-05,
8.8310039e-05,
8.8471413e-05,
8.8656056e-05,
8.8808718e-05,
8.8989565e-05,
8.9133604e-05,
8.9310530e-05,
8.9445740e-05,
8.9618722e-05,
8.9745162e-05,
8.9914348e-05,
9.0032298e-05,
9.0198133e-05,
9.0307262e-05,
9.0469168e-05,
9.0569342e-05,
9.0727262e-05,
9.0818560e-05,
9.0972743e-05,
9.1055068e-05,
9.1205195e-05,
9.1278874e-05,
9.1424956e-05,
9.1489654e-05,
9.1631934e-05,
9.1687826e-05,
9.1826393e-05,
9.1873674e-05,
9.2008241e-05,
9.2046317e-05,
9.2176773e-05,
9.2205838e-05,
9.2332351e-05,
9.2352143e-05,
9.2475006e-05,
9.2485801e-05,
9.2604848e-05,
9.2606667e-05,
9.2721872e-05,
9.2714492e-05,
9.2825979e-05,
9.2809644e-05,
9.2917370e-05,
9.2891942e-05,
9.2995453e-05,
9.2960675e-05,
9.3060401e-05,
9.3016535e-05,
9.3112584e-05,
9.3059375e-05,
9.3151687e-05,
9.3089587e-05,
9.3178380e-05,
9.3106974e-05,
9.3191747e-05,
9.3111191e-05,
9.3192142e-05,
9.3102473e-05,
9.3179621e-05,
9.3080601e-05,
9.3153833e-05,
9.3045688e-05,
9.3115331e-05,
9.2998166e-05,
9.3064013e-05,
9.2937765e-05,
9.2999877e-05,
9.2864648e-05,
9.2923155e-05,
9.2778627e-05,
9.2832924e-05,
9.2679515e-05,
9.2730006e-05,
9.2567583e-05,
9.2614255e-05,
9.2442880e-05,
9.2485781e-05,
9.2305719e-05,
9.2344741e-05,
9.2155831e-05,
9.2190780e-05,
9.1993166e-05,
9.2024466e-05,
9.1817812e-05,
9.1845124e-05,
9.1629538e-05,
9.1653012e-05,
9.1428851e-05,
9.1448533e-05,
9.1215753e-05,
9.1231561e-05,
9.0990367e-05,
9.1002303e-05,
9.0752653e-05,
9.0760538e-05,
9.0502237e-05,
9.0506197e-05,
9.0239646e-05,
9.0239886e-05,
8.9964639e-05,
8.9960818e-05,
8.9677143e-05,
8.9670135e-05,
8.9378119e-05,
8.9367259e-05,
8.9066897e-05,
8.9052318e-05,
8.8743575e-05,
8.8725313e-05,
8.8408141e-05,
8.8386214e-05,
8.8060652e-05,
8.8035331e-05,
8.7701743e-05,
8.7672632e-05,
8.7330762e-05,
8.7298161e-05,
8.6948200e-05,
8.6912675e-05,
8.6554402e-05,
8.6515289e-05,
8.6148670e-05,
8.6106275e-05,
8.5731262e-05,
8.5685758e-05,
8.5302629e-05,
8.5254056e-05,
8.4862915e-05,
8.4811421e-05,
8.4412087e-05,
8.4357174e-05,
8.3949850e-05,
8.3892210e-05,
8.3477036e-05,
8.3416331e-05,
8.2992868e-05,
8.2929276e-05,
8.2497703e-05,
8.2431207e-05,
8.1991803e-05,
8.1922715e-05,
8.1475710e-05,
8.1403899e-05,
8.0949244e-05,
8.0874644e-05,
8.0411816e-05,
8.0334466e-05,
7.9864203e-05,
7.9784444e-05,
7.9306747e-05,
7.9223943e-05,
7.8738461e-05,
7.8653412e-05,
7.8160547e-05,
7.8072910e-05,
7.7572964e-05,
7.7483000e-05,
7.6975755e-05,
7.6883253e-05,
7.6368747e-05,
7.6273469e-05,
7.5751588e-05,
7.5654111e-05,
7.5125559e-05,
7.5025649e-05,
7.4490164e-05,
7.4387654e-05,
7.3845362e-05,
7.3740773e-05,
7.3191806e-05,
7.3085103e-05,
7.2529461e-05,
7.2420510e-05,
7.1857915e-05,
7.1746792e-05,
7.1177554e-05,
7.1064108e-05,
7.0488666e-05,
7.0373481e-05,
6.9791792e-05,
6.9674437e-05,
6.9086299e-05,
6.8967032e-05,
6.8373011e-05,
6.8251703e-05,
6.7651431e-05,
6.7528553e-05,
6.6921961e-05,
6.6797030e-05,
6.6184263e-05,
6.6057828e-05,
6.5439094e-05,
6.5311283e-05,
6.4686996e-05,
6.4557865e-05,
6.3927404e-05,
6.3796750e-05,
6.3160394e-05,
6.3028544e-05,
6.2386743e-05,
6.2253375e-05,
6.1605781e-05,
6.1471315e-05,
6.0817996e-05,
6.0682283e-05,
6.0023642e-05,
5.9887195e-05,
5.9223082e-05,
5.9086115e-05,
5.8416594e-05,
5.8278639e-05,
5.7603329e-05,
5.7464610e-05,
5.6784050e-05,
5.6644867e-05,
5.5959042e-05,
5.5819272e-05,
5.5128359e-05,
5.4987951e-05,
5.4291919e-05,
5.4151447e-05,
5.3450610e-05,
5.3309876e-05,
5.2604062e-05,
5.2463208e-05,
5.1752341e-05,
5.1610991e-05,
5.0895286e-05,
5.0753945e-05,
5.0033704e-05,
4.9892560e-05,
4.9167627e-05,
4.9026657e-05,
4.8297476e-05,
4.8156380e-05,
4.7422794e-05,
4.7282283e-05,
4.6544415e-05,
4.6403901e-05,
4.5661837e-05,
4.5521521e-05,
4.4775229e-05,
4.4635229e-05,
4.3885161e-05,
4.3745800e-05,
4.2991959e-05,
4.2853185e-05,
4.2095617e-05,
4.1957420e-05,
4.1196306e-05,
4.1058549e-05,
4.0294029e-05,
4.0156859e-05,
3.9388858e-05,
3.9252304e-05,
3.8481073e-05,
3.8345320e-05,
3.7570846e-05,
3.7436213e-05,
3.6658792e-05,
3.6525222e-05,
3.5744781e-05,
3.5612161e-05,
3.4828771e-05,
3.4697254e-05,
3.3910854e-05,
3.3780543e-05,
3.2991649e-05,
3.2862224e-05,
3.2070646e-05,
3.1942826e-05,
3.1148849e-05,
3.1022495e-05,
3.0226059e-05,
3.0101362e-05,
2.9302586e-05,
2.9179473e-05,
2.8378311e-05,
2.8256842e-05,
2.7453252e-05,
2.7333762e-05,
2.6527842e-05,
2.6410524e-05,
2.5602613e-05,
2.5487205e-05,
2.4677292e-05,
2.4564253e-05,
2.3752525e-05,
2.3641582e-05,
2.2827961e-05,
2.2719305e-05,
2.1903876e-05,
2.1797623e-05,
2.0980384e-05,
2.0876615e-05,
2.0057661e-05,
1.9956683e-05,
1.9136255e-05,
1.9038242e-05,
1.8216165e-05,
1.8121099e-05,
1.7297702e-05,
1.7205380e-05,
1.6380601e-05,
1.6291174e-05,
1.5465193e-05,
1.5378933e-05,
1.4551782e-05,
1.4468705e-05,
1.3640453e-05,
1.3560565e-05,
1.2731568e-05,
1.2655029e-05,
1.1825307e-05,
1.1752214e-05,
1.0921497e-05,
1.0851914e-05,
1.0020584e-05,
9.9542338e-06,
9.1221595e-06,
9.0595272e-06,
8.2273009e-06,
8.1684522e-06,
7.3358337e-06,
7.2805701e-06,
6.4476784e-06,
6.3962303e-06,
5.5634352e-06,
5.5157969e-06,
4.6829192e-06,
4.6391147e-06,
3.8062466e-06,
3.7663868e-06,
2.9336377e-06,
2.8978034e-06,
2.0653663e-06,
2.0338020e-06,
1.2021807e-06,
1.1746251e-06,
3.4338394e-07,
3.1978225e-07,
-5.1086231e-07,
-5.3000719e-07,
-1.3599206e-06,
-1.3748004e-06,
-2.2039974e-06,
-2.2145486e-06,
-3.0428167e-06,
-3.0488014e-06,
-3.8762299e-06,
-3.8774564e-06,
-4.7036259e-06,
-4.7001498e-06,
-5.5252474e-06,
-5.5173712e-06,
-6.3413846e-06,
-6.3287614e-06,
-7.1513565e-06,
-7.1337175e-06,
-7.9550247e-06,
-7.9325356e-06,
-8.7523051e-06,
-8.7248148e-06,
-9.5430822e-06,
-9.5103145e-06,
-1.0327099e-05,
-1.0289014e-05,
-1.1104147e-05,
-1.1060760e-05,
-1.1874805e-05,
-1.1826158e-05,
-1.2638339e-05,
-1.2583937e-05,
-1.3394127e-05,
-1.3334273e-05,
-1.4142755e-05,
-1.4077263e-05,
-1.4883725e-05,
-1.4812543e-05,
-1.5617044e-05,
-1.5540041e-05,
-1.6342515e-05,
-1.6259656e-05,
-1.7060222e-05,
-1.6971483e-05,
-1.7770234e-05,
-1.7675281e-05,
-1.8471419e-05,
-1.8370083e-05,
-1.9163930e-05,
-1.9056738e-05,
-1.9848421e-05,
-1.9734904e-05,
-2.0524210e-05,
-2.0404486e-05,
-2.1191160e-05,
-2.1065268e-05,
-2.1849372e-05,
-2.1717125e-05,
-2.2498712e-05,
-2.2360224e-05,
-2.3138815e-05,
-2.2993522e-05,
-2.3769143e-05,
-2.3617431e-05,
-2.4390416e-05,
-2.4232435e-05,
-2.5002465e-05,
-2.4837901e-05,
-2.5604937e-05,
-2.5433752e-05,
-2.6197423e-05,
-2.6019754e-05,
-2.6780208e-05,
-2.6595859e-05,
-2.7353026e-05,
-2.7162048e-05,
-2.7915501e-05,
-2.7717544e-05,
-2.8467875e-05,
-2.8263535e-05,
-2.9010552e-05,
-2.8799418e-05,
-2.9542611e-05,
-2.9324784e-05,
-3.0064300e-05,
-2.9839435e-05,
-3.0575177e-05,
-3.0343502e-05,
-3.1075387e-05,
-3.0836899e-05,
-3.1564982e-05,
-3.1319388e-05,
-3.2043569e-05,
-3.1791053e-05,
-3.2511683e-05,
-3.2252298e-05,
-3.2968905e-05,
-3.2702111e-05,
-3.3414526e-05,
-3.3140668e-05,
-3.3849006e-05,
-3.3567819e-05,
-3.4272268e-05,
-3.3983881e-05,
-3.4684088e-05,
-3.4388678e-05,
-3.5084782e-05,
-3.4781915e-05,
-3.5473937e-05,
-3.5164031e-05,
-3.5851909e-05,
-3.5534314e-05,
-3.6217866e-05,
-3.5892535e-05,
-3.6571995e-05,
-3.6239132e-05,
-3.6914352e-05,
-3.6574083e-05,
-3.7245275e-05,
-3.6897305e-05,
-3.7564200e-05,
-3.7208716e-05,
-3.7871201e-05,
-3.7508064e-05,
-3.8166214e-05,
-3.7795553e-05,
-3.8449164e-05,
-3.8070417e-05,
-3.8719625e-05,
-3.8333222e-05,
-3.8978279e-05,
-3.8584065e-05,
-3.9224701e-05,
-3.8822978e-05,
-3.9459204e-05,
-3.9049441e-05,
-3.9681312e-05,
-3.9263585e-05,
-3.9890737e-05,
-3.9465314e-05,
-4.0087978e-05,
-3.9654669e-05,
-4.0272576e-05,
-3.9831483e-05,
-4.0444790e-05,
-3.9996119e-05,
-4.0604863e-05,
-4.0148326e-05,
-4.0752418e-05,
-4.0288144e-05,
-4.0887310e-05,
-4.0415168e-05,
-4.1009637e-05,
-4.0529505e-05,
-4.1119071e-05,
-4.0631366e-05,
-4.1216265e-05,
-4.0720795e-05,
-4.1300712e-05,
-4.0797798e-05,
-4.1372830e-05,
-4.0862185e-05,
-4.1432261e-05,
-4.0913890e-05,
-4.1479155e-05,
-4.0952926e-05,
-4.1513033e-05,
-4.0979341e-05,
-4.1534432e-05,
-4.0992991e-05,
-4.1543245e-05,
-4.0994328e-05,
-4.1539656e-05,
-4.0983167e-05,
-4.1523422e-05,
-4.0959365e-05,
-4.1494561e-05,
-4.0922752e-05,
-4.1452891e-05,
-4.0873563e-05,
-4.1398838e-05,
-4.0811720e-05,
-4.1332065e-05,
-4.0737496e-05,
-4.1252868e-05,
-4.0650700e-05,
-4.1161428e-05,
-4.0551666e-05,
-4.1057373e-05,
-4.0439944e-05,
-4.0940794e-05,
-4.0315653e-05,
-4.0811542e-05,
-4.0178962e-05,
-4.0669996e-05,
-4.0029970e-05,
-4.0516195e-05,
-3.9868559e-05,
-4.0350211e-05,
-3.9694886e-05,
-4.0171788e-05,
-3.9509005e-05,
-3.9981353e-05,
-3.9310839e-05,
-3.9778349e-05,
-3.9100135e-05,
-3.9563144e-05,
-3.8877484e-05,
-3.9335885e-05,
-3.8642989e-05,
-3.9096797e-05,
-3.8396419e-05,
-3.8845528e-05,
-3.8137828e-05,
-3.8582490e-05,
-3.7867257e-05,
-3.8307356e-05,
-3.7584961e-05,
-3.8020505e-05,
-3.7290586e-05,
-3.7721615e-05,
-3.6984442e-05,
-3.7411215e-05,
-3.6667024e-05,
-3.7089514e-05,
-3.6338063e-05,
-3.6756010e-05,
-3.5997339e-05,
-3.6410745e-05,
-3.5645164e-05,
-3.6054151e-05,
-3.5281635e-05,
-3.5686280e-05,
-3.4906910e-05,
-3.5307041e-05,
-3.4520722e-05,
-3.4916627e-05,
-3.4123745e-05,
-3.4515457e-05,
-3.3715849e-05,
-3.4103311e-05,
-3.3296742e-05,
-3.3679715e-05,
-3.2866628e-05,
-3.3245377e-05,
-3.2425887e-05,
-3.2800449e-05,
-3.1974649e-05,
-3.2345155e-05,
-3.1512809e-05,
-3.1878977e-05,
-3.1040378e-05,
-3.1402628e-05,
-3.0557939e-05,
-3.0916074e-05,
-3.0064939e-05,
-3.0418884e-05,
-2.9561533e-05,
-2.9911531e-05,
-2.9048284e-05,
-2.9394499e-05,
-2.8525329e-05,
-2.8867714e-05,
-2.7992642e-05,
-2.8331230e-05,
-2.7450055e-05,
-2.7784758e-05,
-2.6897898e-05,
-2.7229119e-05,
-2.6336398e-05,
-2.6663878e-05,
-2.5765207e-05,
-2.6089109e-05,
-2.5184901e-05,
-2.5505741e-05,
-2.4595877e-05,
-2.4913367e-05,
-2.3997956e-05,
-2.4312016e-05,
-2.3391017e-05,
-2.3701688e-05,
-2.2775157e-05,
-2.3082737e-05,
-2.2150948e-05,
-2.2455485e-05,
-2.1518173e-05,
-2.1819623e-05,
-2.0877074e-05,
-2.1175830e-05,
-2.0228167e-05,
-2.0524350e-05,
-1.9571302e-05,
-1.9864625e-05,
-1.8906396e-05,
-1.9196884e-05,
-1.8233599e-05,
-1.8521404e-05,
-1.7553293e-05,
-1.7838830e-05,
-1.6865959e-05,
-1.7148794e-05,
-1.6171081e-05,
-1.6451526e-05,
-1.5469387e-05,
-1.5747650e-05,
-1.4760900e-05,
-1.5036633e-05,
-1.4045229e-05,
-1.4318769e-05,
-1.3322774e-05,
-1.3594192e-05,
-1.2594008e-05,
-1.2863418e-05,
-1.1859242e-05,
-1.2126809e-05,
-1.1118488e-05,
-1.1383762e-05,
-1.0371467e-05,
-1.0634946e-05,
-9.6189655e-06,
-9.8804877e-06,
-8.8606513e-06,
-9.1200343e-06,
-8.0965934e-06,
-8.3544480e-06,
-7.3273516e-06,
-7.5837002e-06,
-6.5533203e-06,
-6.8081235e-06,
-5.7745591e-06,
-6.0277186e-06,
-4.9906817e-06,
-5.2422643e-06,
-4.2021429e-06,
-4.4524472e-06,
-3.4092810e-06,
-3.6581253e-06,
-2.6119809e-06,
-2.8596526e-06,
-1.8108372e-06,
-2.0574114e-06,
-1.0059193e-06,
-1.2515857e-06,
-1.9739359e-07,
-4.4209678e-07,
6.1464187e-07,
3.7106823e-07,
1.4304655e-06,
1.1875144e-06,
2.2491176e-06,
2.0068671e-06,
3.0708404e-06,
2.8291581e-06,
3.8952262e-06,
3.6537694e-06,
4.7218851e-06,
4.4810510e-06,
5.5511215e-06,
5.3104436e-06,
6.3827288e-06,
6.1421318e-06,
7.2163979e-06,
6.9759926e-06,
8.0519737e-06,
7.8112861e-06,
8.8887874e-06,
8.6480575e-06,
9.7272055e-06,
9.4861304e-06,
1.0566620e-05,
1.0325172e-05,
1.1407191e-05,
1.1165474e-05,
1.2248778e-05,
1.2006483e-05,
1.3091098e-05,
1.2847997e-05,
1.3933889e-05,
1.3689959e-05,
1.4776657e-05,
1.4531726e-05,
1.5619430e-05,
1.5373610e-05,
1.6462154e-05,
1.6215285e-05,
1.7304519e-05,
1.7056709e-05,
1.8146639e-05,
1.7897500e-05,
1.8987809e-05,
1.8737557e-05,
1.9828003e-05,
1.9576330e-05,
2.0667157e-05,
2.0413863e-05,
2.1504667e-05,
2.1249845e-05,
2.2340895e-05,
2.2084450e-05,
2.3175351e-05,
2.2917259e-05,
2.4008031e-05,
2.3748204e-05,
2.4838458e-05,
2.4576555e-05,
2.5666241e-05,
2.5402635e-05,
2.6491557e-05,
2.6226048e-05,
2.7314284e-05,
2.7046534e-05,
2.8133915e-05,
2.7864140e-05,
2.8950655e-05,
2.8678627e-05,
2.9764003e-05,
2.9489651e-05,
3.0573874e-05,
3.0296940e-05,
3.1379612e-05,
3.1100158e-05,
3.2181552e-05,
3.1899545e-05,
3.2979531e-05,
3.2694988e-05,
3.3773165e-05,
3.3485673e-05,
3.4562222e-05,
3.4271956e-05,
3.5346633e-05,
3.5053324e-05,
3.6126108e-05,
3.5829636e-05,
3.6900317e-05,
3.6600519e-05,
3.7669125e-05,
3.7366222e-05,
3.8432838e-05,
3.8126446e-05,
3.9190861e-05,
3.8881036e-05,
3.9942759e-05,
3.9629215e-05,
4.0688662e-05,
4.0371401e-05,
4.1428308e-05,
4.1107117e-05,
4.2161488e-05,
4.1836283e-05,
4.2887861e-05,
4.2558645e-05,
4.3607702e-05,
4.3274454e-05,
4.4320576e-05,
4.3982995e-05,
4.5026018e-05,
4.4684007e-05,
4.5723813e-05,
4.5377521e-05,
4.6414297e-05,
4.6063467e-05,
4.7097009e-05,
4.6741548e-05,
4.7771763e-05,
4.7411585e-05,
4.8438421e-05,
4.8073566e-05,
4.9097012e-05,
4.8727174e-05,
4.9746796e-05,
4.9371935e-05,
5.0387762e-05,
5.0007756e-05,
5.1019899e-05,
5.0635220e-05,
5.1643339e-05,
5.1253465e-05,
5.2257419e-05,
5.1862385e-05,
5.2862126e-05,
5.2461827e-05,
5.3457434e-05,
5.3051824e-05,
5.4042893e-05,
5.3631697e-05,
5.4618277e-05,
5.4201580e-05,
5.5183715e-05,
5.4761517e-05,
5.5739229e-05,
5.5311707e-05,
5.6284398e-05,
5.5851064e-05,
5.6818715e-05,
5.6379725e-05,
5.7342412e-05,
5.6897712e-05,
5.7855373e-05,
5.7404782e-05,
5.8357061e-05,
5.7900578e-05,
5.8847725e-05,
5.8385367e-05,
5.9327271e-05,
5.8859010e-05,
5.9795608e-05,
5.9321037e-05,
6.0251893e-05,
5.9771041e-05,
6.0696285e-05,
6.0209276e-05,
6.1128953e-05,
6.0635752e-05,
6.1549657e-05,
6.1049980e-05,
6.1958148e-05,
6.1452131e-05,
6.2354478e-05,
6.1842028e-05,
6.2738450e-05,
6.2219404e-05,
6.3109831e-05,
6.2583680e-05,
6.3467995e-05,
6.2935279e-05,
6.3813640e-05,
6.3274209e-05,
6.4146440e-05,
6.3600181e-05,
6.4466156e-05,
6.3912881e-05,
6.4772800e-05,
6.4212458e-05,
6.5065990e-05,
6.4498499e-05,
6.5345682e-05,
6.4770937e-05,
6.5611644e-05,
6.5029487e-05,
6.5863902e-05,
6.5274753e-05,
6.6102822e-05,
6.5506329e-05,
6.6327709e-05,
6.5723781e-05,
6.6538502e-05,
6.5927235e-05,
6.6735269e-05,
6.6116381e-05,
6.6917649e-05,
6.6291186e-05,
6.7085698e-05,
6.6451705e-05,
6.7239292e-05,
6.6597812e-05,
6.7378819e-05,
6.6729718e-05,
6.7503749e-05,
6.6846850e-05,
6.7613713e-05,
6.6949076e-05,
6.7708826e-05,
6.7036672e-05,
6.7789072e-05,
6.7109201e-05,
6.7854489e-05,
6.7166959e-05,
6.7904956e-05,
6.7209776e-05,
6.7940401e-05,
6.7237628e-05,
6.7961079e-05,
6.7250277e-05,
6.7966104e-05,
6.7247420e-05,
6.7955796e-05,
6.7229334e-05,
6.7930206e-05,
6.7195972e-05,
6.7889209e-05,
6.7147340e-05,
6.7833033e-05,
6.7083350e-05,
6.7761301e-05,
6.7003704e-05,
6.7673951e-05,
6.6908643e-05,
6.7571060e-05,
6.6797676e-05,
6.7452226e-05,
6.6670977e-05,
6.7317931e-05,
6.6528826e-05,
6.7168019e-05,
6.6370880e-05,
6.7002412e-05,
6.6197515e-05,
6.6821104e-05,
6.6008104e-05,
6.6623793e-05,
6.5802674e-05,
6.6410589e-05,
6.5581633e-05,
6.6181490e-05,
6.5344483e-05,
6.5936640e-05,
6.5091625e-05,
6.5676028e-05,
6.4822952e-05,
6.5399496e-05,
6.4538317e-05,
6.5107130e-05,
6.4237854e-05,
6.4798601e-05,
6.3921137e-05,
6.4474041e-05,
6.3588526e-05,
6.4133768e-05,
6.3240195e-05,
6.3777522e-05,
6.2875894e-05,
6.3405593e-05,
6.2495809e-05,
6.3017621e-05,
6.2099645e-05,
6.2613712e-05,
6.1687726e-05,
6.2193963e-05,
6.1259668e-05,
6.1758055e-05,
6.0815820e-05,
6.1306458e-05,
6.0356369e-05,
6.0839335e-05,
5.9881043e-05,
6.0356159e-05,
5.9390001e-05,
5.9857273e-05,
5.8883086e-05,
5.9342506e-05,
5.8360305e-05,
5.8812119e-05,
5.7822041e-05,
5.8265999e-05,
5.7267866e-05,
5.7704226e-05,
5.6698479e-05,
5.7127121e-05,
5.6113686e-05,
5.6534448e-05,
5.5513045e-05,
5.5925991e-05,
5.4897107e-05,
5.5302148e-05,
5.4265677e-05,
5.4663032e-05,
5.3619040e-05,
5.4008726e-05,
5.2957090e-05,
5.3339029e-05,
5.2279974e-05,
5.2654397e-05,
5.1587929e-05,
5.1954686e-05,
5.0880738e-05,
5.1239577e-05,
5.0158228e-05,
5.0509596e-05,
4.9421078e-05,
4.9764870e-05,
4.8669252e-05,
4.9005515e-05,
4.7902826e-05,
4.8231382e-05,
4.7121421e-05,
4.7442590e-05,
4.6325758e-05,
4.6639550e-05,
4.5515500e-05,
4.5821892e-05,
4.4690732e-05,
4.4989674e-05,
4.3851733e-05,
4.4143731e-05,
4.2998831e-05,
4.3283696e-05,
4.2132017e-05,
4.2409620e-05,
4.1251080e-05,
4.1521397e-05,
4.0356095e-05,
4.0619587e-05,
3.9447668e-05,
3.9704201e-05,
3.8525477e-05,
3.8775116e-05,
3.7589752e-05,
3.7832718e-05,
3.6641075e-05,
3.6877410e-05,
3.5679136e-05,
3.5908752e-05,
3.4703989e-05,
3.4926909e-05,
3.3715651e-05,
3.3932084e-05,
3.2714746e-05,
3.2924830e-05,
3.1701197e-05,
3.1904901e-05,
3.0675124e-05,
3.0872537e-05,
2.9636643e-05,
2.9828045e-05,
2.8586284e-05,
2.8771273e-05,
2.7523459e-05,
2.7702288e-05,
2.6448557e-05,
2.6621389e-05,
2.5362000e-05,
2.5529019e-05,
2.4264241e-05,
2.4425209e-05,
2.3154818e-05,
2.3309922e-05,
2.2034159e-05,
2.2183522e-05,
2.0902357e-05,
2.1046142e-05,
1.9759625e-05,
1.9897527e-05,
1.8605835e-05,
1.8738287e-05,
1.7441604e-05,
1.7568663e-05,
1.6267285e-05,
1.6388975e-05,
1.5082748e-05,
1.5198838e-05,
1.3887933e-05,
1.3998647e-05,
1.2683198e-05,
1.2788708e-05,
1.1468841e-05,
1.1569194e-05,
1.0244735e-05,
1.0340044e-05,
9.0114749e-06,
9.1019785e-06,
7.7693530e-06,
7.8550439e-06,
6.5183983e-06,
6.5992396e-06,
5.2583840e-06,
5.3344697e-06,
3.9898714e-06,
4.0612848e-06,
2.7130313e-06,
2.7799692e-06,
1.4282857e-06,
1.4907438e-06,
1.3538100e-07,
1.9369879e-07,
-1.1648632e-06,
-1.1107236e-06,
-2.4726382e-06,
-2.4225610e-06,
-3.7878980e-06,
-3.7418797e-06,
-5.1104306e-06,
-5.0681263e-06,
-6.4396470e-06,
-6.4010337e-06,
-7.7753573e-06,
-7.7402888e-06,
-9.1173364e-06,
-9.0858392e-06,
-1.0465652e-05,
-1.0437405e-05,
-1.1819799e-05,
-1.1794892e-05,
-1.3179895e-05,
-1.3158113e-05,
-1.4545621e-05,
-1.4526860e-05,
-1.5916484e-05,
-1.5900479e-05,
-1.7292329e-05,
-1.7278975e-05,
-1.8672798e-05,
-1.8662098e-05,
-2.0057972e-05,
-2.0049926e-05,
-2.1447547e-05,
-2.1441864e-05,
-2.2841366e-05,
-2.2838029e-05,
-2.4239057e-05,
-2.4237885e-05,
-2.5640283e-05,
-2.5641070e-05,
-2.7044711e-05,
-2.7047463e-05,
-2.8452331e-05,
-2.8456861e-05,
-2.9862769e-05,
-2.9869186e-05,
-3.1276109e-05,
-3.1284280e-05,
-3.2691731e-05,
-3.2701426e-05,
-3.4109630e-05,
-3.4120732e-05,
-3.5529234e-05,
-3.5541634e-05,
-3.6950412e-05,
-3.6963941e-05,
-3.8373073e-05,
-3.8387785e-05,
-3.9796902e-05,
-3.9812610e-05,
-4.1221744e-05,
-4.1238462e-05,
-4.2647336e-05,
-4.2664787e-05,
-4.4073031e-05,
-4.4091230e-05,
-4.5498913e-05,
-4.5517648e-05,
-4.6924616e-05,
-4.6943835e-05,
-4.8349998e-05,
-4.8369634e-05,
-4.9774933e-05,
-4.9794749e-05,
-5.1198885e-05,
-5.1218806e-05,
-5.2621891e-05,
-5.2641703e-05,
-5.4043350e-05,
-5.4062779e-05,
-5.5462912e-05,
-5.5482232e-05,
-5.6880673e-05,
-5.6899569e-05,
-5.8296430e-05,
-5.8314853e-05,
-5.9709797e-05,
-5.9727628e-05,
-6.1120422e-05,
-6.1137297e-05,
-6.2528063e-05,
-6.2544052e-05,
-6.3932631e-05,
-6.3947356e-05,
-6.5333503e-05,
-6.5346834e-05,
-6.6730695e-05,
-6.6742827e-05,
-6.8124027e-05,
-6.8134656e-05,
-6.9513243e-05,
-6.9522210e-05,
-7.0897863e-05,
-7.0904919e-05,
-7.2277442e-05,
-7.2282584e-05,
-7.3652221e-05,
-7.3655423e-05,
-7.5021727e-05,
-7.5022621e-05,
-7.6385565e-05,
-7.6384179e-05,
-7.7743900e-05,
-7.7740055e-05,
-7.9096105e-05,
-7.9089726e-05,
-8.0441947e-05,
-8.0432760e-05,
-8.1781103e-05,
-8.1768938e-05,
-8.3113335e-05,
-8.3098473e-05,
-8.4438827e-05,
-8.4420880e-05,
-8.5756826e-05,
-8.5735635e-05,
-8.7067210e-05,
-8.7042849e-05,
-8.8369987e-05,
-8.8341999e-05,
-8.9664389e-05,
-8.9632965e-05,
-9.0950349e-05,
-9.0915266e-05,
-9.2227706e-05,
-9.2189014e-05,
-9.3496495e-05,
-9.3454144e-05,
-9.4756200e-05,
-9.4709717e-05,
-9.6006325e-05,
-9.5955829e-05,
-9.7246948e-05,
-9.7192383e-05,
-9.8477743e-05,
-9.8418691e-05,
-9.9698303e-05,
-9.9634924e-05,
-1.0090854e-04,
-1.0084078e-04,
-1.0210831e-04,
-1.0203621e-04,
-1.0329756e-04,
-1.0322060e-04,
-1.0447531e-04,
-1.0439347e-04,
-1.0564172e-04,
-1.0555515e-04,
-1.0679685e-04,
-1.0670527e-04,
-1.0794006e-04,
-1.0784331e-04,
-1.0907145e-04,
-1.0896938e-04,
-1.1019066e-04,
-1.1008330e-04,
-1.1129751e-04,
-1.1118468e-04,
-1.1239157e-04,
-1.1227285e-04,
-1.1347235e-04,
-1.1334808e-04,
-1.1454027e-04,
-1.1441035e-04,
-1.1559507e-04,
-1.1545911e-04,
-1.1663606e-04,
-1.1649414e-04,
-1.1766350e-04,
-1.1751527e-04,
-1.1867689e-04,
-1.1852233e-04,
-1.1967606e-04,
-1.1951487e-04,
-1.2066042e-04,
-1.2049269e-04,
-1.2163034e-04,
-1.2145615e-04,
-1.2258572e-04,
-1.2240476e-04,
-1.2352599e-04,
-1.2333804e-04,
-1.2445081e-04,
-1.2425602e-04,
-1.2536022e-04,
-1.2515829e-04,
-1.2625393e-04,
-1.2604480e-04,
-1.2713176e-04,
-1.2691525e-04,
-1.2799338e-04,
-1.2776976e-04,
-1.2883923e-04,
-1.2860815e-04,
-1.2966859e-04,
-1.2942987e-04,
-1.3048105e-04,
-1.3023467e-04,
-1.3127671e-04,
-1.3102272e-04,
-1.3205531e-04,
-1.3179358e-04,
-1.3281687e-04,
-1.3254730e-04,
-1.3356108e-04,
-1.3328364e-04,
-1.3428794e-04,
-1.3400263e-04,
-1.3499720e-04,
-1.3470375e-04,
-1.3568835e-04,
-1.3538673e-04,
-1.3636129e-04,
-1.3605159e-04,
-1.3701629e-04,
-1.3669839e-04,
-1.3765287e-04,
-1.3732679e-04,
-1.3827116e-04,
-1.3793669e-04,
-1.3887075e-04,
-1.3852790e-04,
-1.3945171e-04,
-1.3910024e-04,
-1.4001339e-04,
-1.3965336e-04,
-1.4055593e-04,
-1.4018728e-04,
-1.4107931e-04,
-1.4070215e-04,
-1.4158356e-04,
-1.4119768e-04,
-1.4206826e-04,
-1.4167373e-04,
-1.4253337e-04,
-1.4212994e-04,
-1.4297867e-04,
-1.4256634e-04,
-1.4340410e-04,
-1.4298266e-04,
-1.4380937e-04,
-1.4337892e-04,
-1.4419467e-04,
-1.4375512e-04,
-1.4455992e-04,
-1.4411131e-04,
-1.4490488e-04,
-1.4444694e-04,
-1.4522931e-04,
-1.4476214e-04,
-1.4553303e-04,
-1.4505651e-04,
-1.4581615e-04,
-1.4533028e-04,
-1.4607853e-04,
-1.4558325e-04,
-1.4632029e-04,
-1.4581546e-04,
-1.4654111e-04,
-1.4602678e-04,
-1.4674109e-04,
-1.4621711e-04,
-1.4691983e-04,
-1.4638605e-04,
-1.4707738e-04,
-1.4653385e-04,
-1.4721351e-04,
-1.4666036e-04,
-1.4732853e-04,
-1.4676567e-04,
-1.4742226e-04,
-1.4684968e-04,
-1.4749466e-04,
-1.4691215e-04,
-1.4754534e-04,
-1.4695307e-04,
-1.4757459e-04,
-1.4697234e-04,
-1.4758206e-04,
-1.4696992e-04,
-1.4756796e-04,
-1.4694583e-04,
-1.4753213e-04,
-1.4690029e-04,
-1.4747472e-04,
-1.4683302e-04,
-1.4739561e-04,
-1.4674395e-04,
-1.4729446e-04,
-1.4663285e-04,
-1.4717148e-04,
-1.4650000e-04,
-1.4702669e-04,
-1.4634526e-04,
-1.4685995e-04,
-1.4616879e-04,
-1.4667144e-04,
-1.4597033e-04,
-1.4646110e-04,
-1.4575023e-04,
-1.4622879e-04,
-1.4550808e-04,
-1.4597448e-04,
-1.4524383e-04,
-1.4569804e-04,
-1.4495770e-04,
-1.4540002e-04,
-1.4464985e-04,
-1.4508001e-04,
-1.4432003e-04,
-1.4473815e-04,
-1.4396849e-04,
-1.4437442e-04,
-1.4359495e-04,
-1.4398893e-04,
-1.4319961e-04,
-1.4358142e-04,
-1.4278237e-04,
-1.4315196e-04,
-1.4234324e-04,
-1.4270091e-04,
-1.4188272e-04,
-1.4222838e-04,
-1.4140042e-04,
-1.4173390e-04,
-1.4089624e-04,
-1.4121789e-04,
-1.4037053e-04,
-1.4068018e-04,
-1.3982323e-04,
-1.4012101e-04,
-1.3925425e-04,
-1.3954024e-04,
-1.3866387e-04,
-1.3893801e-04,
-1.3805225e-04,
-1.3831479e-04,
-1.3741955e-04,
-1.3767007e-04,
-1.3676518e-04,
-1.3700394e-04,
-1.3608961e-04,
-1.3631683e-04,
-1.3539300e-04,
-1.3560863e-04,
-1.3467552e-04,
-1.3487950e-04,
-1.3393689e-04,
-1.3412949e-04,
-1.3317750e-04,
-1.3335880e-04,
-1.3239762e-04,
-1.3256749e-04,
-1.3159687e-04,
-1.3175514e-04,
-1.3077530e-04,
-1.3092240e-04,
-1.2993362e-04,
-1.3006944e-04,
-1.2907158e-04,
-1.2919628e-04,
-1.2818946e-04,
-1.2830285e-04,
-1.2728710e-04,
-1.2738941e-04,
-1.2636478e-04,
-1.2645621e-04,
-1.2542275e-04,
-1.2550295e-04,
-1.2446059e-04,
-1.2452990e-04,
-1.2347910e-04,
-1.2353767e-04,
-1.2247847e-04,
-1.2252606e-04,
-1.2145835e-04,
-1.2149524e-04,
-1.2041906e-04,
-1.2044511e-04,
-1.1936072e-04,
-1.1937607e-04,
-1.1828354e-04,
-1.1828836e-04,
-1.1718761e-04,
-1.1718175e-04,
-1.1607308e-04,
-1.1605696e-04,
-1.1494063e-04,
-1.1491409e-04,
-1.1378987e-04,
-1.1375286e-04,
-1.1262092e-04,
-1.1257363e-04,
-1.1143400e-04,
-1.1137657e-04,
-1.1022955e-04,
-1.1016207e-04,
-1.0900768e-04,
-1.0893017e-04,
-1.0776834e-04,
-1.0768107e-04,
-1.0651220e-04,
-1.0641523e-04,
-1.0523922e-04,
-1.0513240e-04,
-1.0394915e-04,
-1.0383267e-04,
-1.0264265e-04,
-1.0251659e-04,
-1.0131976e-04,
-1.0118448e-04,
-9.9981024e-05,
-9.9836503e-05,
-9.8626386e-05,
-9.8472603e-05,
-9.7255907e-05,
-9.7093345e-05,
-9.5870231e-05,
-9.5698720e-05,
-9.4469036e-05,
-9.4288639e-05,
-9.3052678e-05,
-9.2863720e-05,
-9.1621792e-05,
-9.1424291e-05,
-9.0176406e-05,
-8.9970620e-05,
-8.8716848e-05,
-8.8502680e-05,
-8.7243030e-05,
-8.7020582e-05,
-8.5755341e-05,
-8.5524943e-05,
-8.4254041e-05,
-8.4015695e-05,
-8.2739208e-05,
-8.2493114e-05,
-8.1211457e-05,
-8.0957891e-05,
-7.9671020e-05,
-7.9409949e-05,
-7.8118076e-05,
-7.7849587e-05,
-7.6552699e-05,
-7.6276776e-05,
-7.4975081e-05,
-7.4691992e-05,
-7.3385810e-05,
-7.3095601e-05,
-7.1784899e-05,
-7.1487830e-05,
-7.0172828e-05,
-6.9869055e-05,
-6.8549939e-05,
-6.8239552e-05,
-6.6916258e-05,
-6.6599262e-05,
-6.5272162e-05,
-6.4948643e-05,
-6.3617654e-05,
-6.3287817e-05,
-6.1953324e-05,
-6.1617427e-05,
-6.0279595e-05,
-5.9937574e-05,
-5.8596550e-05,
-5.8248670e-05,
-5.6904588e-05,
-5.6550976e-05,
-5.5203862e-05,
-5.4844500e-05,
-5.3494542e-05,
-5.3129657e-05,
-5.1777150e-05,
-5.1406820e-05,
-5.0051790e-05,
-4.9676346e-05,
-4.8319127e-05,
-4.7938664e-05,
-4.6579287e-05,
-4.6193853e-05,
-4.4832523e-05,
-4.4442220e-05,
-4.3079012e-05,
-4.2684097e-05,
-4.1319095e-05,
-4.0919646e-05,
-3.9553111e-05,
-3.9149527e-05,
-3.7781568e-05,
-3.7373776e-05,
-3.6004694e-05,
-3.5593012e-05,
-3.4222872e-05,
-3.3807322e-05,
-3.2436212e-05,
-3.2016966e-05,
-3.0645037e-05,
-3.0222166e-05,
-2.8849645e-05,
-2.8423498e-05,
-2.7050526e-05,
-2.6621296e-05,
-2.5247979e-05,
-2.4815942e-05,
-2.3442289e-05,
-2.3007419e-05,
-2.1633882e-05,
-2.1196412e-05,
-1.9822877e-05,
-1.9382911e-05,
-1.8009600e-05,
-1.7567305e-05,
-1.6194384e-05,
-1.5749983e-05,
-1.4377829e-05,
-1.3931499e-05,
-1.2560184e-05,
-1.2112086e-05,
-1.0741680e-05,
-1.0291880e-05,
-8.9224160e-06,
-8.4712774e-06,
-7.1031275e-06,
-6.6506869e-06,
-5.2838715e-06,
-4.8303230e-06,
-3.4650410e-06,
-3.0105300e-06,
-1.6470717e-06,
-1.1919259e-06,
1.6946370e-07,
6.2534710e-07,
1.9846365e-06,
2.4411086e-06,
3.7980617e-06,
4.2550727e-06,
5.6095462e-06,
6.0664861e-06,
7.4182983e-06,
7.8753410e-06,
9.2242361e-06,
9.6811799e-06,
1.1027094e-05,
1.1483702e-05,
1.2826259e-05,
1.3282323e-05,
1.4621557e-05,
1.5077198e-05,
1.6412923e-05,
1.6867859e-05,
1.8199756e-05,
1.8653831e-05,
1.9981626e-05,
2.0434494e-05,
2.1758236e-05,
2.2209954e-05,
2.3529348e-05,
2.3979767e-05,
2.5294697e-05,
2.5743478e-05,
2.7053690e-05,
2.7500851e-05,
2.8806417e-05,
2.9251789e-05,
3.0552306e-05,
3.0995566e-05,
3.2290878e-05,
3.2731846e-05,
3.4021648e-05,
3.4460386e-05,
3.5744872e-05,
3.6181017e-05,
3.7459954e-05,
3.7893390e-05,
3.9166438e-05,
3.9596888e-05,
4.0864083e-05,
4.1291608e-05,
4.2552753e-05,
4.2976922e-05,
4.4231626e-05,
4.4652344e-05,
4.5900645e-05,
4.6317760e-05,
4.7559448e-05,
4.7973096e-05,
4.9208183e-05,
4.9617796e-05,
5.0846058e-05,
5.1251467e-05,
5.2472557e-05,
5.2873719e-05,
5.4087836e-05,
5.4484583e-05,
5.5691302e-05,
5.6083234e-05,
5.7282435e-05,
5.7669697e-05,
5.8861350e-05,
5.9243740e-05,
6.0427725e-05,
6.0805183e-05,
6.1981101e-05,
6.2353289e-05,
6.3520989e-05,
6.3887645e-05,
6.5047082e-05,
6.5408376e-05,
6.6559514e-05,
6.6915044e-05,
6.8057529e-05,
6.8407140e-05,
6.9541073e-05,
6.9884818e-05,
7.1009908e-05,
7.1347553e-05,
7.2463716e-05,
7.2795042e-05,
7.3901728e-05,
7.4226714e-05,
7.5323942e-05,
7.5642373e-05,
7.6730280e-05,
7.7042253e-05,
7.8120398e-05,
7.8425534e-05,
7.9493813e-05,
7.9792145e-05,
8.0850584e-05,
8.1141836e-05,
8.2190052e-05,
8.2474125e-05,
8.3512064e-05,
8.3788720e-05,
8.4816094e-05,
8.5085400e-05,
8.6102238e-05,
8.6364169e-05,
8.7370464e-05,
8.7624724e-05,
8.8619985e-05,
8.8866448e-05,
8.9850730e-05,
9.0089412e-05,
9.1062519e-05,
9.1292992e-05,
9.2254770e-05,
9.2477116e-05,
9.3427498e-05,
9.3641499e-05,
9.4580463e-05,
9.4786124e-05,
9.5713495e-05,
9.5910844e-05,
9.6826408e-05,
9.7014866e-05,
9.7918499e-05,
9.8098174e-05,
9.8989889e-05,
9.9160657e-05,
1.0004017e-04,
1.0020180e-04,
1.0106924e-04,
1.0122175e-04,
1.0207694e-04,
1.0222014e-04,
1.0306309e-04,
1.0319692e-04,
1.0402734e-04,
1.0415168e-04,
1.0496930e-04,
1.0508384e-04,
1.0588883e-04,
1.0599366e-04,
1.0678592e-04,
1.0688083e-04,
1.0766017e-04,
1.0774518e-04,
1.0851172e-04,
1.0858661e-04,
1.0934006e-04,
1.0940470e-04,
1.1014511e-04,
1.1019940e-04,
1.1092655e-04,
1.1097028e-04,
1.1168396e-04,
1.1171733e-04,
1.1241766e-04,
1.1244055e-04,
1.1312736e-04,
1.1313960e-04,
1.1381283e-04,
1.1381450e-04,
1.1447402e-04,
1.1446476e-04,
1.1511038e-04,
1.1509024e-04,
1.1572207e-04,
1.1569090e-04,
1.1630883e-04,
1.1626652e-04,
1.1687041e-04,
1.1681723e-04,
1.1740702e-04,
1.1734273e-04,
1.1791829e-04,
1.1784282e-04,
1.1840411e-04,
1.1831742e-04,
1.1886416e-04,
1.1876608e-04,
1.1929833e-04,
1.1918905e-04,
1.1970684e-04,
1.1958613e-04,
1.2008936e-04,
1.1995722e-04,
1.2044587e-04,
1.2030226e-04,
1.2077619e-04,
1.2062103e-04,
1.2108015e-04,
1.2091341e-04,
1.2135767e-04,
1.2117921e-04,
1.2160848e-04,
1.2141840e-04,
1.2183288e-04,
1.2163127e-04,
1.2203072e-04,
1.2181733e-04,
1.2220175e-04,
1.2197658e-04,
1.2234603e-04,
1.2210890e-04,
1.2246325e-04,
1.2221430e-04,
1.2255351e-04,
1.2229264e-04,
1.2261670e-04,
1.2234385e-04,
1.2265281e-04,
1.2236814e-04,
1.2266212e-04,
1.2236542e-04,
1.2264405e-04,
1.2233525e-04,
1.2259873e-04,
1.2227782e-04,
1.2252620e-04,
1.2219314e-04,
1.2242634e-04,
1.2208118e-04,
1.2229920e-04,
1.2194189e-04,
1.2214473e-04,
1.2177533e-04,
1.2196307e-04,
1.2158159e-04,
1.2175413e-04,
1.2136026e-04,
1.2151749e-04,
1.2111148e-04,
1.2125359e-04,
1.2083542e-04,
1.2096247e-04,
1.2053214e-04,
1.2064408e-04,
1.2020141e-04,
1.2029827e-04,
1.1984345e-04,
1.1992523e-04,
1.1945829e-04,
1.1952497e-04,
1.1904587e-04,
1.1909726e-04,
1.1860590e-04,
1.1864225e-04,
1.1813900e-04,
1.1816028e-04,
1.1764511e-04,
1.1765136e-04,
1.1712408e-04,
1.1711526e-04,
1.1657587e-04,
1.1655205e-04,
1.1600078e-04,
1.1596202e-04,
1.1539881e-04,
1.1534507e-04,
1.1476993e-04,
1.1470109e-04,
1.1411423e-04,
1.1403070e-04,
1.1343223e-04,
1.1333375e-04,
1.1272367e-04,
1.1261021e-04,
1.1198836e-04,
1.1186001e-04,
1.1122667e-04,
1.1108359e-04,
1.1043878e-04,
1.1028099e-04,
1.0962472e-04,
1.0945228e-04,
1.0878454e-04,
1.0859750e-04,
1.0791862e-04,
1.0771716e-04,
1.0702692e-04,
1.0681090e-04,
1.0610943e-04,
1.0587877e-04,
1.0516617e-04,
1.0492121e-04,
1.0419772e-04,
1.0393859e-04,
1.0320403e-04,
1.0293068e-04,
1.0218526e-04,
1.0189782e-04,
1.0114148e-04,
1.0084011e-04,
1.0007310e-04,
9.9757707e-05,
9.8979824e-05,
9.8650612e-05,
9.7862025e-05,
9.7518999e-05,
9.6720070e-05,
9.6363586e-05,
9.5554222e-05,
9.5184231e-05,
9.4364357e-05,
9.3980939e-05,
9.3150775e-05,
9.2754021e-05,
9.1913612e-05,
9.1503758e-05,
9.0653189e-05,
9.0230083e-05,
8.9369466e-05,
8.8933478e-05,
8.8062917e-05,
8.7614206e-05,
8.6734054e-05,
8.6272604e-05,
8.5382657e-05,
8.4908409e-05,
8.4008838e-05,
8.3522141e-05,
8.2612991e-05,
8.2113887e-05,
8.1195424e-05,
8.0684158e-05,
7.9756333e-05,
7.9232883e-05,
7.8296039e-05,
7.7760620e-05,
7.6814797e-05,
7.6267671e-05,
7.5313006e-05,
7.4753845e-05,
7.3790400e-05,
7.3219415e-05,
7.2247508e-05,
7.1665048e-05,
7.0684642e-05,
7.0090787e-05,
6.9102246e-05,
6.8497158e-05,
6.7500397e-05,
6.6884209e-05,
6.5879513e-05,
6.5252254e-05,
6.4239758e-05,
6.3601703e-05,
6.2581333e-05,
6.1932348e-05,
6.0904490e-05,
6.0244956e-05,
5.9209893e-05,
5.8539837e-05,
5.7497643e-05,
5.6817327e-05,
5.5768200e-05,
5.5077627e-05,
5.4021588e-05,
5.3321005e-05,
5.2258199e-05,
5.1547679e-05,
5.0478398e-05,
4.9758130e-05,
4.8682287e-05,
4.7952592e-05,
4.6870616e-05,
4.6131679e-05,
4.5043695e-05,
4.4295415e-05,
4.3201653e-05,
4.2444436e-05,
4.1344865e-05,
4.0578638e-05,
3.9473482e-05,
3.8698598e-05,
3.7587989e-05,
3.6804670e-05,
3.5688895e-05,
3.4897188e-05,
3.3776354e-05,
3.2976758e-05,
3.1851059e-05,
3.1043523e-05,
2.9912991e-05,
2.9097679e-05,
2.7962655e-05,
2.7139820e-05,
2.6000254e-05,
2.5169983e-05,
2.4026231e-05,
2.3188916e-05,
2.2041161e-05,
2.1197034e-05,
2.0045370e-05,
1.9194443e-05,
1.8039196e-05,
1.7181813e-05,
1.6022976e-05,
1.5159166e-05,
1.3996902e-05,
1.3127053e-05,
1.1961714e-05,
1.1085778e-05,
9.9174824e-06,
9.0358378e-06,
7.8649358e-06,
6.9778213e-06,
5.8044527e-06,
4.9119956e-06,
3.7361454e-06,
2.8385188e-06,
1.6605890e-06,
7.5795888e-07,
-4.2200946e-07,
-1.3293517e-06,
-2.5110220e-06,
-3.4228162e-06,
-4.6060023e-06,
-5.5223204e-06,
-6.7067090e-06,
-7.6270573e-06,
-8.8123709e-06,
-9.7367643e-06,
-1.0922955e-05,
-1.1851264e-05,
-1.3038162e-05,
-1.3970021e-05,
-1.5157301e-05,
-1.6092582e-05,
-1.7280028e-05,
-1.8218350e-05,
-1.9405703e-05,
-2.0347014e-05,
-2.1534274e-05,
-2.2478343e-05,
-2.3665044e-05,
-2.4611557e-05,
-2.5797543e-05,
-2.6746584e-05,
-2.7931766e-05,
-2.8883050e-05,
-3.0067014e-05,
-3.1020152e-05,
-3.2202709e-05,
-3.3157572e-05,
-3.4338444e-05,
-3.5294779e-05,
-3.6473921e-05,
-3.7431627e-05,
-3.8608935e-05,
-3.9567550e-05,
-4.0742553e-05,
-4.1701992e-05,
-4.2874683e-05,
-4.3834832e-05,
-4.5004913e-05,
-4.5965383e-05,
-4.7132491e-05,
-4.8093022e-05,
-4.9257066e-05,
-5.0217538e-05,
-5.1378303e-05,
-5.2338578e-05,
-5.3495996e-05,
-5.4455750e-05,
-5.5609458e-05,
-5.6568290e-05,
-5.7718057e-05,
-5.8675950e-05,
-5.9821775e-05,
-6.0778379e-05,
-6.1919776e-05,
-6.2874830e-05,
-6.4011692e-05,
-6.4965027e-05,
-6.6097253e-05,
-6.7048696e-05,
-6.8176074e-05,
-6.9125441e-05,
-7.0247662e-05,
-7.1194530e-05,
-7.2311240e-05,
-7.3255397e-05,
-7.4366653e-05,
-7.5308031e-05,
-7.6413576e-05,
-7.7351751e-05,
-7.8451173e-05,
-7.9386093e-05,
-8.0479340e-05,
-8.1410849e-05,
-8.2497734e-05,
-8.3425521e-05,
-8.4505743e-05,
-8.5429547e-05,
-8.6502709e-05,
-8.7422266e-05,
-8.8488264e-05,
-8.9403463e-05,
-9.0462353e-05,
-9.1373014e-05,
-9.2424279e-05,
-9.3330008e-05,
-9.4373534e-05,
-9.5274318e-05,
-9.6309837e-05,
-9.7205532e-05,
-9.8232793e-05,
-9.9122994e-05,
-1.0014168e-04,
-1.0102617e-04,
-1.0203606e-04,
-1.0291482e-04,
-1.0391586e-04,
-1.0478874e-04,
-1.0578075e-04,
-1.0664738e-04,
-1.0762990e-04,
-1.0849001e-04,
-1.0946311e-04,
-1.1031653e-04,
-1.1127982e-04,
-1.1212643e-04,
-1.1307966e-04,
-1.1391913e-04,
-1.1486209e-04,
-1.1569428e-04,
-1.1662693e-04,
-1.1745181e-04,
-1.1837386e-04,
-1.1919123e-04,
-1.2010234e-04,
-1.2091169e-04,
-1.2181171e-04,
-1.2261300e-04,
-1.2350185e-04,
-1.2429474e-04,
-1.2517226e-04,
-1.2595661e-04,
-1.2682247e-04,
-1.2759813e-04,
-1.2845228e-04,
-1.2921906e-04,
-1.3006137e-04,
-1.3081902e-04,
-1.3164914e-04,
-1.3239746e-04,
-1.3321508e-04,
-1.3395370e-04,
-1.3475884e-04,
-1.3548784e-04,
-1.3628023e-04,
-1.3699926e-04,
-1.3777892e-04,
-1.3848772e-04,
-1.3925432e-04,
-1.3995287e-04,
-1.4070632e-04,
-1.4139434e-04,
-1.4213435e-04,
-1.4281148e-04,
-1.4353793e-04,
-1.4420405e-04,
-1.4491667e-04,
-1.4557170e-04,
-1.4627051e-04,
-1.4691433e-04,
-1.4759902e-04,
-1.4823142e-04,
-1.4890193e-04,
-1.4952254e-04,
-1.5017867e-04,
-1.5078749e-04,
-1.5142899e-04,
-1.5202575e-04,
-1.5265236e-04,
-1.5323690e-04,
-1.5384864e-04,
-1.5442082e-04,
-1.5501743e-04,
-1.5557731e-04,
-1.5615871e-04,
-1.5670590e-04,
-1.5727188e-04,
-1.5780633e-04,
-1.5835656e-04,
-1.5887795e-04,
-1.5941252e-04,
-1.5992074e-04,
-1.6043933e-04,
-1.6093427e-04,
-1.6143676e-04,
-1.6191843e-04,
-1.6240470e-04,
-1.6287281e-04,
-1.6334272e-04,
-1.6379731e-04,
-1.6425062e-04,
-1.6469134e-04,
-1.6512790e-04,
-1.6555468e-04,
-1.6597417e-04,
-1.6638694e-04,
-1.6678953e-04,
-1.6718803e-04,
-1.6757349e-04,
-1.6795771e-04,
-1.6832603e-04,
-1.6869585e-04,
-1.6904668e-04,
-1.6940179e-04,
-1.6973521e-04,
-1.7007562e-04,
-1.7039129e-04,
-1.7071686e-04,
-1.7101467e-04,
-1.7132522e-04,
-1.7160511e-04,
-1.7190076e-04,
-1.7216266e-04,
-1.7244310e-04,
-1.7268697e-04,
-1.7295213e-04,
-1.7317784e-04,
-1.7342744e-04,
-1.7363472e-04,
-1.7386872e-04,
-1.7405775e-04,
-1.7427599e-04,
-1.7444651e-04,
-1.7464898e-04,
-1.7480092e-04,
-1.7498741e-04,
-1.7512084e-04,
-1.7529147e-04,
-1.7540607e-04,
-1.7556057e-04,
-1.7565643e-04,
-1.7579467e-04,
-1.7587153e-04,
-1.7599330e-04,
-1.7605120e-04,
-1.7615672e-04,
-1.7619567e-04,
-1.7628458e-04,
-1.7630449e-04,
-1.7637678e-04,
-1.7637756e-04,
-1.7643321e-04,
-1.7641486e-04,
-1.7645374e-04,
-1.7641598e-04,
-1.7643805e-04,
-1.7638094e-04,
-1.7638602e-04,
-1.7630941e-04,
-1.7629753e-04,
-1.7620165e-04,
-1.7617294e-04,
-1.7605746e-04,
-1.7601156e-04,
-1.7587651e-04,
-1.7581348e-04,
-1.7565883e-04,
-1.7557875e-04,
-1.7540437e-04,
-1.7530697e-04,
-1.7511279e-04,
-1.7499826e-04,
-1.7478422e-04,
-1.7465241e-04,
-1.7441861e-04,
-1.7426967e-04,
-1.7401616e-04,
-1.7384986e-04,
-1.7357626e-04,
-1.7339255e-04,
-1.7309901e-04,
-1.7289800e-04,
-1.7258458e-04,
-1.7236623e-04,
-1.7203276e-04,
-1.7179694e-04,
-1.7144347e-04,
-1.7119041e-04,
-1.7081681e-04,
-1.7054645e-04,
-1.7015291e-04,
-1.6986529e-04,
-1.6945160e-04,
-1.6914636e-04,
-1.6871247e-04,
-1.6838992e-04,
-1.6793605e-04,
-1.6759619e-04,
-1.6712237e-04,
-1.6676503e-04,
-1.6627117e-04,
-1.6589644e-04,
-1.6538263e-04,
-1.6499053e-04,
-1.6445668e-04,
-1.6404735e-04,
-1.6349353e-04,
-1.6306682e-04,
-1.6249285e-04,
-1.6204867e-04,
-1.6145487e-04,
-1.6099359e-04,
-1.6038004e-04,
-1.5990150e-04,
-1.5926808e-04,
-1.5877219e-04,
-1.5811901e-04,
-1.5760588e-04,
-1.5693306e-04,
-1.5640256e-04,
-1.5571014e-04,
-1.5516255e-04,
-1.5445047e-04,
-1.5388561e-04,
-1.5315392e-04,
-1.5257205e-04,
-1.5182111e-04,
-1.5122231e-04,
-1.5045197e-04,
-1.4983608e-04,
-1.4904627e-04,
-1.4841342e-04,
-1.4760437e-04,
-1.4695462e-04,
-1.4612645e-04,
-1.4545977e-04,
-1.4461261e-04,
-1.4392917e-04,
-1.4306295e-04,
-1.4236279e-04,
-1.4147772e-04,
-1.4076113e-04,
-1.3985731e-04,
-1.3912411e-04,
-1.3820138e-04,
-1.3745162e-04,
-1.3651020e-04,
-1.3574412e-04,
-1.3478418e-04,
-1.3400190e-04,
-1.3302347e-04,
-1.3222509e-04,
-1.3122837e-04,
-1.3041387e-04,
-1.2939884e-04,
-1.2856851e-04,
-1.2753534e-04,
-1.2668929e-04,
-1.2563794e-04,
-1.2477603e-04,
-1.2370657e-04,
-1.2282914e-04,
-1.2174195e-04,
-1.2084913e-04,
-1.1974424e-04,
-1.1883621e-04,
-1.1771364e-04,
-1.1679049e-04,
-1.1565045e-04,
-1.1471212e-04,
-1.1355472e-04,
-1.1260155e-04,
-1.1142690e-04,
-1.1045896e-04,
-1.0926705e-04,
-1.0828445e-04,
-1.0707559e-04,
-1.0607872e-04,
-1.0485310e-04,
-1.0384198e-04,
-1.0259969e-04,
-1.0157440e-04,
-1.0031553e-04,
-9.9276210e-05,
-9.8000930e-05,
-9.6947662e-05,
-9.5656261e-05,
-9.4589235e-05,
-9.3281871e-05,
-9.2201265e-05,
-9.0878011e-05,
-8.9784065e-05,
-8.8445293e-05,
-8.7338202e-05,
-8.5983858e-05,
-8.4863716e-05,
-8.3494063e-05,
-8.2360903e-05,
-8.0976083e-05,
-7.9830183e-05,
-7.8430366e-05,
-7.7271953e-05,
-7.5857524e-05,
-7.4686657e-05,
-7.3257835e-05,
-7.2074731e-05,
-7.0631631e-05,
-6.9436527e-05,
-6.7979416e-05,
-6.6772324e-05,
-6.5301247e-05,
-6.4082480e-05,
-6.2597712e-05,
-6.1367390e-05,
-5.9869158e-05,
-5.8627609e-05,
-5.7116105e-05,
-5.5863583e-05,
-5.4339113e-05,
-5.3075726e-05,
-5.1538480e-05,
-5.0264323e-05,
-4.8714489e-05,
-4.7429882e-05,
-4.5867614e-05,
-4.4572623e-05,
-4.2998260e-05,
-4.1693271e-05,
-4.0106934e-05,
-3.8792196e-05,
-3.7194235e-05,
-3.5869937e-05,
-3.4260534e-05,
-3.2926963e-05,
-3.1306286e-05,
-2.9963643e-05,
-2.8331856e-05,
-2.6980190e-05,
-2.5337660e-05,
-2.3977418e-05,
-2.2324249e-05,
-2.0955678e-05,
-1.9292291e-05,
-1.7915600e-05,
-1.6242067e-05,
-1.4857607e-05,
-1.3174292e-05,
-1.1782165e-05,
-1.0089236e-05,
-8.6897835e-06,
-6.9873915e-06,
-5.5808143e-06,
-3.8691961e-06,
-2.4557436e-06,
-7.3541924e-07,
6.8458348e-07,
2.4134832e-06,
3.8397062e-06,
5.5769553e-06,
7.0093348e-06,
8.7546590e-06,
1.0192766e-05,
1.1945942e-05,
1.3389588e-05,
1.5150342e-05,
1.6599234e-05,
1.8367367e-05,
1.9821287e-05,
2.1596452e-05,
2.3054894e-05,
2.4836645e-05,
2.6299628e-05,
2.8087752e-05,
2.9554969e-05,
3.1349360e-05,
3.2820609e-05,
3.4620749e-05,
3.6095650e-05,
3.7901416e-05,
3.9379677e-05,
4.1190653e-05,
4.2672111e-05,
4.4488113e-05,
4.5972398e-05,
4.7793003e-05,
4.9279614e-05,
5.1104656e-05,
5.2593723e-05,
5.4422860e-05,
5.5914076e-05,
5.7747033e-05,
5.9239940e-05,
6.1076227e-05,
6.2570647e-05,
6.4410079e-05,
6.5905671e-05,
6.7747881e-05,
6.9244521e-05,
7.1089279e-05,
7.2586463e-05,
7.4433344e-05,
7.5930831e-05,
7.7779868e-05,
7.9277452e-05,
8.1128167e-05,
8.2625524e-05,
8.4477438e-05,
8.5974085e-05,
8.7826997e-05,
8.9322871e-05,
9.1176466e-05,
9.2671176e-05,
9.4525264e-05,
9.6018576e-05,
9.7872702e-05,
9.9364170e-05,
1.0121805e-04,
1.0270761e-04,
1.0456116e-04,
1.0604820e-04,
1.0790092e-04,
1.0938516e-04,
1.1123663e-04,
1.1271783e-04,
1.1456802e-04,
1.1604597e-04,
1.1789445e-04,
1.1936876e-04,
1.2121537e-04,
1.2268564e-04,
1.2452979e-04,
1.2599576e-04,
1.2783739e-04,
1.2929889e-04,
1.3113758e-04,
1.3259396e-04,
1.3442937e-04,
1.3588043e-04,
1.3771236e-04,
1.3915801e-04,
1.4098621e-04,
1.4242601e-04,
1.4425001e-04,
1.4568368e-04,
1.4750318e-04,
1.4893022e-04,
1.5074484e-04,
1.5216516e-04,
1.5397483e-04,
1.5538806e-04,
1.5719215e-04,
1.5859789e-04,
1.6039629e-04,
1.6179431e-04,
1.6358686e-04,
1.6497696e-04,
1.6676313e-04,
1.6814480e-04,
1.6992420e-04,
1.7129729e-04,
1.7306950e-04,
1.7443358e-04,
1.7619853e-04,
1.7755356e-04,
1.7931086e-04,
1.8065634e-04,
1.8240550e-04,
1.8374124e-04,
1.8548218e-04,
1.8680787e-04,
1.8854025e-04,
1.8985546e-04,
1.9157879e-04,
1.9288319e-04,
1.9459725e-04,
1.9589064e-04,
1.9759501e-04,
1.9887719e-04,
2.0057185e-04,
2.0184256e-04,
2.0352685e-04,
2.0478566e-04,
2.0645936e-04,
2.0770614e-04,
2.0936899e-04,
2.1060331e-04,
2.1225488e-04,
2.1347633e-04,
2.1511642e-04,
2.1632483e-04,
2.1795323e-04,
2.1914825e-04,
2.2076462e-04,
2.2194618e-04,
2.2355027e-04,
2.2471780e-04,
2.2630910e-04,
2.2746234e-04,
2.2904072e-04,
2.3017950e-04,
2.3174455e-04,
2.3286846e-04,
2.3441990e-04,
2.3552869e-04,
2.3706639e-04,
2.3815987e-04,
2.3968353e-04,
2.4076124e-04,
2.4227059e-04,
2.4333241e-04,
2.4482700e-04,
2.4587234e-04,
2.4735199e-04,
2.4838073e-04,
2.4984522e-04,
2.5085713e-04,
2.5230607e-04,
2.5330082e-04,
2.5473408e-04,
2.5571147e-04,
2.5712876e-04,
2.5808846e-04,
2.5948943e-04,
2.6043110e-04,
2.6181557e-04,
2.6273892e-04,
2.6410640e-04,
2.6501117e-04,
2.6636162e-04,
2.6724768e-04,
2.6858082e-04,
2.6944785e-04,
2.7076333e-04,
2.7161117e-04,
2.7290879e-04,
2.7373710e-04,
2.7501650e-04,
2.7582493e-04,
2.7708588e-04,
2.7787429e-04,
2.7911661e-04,
2.7988459e-04,
2.8110791e-04,
2.8185550e-04,
2.8305967e-04,
2.8378660e-04,
2.8497137e-04,
2.8567727e-04,
2.8684232e-04,
2.8752712e-04,
2.8867206e-04,
2.8933540e-04,
2.9046003e-04,
2.9110175e-04,
2.9220591e-04,
2.9282588e-04,
2.9390936e-04,
2.9450717e-04,
2.9556974e-04,
2.9614537e-04,
2.9718679e-04,
2.9773995e-04,
2.9875998e-04,
2.9929034e-04,
3.0028873e-04,
3.0079615e-04,
3.0177253e-04,
3.0225686e-04,
3.0321118e-04,
3.0367234e-04,
3.0460438e-04,
3.0504213e-04,
3.0595159e-04,
3.0636561e-04,
3.0725242e-04,
3.0764252e-04,
3.0850645e-04,
3.0887246e-04,
3.0971319e-04,
3.1005487e-04,
3.1087238e-04,
3.1118945e-04,
3.1198352e-04,
3.1227606e-04,
3.1304658e-04,
3.1331434e-04,
3.1406103e-04,
3.1430371e-04,
3.1502629e-04,
3.1524378e-04,
3.1594226e-04,
3.1613434e-04,
3.1680852e-04,
3.1697504e-04,
3.1762464e-04,
3.1776550e-04,
3.1839053e-04,
3.1850544e-04,
3.1910585e-04,
3.1919482e-04,
3.1977030e-04,
3.1983298e-04,
3.2038334e-04,
3.2041959e-04,
3.2094467e-04,
3.2095453e-04,
3.2145434e-04,
3.2143760e-04,
3.2191194e-04,
3.2186833e-04,
3.2231709e-04,
3.2224669e-04,
3.2266970e-04,
3.2257235e-04,
3.2296961e-04,
3.2284505e-04,
3.2321620e-04,
3.2306432e-04,
3.2340934e-04,
3.2323015e-04,
3.2354899e-04,
3.2334253e-04,
3.2363505e-04,
3.2340108e-04,
3.2366708e-04,
3.2340537e-04,
3.2364492e-04,
3.2335558e-04,
3.2356842e-04,
3.2325138e-04,
3.2343746e-04,
3.2309238e-04,
3.2325155e-04,
3.2287862e-04,
3.2301090e-04,
3.2261009e-04,
3.2271544e-04,
3.2228672e-04,
3.2236482e-04,
3.2190796e-04,
3.2195872e-04,
3.2147379e-04,
3.2149736e-04,
3.2098427e-04,
3.2098049e-04,
3.2043923e-04,
3.2040797e-04,
3.1983838e-04,
3.1977977e-04,
3.1918199e-04,
3.1909596e-04,
3.1846988e-04,
3.1835637e-04,
3.1770189e-04,
3.1756060e-04,
3.1687770e-04,
3.1670876e-04,
3.1599771e-04,
3.1580116e-04,
3.1506167e-04,
3.1483751e-04,
3.1406961e-04,
3.1381769e-04,
3.1302148e-04,
3.1274203e-04,
3.1191735e-04,
3.1161023e-04,
3.1075708e-04,
3.1042232e-04,
3.0954059e-04,
3.0917811e-04,
3.0826800e-04,
3.0787807e-04,
3.0693972e-04,
3.0652216e-04,
3.0555536e-04,
3.0511024e-04,
3.0411505e-04,
3.0364238e-04,
3.0261897e-04,
3.0211882e-04,
3.0106697e-04,
3.0053932e-04,
2.9945922e-04,
2.9890417e-04,
2.9779574e-04,
2.9721347e-04,
2.9607702e-04,
2.9546761e-04,
2.9430299e-04,
2.9366624e-04,
2.9247353e-04,
2.9180960e-04,
2.9058886e-04,
2.8989790e-04,
2.8864928e-04,
2.8793121e-04,
2.8665471e-04,
2.8590975e-04,
2.8460555e-04,
2.8383373e-04,
2.8250188e-04,
2.8170347e-04,
2.8034419e-04,
2.7951900e-04,
2.7813210e-04,
2.7728018e-04,
2.7586603e-04,
2.7498761e-04,
2.7354625e-04,
2.7264148e-04,
2.7117305e-04,
2.7024192e-04,
2.6874665e-04,
2.6778942e-04,
2.6626731e-04,
2.6528395e-04,
2.6373525e-04,
2.6272606e-04,
2.6115075e-04,
2.6011554e-04,
2.5851377e-04,
2.5745288e-04,
2.5582501e-04,
2.5473858e-04,
2.5308476e-04,
2.5197288e-04,
2.5029322e-04,
2.4915606e-04,
2.4745082e-04,
2.4628849e-04,
2.4455762e-04,
2.4337025e-04,
2.4161415e-04,
2.4040199e-04,
2.3862052e-04,
2.3738371e-04,
2.3557731e-04,
2.3431618e-04,
2.3248506e-04,
2.3119965e-04,
2.2934402e-04,
2.2803451e-04,
2.2615447e-04,
2.2482111e-04,
2.2291687e-04,
2.2155965e-04,
2.1963140e-04,
2.1825076e-04,
2.1629884e-04,
2.1489486e-04,
2.1291932e-04,
2.1149246e-04,
2.0949364e-04,
2.0804407e-04,
2.0602208e-04,
2.0454990e-04,
2.0250500e-04,
2.0101052e-04,
1.9894288e-04,
1.9742631e-04,
1.9533613e-04,
1.9379765e-04,
1.9168539e-04,
1.9012550e-04,
1.8799132e-04,
1.8640997e-04,
1.8425419e-04,
1.8265189e-04,
1.8047465e-04,
1.7885155e-04,
1.7665302e-04,
1.7500939e-04,
1.7278993e-04,
1.7112606e-04,
1.6888594e-04,
1.6720207e-04,
1.6494156e-04,
1.6323811e-04,
1.6095770e-04,
1.5923491e-04,
1.5693461e-04,
1.5519261e-04,
1.5287293e-04,
1.5111206e-04,
1.4877315e-04,
1.4699372e-04,
1.4463588e-04,
1.4283827e-04,
1.4046191e-04,
1.3864629e-04,
1.3625176e-04,
1.3441851e-04,
1.3200615e-04,
1.3015569e-04,
1.2772583e-04,
1.2585817e-04,
1.2341095e-04,
1.2152659e-04,
1.1906254e-04,
1.1716168e-04,
1.1468116e-04,
1.1276423e-04,
1.1026755e-04,
1.0833494e-04,
1.0582238e-04,
1.0387430e-04,
1.0134632e-04,
9.9383226e-05,
9.6840217e-05,
9.4862349e-05,
9.2304496e-05,
9.0311996e-05,
8.7739769e-05,
8.5733324e-05,
8.3147165e-05,
8.1126981e-05,
7.8527328e-05,
7.6493755e-05,
7.3880893e-05,
7.1834336e-05,
6.9208471e-05,
6.7149369e-05,
6.4511043e-05,
6.2439757e-05,
5.9789327e-05,
5.7706104e-05,
5.5043773e-05,
5.2949072e-05,
5.0275556e-05,
4.8169907e-05,
4.5485482e-05,
4.3369180e-05,
4.0674204e-05,
3.8547508e-05,
3.5842405e-05,
3.3705738e-05,
3.0990833e-05,
2.8844828e-05,
2.6120595e-05,
2.3965508e-05,
2.1232257e-05,
1.9068520e-05,
1.6326638e-05,
1.4154851e-05,
1.1404963e-05,
9.2254151e-06,
6.4677293e-06,
4.2808140e-06,
1.5156816e-06,
-6.7823824e-07,
-3.4502089e-06,
-5.6506953e-06,
-8.4289994e-06,
-1.0635468e-05,
-1.3419841e-05,
-1.5631999e-05,
-1.8422025e-05,
-2.0639362e-05,
-2.3434477e-05,
-2.5656460e-05,
-2.8456315e-05,
-3.0682710e-05,
-3.3486955e-05,
-3.5717290e-05,
-3.8525413e-05,
-4.0759111e-05,
-4.3570544e-05,
-4.5807210e-05,
-4.8621403e-05,
-5.0860667e-05,
-5.3677438e-05,
-5.5918829e-05,
-5.8737580e-05,
-6.0980648e-05,
-6.3800809e-05,
-6.6045087e-05,
-6.8866411e-05,
-7.1111565e-05,
-7.3933491e-05,
-7.6178975e-05,
-7.9000865e-05,
-8.1246085e-05,
-8.4067572e-05,
-8.6312206e-05,
-8.9132793e-05,
-9.1376582e-05,
-9.4195879e-05,
-9.6438156e-05,
-9.9255574e-05,
-1.0149590e-04,
-1.0431096e-04,
-1.0654888e-04,
-1.0936121e-04,
-1.1159629e-04,
-1.1440526e-04,
-1.1663689e-04,
-1.1944196e-04,
-1.2166975e-04,
-1.2447061e-04,
-1.2669419e-04,
-1.2949035e-04,
-1.3170934e-04,
-1.3450012e-04,
-1.3671384e-04,
-1.3949884e-04,
-1.4170678e-04,
-1.4448554e-04,
-1.4668730e-04,
-1.4945934e-04,
-1.5165438e-04,
-1.5441917e-04,
-1.5660691e-04,
-1.5936406e-04,
-1.6154421e-04,
-1.6429327e-04,
-1.6646527e-04,
-1.6920573e-04,
-1.7136904e-04,
-1.7410016e-04,
-1.7625428e-04,
-1.7897574e-04,
-1.8112015e-04,
-1.8383145e-04,
-1.8596579e-04,
-1.8866648e-04,
-1.9079011e-04,
-1.9347966e-04,
-1.9559215e-04,
-1.9827023e-04,
-2.0037110e-04,
-2.0303710e-04,
-2.0512569e-04,
-2.0777912e-04,
-2.0985488e-04,
-2.1249519e-04,
-2.1455785e-04,
-2.1718455e-04,
-2.1923355e-04,
-2.2184624e-04,
-2.2388119e-04,
-2.2647934e-04,
-2.2849956e-04,
-2.3108262e-04,
-2.3308780e-04,
-2.3565533e-04,
-2.3764480e-04,
-2.4019618e-04,
-2.4216943e-04,
-2.4470424e-04,
-2.4666078e-04,
-2.4917864e-04,
-2.5111815e-04,
-2.5361839e-04,
-2.5554036e-04,
-2.5802263e-04,
-2.5992654e-04,
-2.6239015e-04,
-2.6427543e-04,
-2.6671996e-04,
-2.6858623e-04,
-2.7101109e-04,
-2.7285777e-04,
-2.7526249e-04,
-2.7708923e-04,
-2.7947336e-04,
-2.8127965e-04,
-2.8364281e-04,
-2.8542814e-04,
-2.8776966e-04,
-2.8953364e-04,
-2.9185308e-04,
-2.9359508e-04,
-2.9589171e-04,
-2.9761135e-04,
-2.9988496e-04,
-3.0158172e-04,
-3.0383168e-04,
-3.0550516e-04,
-3.0773108e-04,
-3.0938086e-04,
-3.1158221e-04,
-3.1320772e-04,
-3.1538404e-04,
-3.1698472e-04,
-3.1913553e-04,
-3.2071106e-04,
-3.2283572e-04,
-3.2438545e-04,
-3.2648351e-04,
-3.2800725e-04,
-3.3007841e-04,
-3.3157552e-04,
-3.3361924e-04,
-3.3508935e-04,
-3.3710521e-04,
-3.3854774e-04,
-3.4053515e-04,
-3.4194962e-04,
-3.4390816e-04,
-3.4529408e-04,
-3.4722346e-04,
-3.4858039e-04,
-3.5047988e-04,
-3.5180730e-04,
-3.5367676e-04,
-3.5497446e-04,
-3.5681322e-04,
-3.5808062e-04,
-3.5988832e-04,
-3.6112490e-04,
-3.6290100e-04,
-3.6410627e-04,
-3.6585029e-04,
-3.6702391e-04,
-3.6873554e-04,
-3.6987705e-04,
-3.7155594e-04,
-3.7266472e-04,
-3.7431029e-04,
-3.7538608e-04,
-3.7699817e-04,
-3.7804051e-04,
-3.7961833e-04,
-3.8062678e-04,
-3.8217002e-04,
-3.8314407e-04,
-3.8465223e-04,
-3.8559153e-04,
-3.8706430e-04,
-3.8796859e-04,
-3.8940555e-04,
-3.9027433e-04,
-3.9167496e-04,
-3.9250763e-04,
-3.9387165e-04,
-3.9466813e-04,
-3.9599508e-04,
-3.9675462e-04,
-3.9804400e-04,
-3.9876639e-04,
-4.0001781e-04,
-4.0070265e-04,
-4.0191575e-04,
-4.0256273e-04,
-4.0373723e-04,
-4.0434600e-04,
-4.0548128e-04,
-4.0605135e-04,
-4.0714694e-04,
-4.0767796e-04,
-4.0873377e-04,
-4.0922551e-04,
-4.1024086e-04,
-4.1069267e-04,
-4.1166738e-04,
-4.1207911e-04,
-4.1301270e-04,
-4.1338407e-04,
-4.1427627e-04,
-4.1460688e-04,
-4.1545728e-04,
-4.1574664e-04,
-4.1655470e-04,
-4.1680251e-04,
-4.1756797e-04,
-4.1777407e-04,
-4.1849679e-04,
-4.1866063e-04,
-4.1933998e-04,
-4.1946129e-04,
-4.2009716e-04,
-4.2017564e-04,
-4.2076763e-04,
-4.2080300e-04,
-4.2135076e-04,
-4.2134264e-04,
-4.2184572e-04,
-4.2179364e-04,
-4.2225181e-04,
-4.2215568e-04,
-4.2256876e-04,
-4.2242837e-04,
-4.2279597e-04,
-4.2261065e-04,
-4.2293246e-04,
-4.2270218e-04,
-4.2297797e-04,
-4.2270227e-04,
-4.2293180e-04,
-4.2261041e-04,
-4.2279328e-04,
-4.2242593e-04,
-4.2256181e-04,
-4.2214818e-04,
-4.2223705e-04,
-4.2177708e-04,
-4.2181859e-04,
-4.2131195e-04,
-4.2130558e-04,
-4.2075181e-04,
-4.2069756e-04,
-4.2009657e-04,
-4.1999414e-04,
-4.1934556e-04,
-4.1919475e-04,
-4.1849834e-04,
-4.1829887e-04,
-4.1755443e-04,
-4.1730610e-04,
-4.1651350e-04,
-4.1621619e-04,
-4.1537514e-04,
-4.1502846e-04,
-4.1413864e-04,
-4.1374219e-04,
-4.1280355e-04,
-4.1235738e-04,
-4.1136968e-04,
-4.1087350e-04,
-4.0983658e-04,
-4.0929017e-04,
-4.0820380e-04,
-4.0760705e-04,
-4.0647104e-04,
-4.0582375e-04,
-4.0463806e-04,
-4.0393995e-04,
-4.0270414e-04,
-4.0195503e-04,
-4.0066897e-04,
-3.9986869e-04,
-3.9853255e-04,
-3.9768098e-04,
-3.9629446e-04,
-3.9539138e-04,
-3.9395438e-04,
-3.9299963e-04,
-3.9151195e-04,
-3.9050543e-04,
-3.8896694e-04,
-3.8790847e-04,
-3.8631909e-04,
-3.8520840e-04,
-3.8356791e-04,
-3.8240501e-04,
-3.8071338e-04,
-3.7949829e-04,
-3.7775556e-04,
-3.7648800e-04,
-3.7469396e-04,
-3.7337379e-04,
-3.7152831e-04,
-3.7015548e-04,
-3.6825846e-04,
-3.6683285e-04,
-3.6488421e-04,
-3.6340582e-04,
-3.6140546e-04,
-3.5987412e-04,
-3.5782210e-04,
-3.5623784e-04,
-3.5413406e-04,
-3.5249692e-04,
-3.5034128e-04,
-3.4865102e-04,
-3.4644343e-04,
-3.4469998e-04,
-3.4244039e-04,
-3.4064378e-04,
-3.3833221e-04,
-3.3648241e-04,
-3.3411882e-04,
-3.3221595e-04,
-3.2980023e-04,
-3.2784426e-04,
-3.2537654e-04,
-3.2336737e-04,
-3.2084751e-04,
-3.1878521e-04,
-3.1621315e-04,
-3.1409765e-04,
-3.1147342e-04,
-3.0930472e-04,
-3.0662845e-04,
-3.0440657e-04,
-3.0167831e-04,
-2.9940336e-04,
-2.9662313e-04,
-2.9429514e-04,
-2.9146288e-04,
-2.8908190e-04,
-2.8619761e-04,
-2.8376358e-04,
-2.8082735e-04,
-2.7834040e-04,
-2.7535224e-04,
-2.7281242e-04,
-2.6977244e-04,
-2.6717989e-04,
-2.6408826e-04,
-2.6144296e-04,
-2.5829978e-04,
-2.5560184e-04,
-2.5240721e-04,
-2.4965656e-04,
-2.4641049e-04,
-2.4360730e-04,
-2.4030982e-04,
-2.3745420e-04,
-2.3410565e-04,
-2.3119771e-04,
-2.2779807e-04,
-2.2483794e-04,
-2.2138743e-04,
-2.1837533e-04,
-2.1487400e-04,
-2.1180995e-04,
-2.0825800e-04,
-2.0514206e-04,
-2.0153964e-04,
-1.9837186e-04,
-1.9471910e-04,
-1.9149975e-04,
-1.8779688e-04,
-1.8452622e-04,
-1.8077352e-04,
-1.7745155e-04,
-1.7364901e-04,
-1.7027599e-04,
-1.6642395e-04,
-1.6300004e-04,
-1.5909853e-04,
-1.5562390e-04,
-1.5167319e-04,
-1.4814795e-04,
-1.4414820e-04,
-1.4057263e-04,
-1.3652418e-04,
-1.3289854e-04,
-1.2880165e-04,
-1.2512624e-04,
-1.2098094e-04,
-1.1725573e-04,
-1.1306230e-04,
-1.0928774e-04,
-1.0504646e-04,
-1.0122266e-04,
-9.6933726e-05,
-9.3061040e-05,
-8.8724668e-05,
-8.4803293e-05,
-8.0419777e-05,
-7.6450076e-05,
-7.2019851e-05,
-6.8002059e-05,
-6.3525277e-05,
-5.9459603e-05,
-5.4936275e-05,
-5.0822955e-05,
-4.6253709e-05,
-4.2093251e-05,
-3.7478167e-05,
-3.3270757e-05,
-2.8610229e-05,
-2.4356253e-05,
-1.9650468e-05,
-1.5350224e-05,
-1.0599611e-05,
-6.2534467e-06,
-1.4583822e-06,
2.9334904e-06,
7.7728388e-06,
1.2210225e-05,
1.7093545e-05,
2.1575821e-05,
2.6502448e-05,
3.1029370e-05,
3.5999259e-05,
4.0570450e-05,
4.5583132e-05,
5.0198282e-05,
5.5253479e-05,
5.9912150e-05,
6.5009419e-05,
6.9711246e-05,
7.4850290e-05,
7.9595042e-05,
8.4775451e-05,
8.9562783e-05,
9.4783972e-05,
9.9613151e-05,
1.0487478e-04,
1.0974584e-04,
1.1504755e-04,
1.1995994e-04,
1.2530132e-04,
1.3025472e-04,
1.3563531e-04,
1.4062917e-04,
1.4604858e-04,
1.5108251e-04,
1.5654042e-04,
1.6161408e-04,
1.6710987e-04,
1.7222272e-04,
1.7775579e-04,
1.8290738e-04,
1.8847767e-04,
1.9366784e-04,
1.9927461e-04,
2.0450275e-04,
2.1014564e-04,
2.1541130e-04,
2.2108972e-04,
2.2639238e-04,
2.3210595e-04,
2.3744522e-04,
2.4319354e-04,
2.4856886e-04,
2.5435130e-04,
2.5976213e-04,
2.6557819e-04,
2.7102435e-04,
2.7687386e-04,
2.8235462e-04,
2.8823676e-04,
2.9375163e-04,
2.9966598e-04,
3.0521440e-04,
3.1116039e-04,
3.1674194e-04,
3.2271916e-04,
3.2833337e-04,
3.3434132e-04,
3.3998746e-04,
3.4602561e-04,
3.5170323e-04,
3.5777110e-04,
3.6347996e-04,
3.6957697e-04,
3.7531606e-04,
3.8144161e-04,
3.8721059e-04,
3.9336413e-04,
3.9916241e-04,
4.0534349e-04,
4.1117063e-04,
4.1737874e-04,
4.2323414e-04,
4.2946865e-04,
4.3535160e-04,
4.4161204e-04,
4.4752205e-04,
4.5380791e-04,
4.5974444e-04,
4.6605481e-04,
4.7201715e-04,
4.7835170e-04,
4.8433942e-04,
4.9069750e-04,
4.9671003e-04,
5.0309114e-04,
5.0912794e-04,
5.1553144e-04,
5.2159182e-04,
5.2801697e-04,
5.3410035e-04,
5.4054667e-04,
5.4665242e-04,
5.5311939e-04,
5.5924676e-04,
5.6573352e-04,
5.7188220e-04,
5.7838840e-04,
5.8455767e-04,
5.9108258e-04,
5.9727190e-04,
6.0381488e-04,
6.1002360e-04,
6.1658393e-04,
6.2281137e-04,
6.2938834e-04,
6.3563400e-04,
6.4222706e-04,
6.4849027e-04,
6.5509885e-04,
6.6137885e-04,
6.6800232e-04,
6.7429875e-04,
6.8093646e-04,
6.8724848e-04,
6.9389973e-04,
7.0022676e-04,
7.0689083e-04,
7.1323216e-04,
7.1990837e-04,
7.2626338e-04,
7.3295115e-04,
7.3931930e-04,
7.4601795e-04,
7.5239860e-04,
7.5910745e-04,
7.6549986e-04,
7.7221840e-04,
7.7862190e-04,
7.8534925e-04,
7.9176314e-04,
7.9849871e-04,
8.0492231e-04,
8.1166541e-04,
8.1809804e-04,
8.2484801e-04,
8.3128905e-04,
8.3804537e-04,
8.4449417e-04,
8.5125605e-04,
8.5771195e-04,
8.6447864e-04,
8.7094085e-04,
8.7771179e-04,
8.8417952e-04,
8.9095392e-04,
8.9742662e-04,
9.0420390e-04,
9.1068084e-04,
9.1746032e-04,
9.2394080e-04,
9.3072179e-04,
9.3720521e-04,
9.4398709e-04,
9.5047270e-04,
9.5725466e-04,
9.6374170e-04,
9.7052299e-04,
9.7701075e-04,
9.8379075e-04,
9.9027853e-04,
9.9705665e-04,
1.0035438e-03,
1.0103194e-03,
1.0168052e-03,
1.0235774e-03,
1.0300612e-03,
1.0368294e-03,
1.0433104e-03,
1.0500739e-03,
1.0565515e-03,
1.0633094e-03,
1.0697828e-03,
1.0765344e-03,
1.0830031e-03,
1.0897478e-03,
1.0962111e-03,
1.1029483e-03,
1.1094053e-03,
1.1161343e-03,
1.1225844e-03,
1.1293042e-03,
1.1357467e-03,
1.1424568e-03,
1.1488910e-03,
1.1555907e-03,
1.1620157e-03,
1.1687042e-03,
1.1751196e-03,
1.1817961e-03,
1.1882013e-03,
1.1948653e-03,
1.2012594e-03,
1.2079101e-03,
1.2142924e-03,
1.2209289e-03,
1.2272987e-03,
1.2339205e-03,
1.2402772e-03,
1.2468835e-03,
1.2532263e-03,
1.2598164e-03,
1.2661446e-03,
1.2727179e-03,
1.2790309e-03,
1.2855867e-03,
1.2918838e-03,
1.2984212e-03,
1.3047016e-03,
1.3112200e-03,
1.3174830e-03,
1.3239816e-03,
1.3302266e-03,
1.3367049e-03,
1.3429312e-03,
1.3493884e-03,
1.3555953e-03,
1.3620306e-03,
1.3682173e-03,
1.3746302e-03,
1.3807962e-03,
1.3871861e-03,
1.3933304e-03,
1.3996962e-03,
1.4058182e-03,
1.4121596e-03,
1.4182587e-03,
1.4245749e-03,
1.4306504e-03,
1.4369408e-03,
1.4429920e-03,
1.4492558e-03,
1.4552820e-03,
1.4615185e-03,
1.4675189e-03,
1.4737276e-03,
1.4797017e-03,
1.4858819e-03,
1.4918287e-03,
1.4979795e-03,
1.5038987e-03,
1.5100196e-03,
1.5159105e-03,
1.5220009e-03,
1.5278627e-03,
1.5339219e-03,
1.5397538e-03,
1.5457810e-03,
1.5515824e-03,
1.5575771e-03,
1.5633475e-03,
1.5693090e-03,
1.5750476e-03,
1.5809752e-03,
1.5866813e-03,
1.5925744e-03,
1.5982476e-03,
1.6041056e-03,
1.6097451e-03,
1.6155672e-03,
1.6211724e-03,
1.6269579e-03,
1.6325280e-03,
1.6382763e-03,
1.6438110e-03,
1.6495215e-03,
1.6550201e-03,
1.6606922e-03,
1.6661539e-03,
1.6717868e-03,
1.6772110e-03,
1.6828045e-03,
1.6881907e-03,
1.6937437e-03,
1.6990911e-03,
1.7046031e-03,
1.7099112e-03,
1.7153816e-03,
1.7206499e-03,
1.7260780e-03,
1.7313061e-03,
1.7366914e-03,
1.7418784e-03,
1.7472202e-03,
1.7523654e-03,
1.7576632e-03,
1.7627663e-03,
1.7680195e-03,
1.7730796e-03,
1.7782875e-03,
1.7833042e-03,
1.7884663e-03,
1.7934390e-03,
1.7985547e-03,
1.8034830e-03,
1.8085518e-03,
1.8134349e-03,
1.8184562e-03,
1.8232935e-03,
1.8282666e-03,
1.8330576e-03,
1.8379821e-03,
1.8427264e-03,
1.8476016e-03,
1.8522984e-03,
1.8571239e-03,
1.8617729e-03,
1.8665482e-03,
1.8711486e-03,
1.8758732e-03,
1.8804246e-03,
1.8850978e-03,
1.8895996e-03,
1.8942209e-03,
1.8986724e-03,
1.9032413e-03,
1.9076423e-03,
1.9121585e-03,
1.9165083e-03,
1.9209711e-03,
1.9252692e-03,
1.9296783e-03,
1.9339242e-03,
1.9382789e-03,
1.9424721e-03,
1.9467719e-03,
1.9509119e-03,
1.9551564e-03,
1.9592427e-03,
1.9634312e-03,
1.9674634e-03,
1.9715957e-03,
1.9755734e-03,
1.9796491e-03,
1.9835716e-03,
1.9875900e-03,
1.9914571e-03,
1.9954178e-03,
1.9992288e-03,
2.0031313e-03,
2.0068858e-03,
2.0107298e-03,
2.0144274e-03,
2.0182124e-03,
2.0218527e-03,
2.0255783e-03,
2.0291609e-03,
2.0328266e-03,
2.0363513e-03,
2.0399566e-03,
2.0434226e-03,
2.0469672e-03,
2.0503743e-03,
2.0538576e-03,
2.0572053e-03,
2.0606271e-03,
2.0639152e-03,
2.0672750e-03,
2.0705030e-03,
2.0738006e-03,
2.0769681e-03,
2.0802030e-03,
2.0833097e-03,
2.0864814e-03,
2.0895270e-03,
2.0926351e-03,
2.0956191e-03,
2.0986634e-03,
2.1015856e-03,
2.1045655e-03,
2.1074255e-03,
2.1103410e-03,
2.1131386e-03,
2.1159892e-03,
2.1187238e-03,
2.1215092e-03,
2.1241807e-03,
2.1269005e-03,
2.1295084e-03,
2.1321624e-03,
2.1347064e-03,
2.1372942e-03,
2.1397741e-03,
2.1422956e-03,
2.1447110e-03,
2.1471657e-03,
2.1495166e-03,
2.1519044e-03,
2.1541903e-03,
2.1565107e-03,
2.1587312e-03,
2.1609841e-03,
2.1631390e-03,
2.1653242e-03,
2.1674133e-03,
2.1695304e-03,
2.1715534e-03,
2.1736023e-03,
2.1755590e-03,
2.1775396e-03,
2.1794297e-03,
2.1813416e-03,
2.1831650e-03,
2.1850079e-03,
2.1867642e-03,
2.1885379e-03,
2.1902269e-03,
2.1919314e-03,
2.1935530e-03,
2.1951881e-03,
2.1967420e-03,
2.1983074e-03,
2.1997935e-03,
2.2012891e-03,
2.2027072e-03,
2.2041328e-03,
2.2054828e-03,
2.2068382e-03,
2.2081197e-03,
2.2094046e-03,
2.2106177e-03,
2.2118321e-03,
2.2129766e-03,
2.2141205e-03,
2.2151963e-03,
2.2162694e-03,
2.2172763e-03,
2.2182784e-03,
2.2192163e-03,
2.2201474e-03,
2.2210162e-03,
2.2218762e-03,
2.2226758e-03,
2.2234645e-03,
2.2241948e-03,
2.2249121e-03,
2.2255731e-03,
2.2262189e-03,
2.2268107e-03,
2.2273850e-03,
2.2279073e-03,
2.2284099e-03,
2.2288626e-03,
2.2292935e-03,
2.2296765e-03,
2.2300358e-03,
2.2303493e-03,
2.2306368e-03,
2.2308806e-03,
2.2310962e-03,
2.2312704e-03,
2.2314142e-03,
2.2315187e-03,
2.2315908e-03,
2.2316256e-03,
2.2316256e-03,
2.2315908e-03,
2.2315187e-03,
2.2314142e-03,
2.2312704e-03,
2.2310962e-03,
2.2308806e-03,
2.2306368e-03,
2.2303493e-03,
2.2300358e-03,
2.2296765e-03,
2.2292935e-03,
2.2288626e-03,
2.2284099e-03,
2.2279073e-03,
2.2273850e-03,
2.2268107e-03,
2.2262189e-03,
2.2255731e-03,
2.2249121e-03,
2.2241948e-03,
2.2234645e-03,
2.2226758e-03,
2.2218762e-03,
2.2210162e-03,
2.2201474e-03,
2.2192163e-03,
2.2182784e-03,
2.2172763e-03,
2.2162694e-03,
2.2151963e-03,
2.2141205e-03,
2.2129766e-03,
2.2118321e-03,
2.2106177e-03,
2.2094046e-03,
2.2081197e-03,
2.2068382e-03,
2.2054828e-03,
2.2041328e-03,
2.2027072e-03,
2.2012891e-03,
2.1997935e-03,
2.1983074e-03,
2.1967420e-03,
2.1951881e-03,
2.1935530e-03,
2.1919314e-03,
2.1902269e-03,
2.1885379e-03,
2.1867642e-03,
2.1850079e-03,
2.1831650e-03,
2.1813416e-03,
2.1794297e-03,
2.1775396e-03,
2.1755590e-03,
2.1736023e-03,
2.1715534e-03,
2.1695304e-03,
2.1674133e-03,
2.1653242e-03,
2.1631390e-03,
2.1609841e-03,
2.1587312e-03,
2.1565107e-03,
2.1541903e-03,
2.1519044e-03,
2.1495166e-03,
2.1471657e-03,
2.1447110e-03,
2.1422956e-03,
2.1397741e-03,
2.1372942e-03,
2.1347064e-03,
2.1321624e-03,
2.1295084e-03,
2.1269005e-03,
2.1241807e-03,
2.1215092e-03,
2.1187238e-03,
2.1159892e-03,
2.1131386e-03,
2.1103410e-03,
2.1074255e-03,
2.1045655e-03,
2.1015856e-03,
2.0986634e-03,
2.0956191e-03,
2.0926351e-03,
2.0895270e-03,
2.0864814e-03,
2.0833097e-03,
2.0802030e-03,
2.0769681e-03,
2.0738006e-03,
2.0705030e-03,
2.0672750e-03,
2.0639152e-03,
2.0606271e-03,
2.0572053e-03,
2.0538576e-03,
2.0503743e-03,
2.0469672e-03,
2.0434226e-03,
2.0399566e-03,
2.0363513e-03,
2.0328266e-03,
2.0291609e-03,
2.0255783e-03,
2.0218527e-03,
2.0182124e-03,
2.0144274e-03,
2.0107298e-03,
2.0068858e-03,
2.0031313e-03,
1.9992288e-03,
1.9954178e-03,
1.9914571e-03,
1.9875900e-03,
1.9835716e-03,
1.9796491e-03,
1.9755734e-03,
1.9715957e-03,
1.9674634e-03,
1.9634312e-03,
1.9592427e-03,
1.9551564e-03,
1.9509119e-03,
1.9467719e-03,
1.9424721e-03,
1.9382789e-03,
1.9339242e-03,
1.9296783e-03,
1.9252692e-03,
1.9209711e-03,
1.9165083e-03,
1.9121585e-03,
1.9076423e-03,
1.9032413e-03,
1.8986724e-03,
1.8942209e-03,
1.8895996e-03,
1.8850978e-03,
1.8804246e-03,
1.8758732e-03,
1.8711486e-03,
1.8665482e-03,
1.8617729e-03,
1.8571239e-03,
1.8522984e-03,
1.8476016e-03,
1.8427264e-03,
1.8379821e-03,
1.8330576e-03,
1.8282666e-03,
1.8232935e-03,
1.8184562e-03,
1.8134349e-03,
1.8085518e-03,
1.8034830e-03,
1.7985547e-03,
1.7934390e-03,
1.7884663e-03,
1.7833042e-03,
1.7782875e-03,
1.7730796e-03,
1.7680195e-03,
1.7627663e-03,
1.7576632e-03,
1.7523654e-03,
1.7472202e-03,
1.7418784e-03,
1.7366914e-03,
1.7313061e-03,
1.7260780e-03,
1.7206499e-03,
1.7153816e-03,
1.7099112e-03,
1.7046031e-03,
1.6990911e-03,
1.6937437e-03,
1.6881907e-03,
1.6828045e-03,
1.6772110e-03,
1.6717868e-03,
1.6661539e-03,
1.6606922e-03,
1.6550201e-03,
1.6495215e-03,
1.6438110e-03,
1.6382763e-03,
1.6325280e-03,
1.6269579e-03,
1.6211724e-03,
1.6155672e-03,
1.6097451e-03,
1.6041056e-03,
1.5982476e-03,
1.5925744e-03,
1.5866813e-03,
1.5809752e-03,
1.5750476e-03,
1.5693090e-03,
1.5633475e-03,
1.5575771e-03,
1.5515824e-03,
1.5457810e-03,
1.5397538e-03,
1.5339219e-03,
1.5278627e-03,
1.5220009e-03,
1.5159105e-03,
1.5100196e-03,
1.5038987e-03,
1.4979795e-03,
1.4918287e-03,
1.4858819e-03,
1.4797017e-03,
1.4737276e-03,
1.4675189e-03,
1.4615185e-03,
1.4552820e-03,
1.4492558e-03,
1.4429920e-03,
1.4369408e-03,
1.4306504e-03,
1.4245749e-03,
1.4182587e-03,
1.4121596e-03,
1.4058182e-03,
1.3996962e-03,
1.3933304e-03,
1.3871861e-03,
1.3807962e-03,
1.3746302e-03,
1.3682173e-03,
1.3620306e-03,
1.3555953e-03,
1.3493884e-03,
1.3429312e-03,
1.3367049e-03,
1.3302266e-03,
1.3239816e-03,
1.3174830e-03,
1.3112200e-03,
1.3047016e-03,
1.2984212e-03,
1.2918838e-03,
1.2855867e-03,
1.2790309e-03,
1.2727179e-03,
1.2661446e-03,
1.2598164e-03,
1.2532263e-03,
1.2468835e-03,
1.2402772e-03,
1.2339205e-03,
1.2272987e-03,
1.2209289e-03,
1.2142924e-03,
1.2079101e-03,
1.2012594e-03,
1.1948653e-03,
1.1882013e-03,
1.1817961e-03,
1.1751196e-03,
1.1687042e-03,
1.1620157e-03,
1.1555907e-03,
1.1488910e-03,
1.1424568e-03,
1.1357467e-03,
1.1293042e-03,
1.1225844e-03,
1.1161343e-03,
1.1094053e-03,
1.1029483e-03,
1.0962111e-03,
1.0897478e-03,
1.0830031e-03,
1.0765344e-03,
1.0697828e-03,
1.0633094e-03,
1.0565515e-03,
1.0500739e-03,
1.0433104e-03,
1.0368294e-03,
1.0300612e-03,
1.0235774e-03,
1.0168052e-03,
1.0103194e-03,
1.0035438e-03,
9.9705665e-04,
9.9027853e-04,
9.8379075e-04,
9.7701075e-04,
9.7052299e-04,
9.6374170e-04,
9.5725466e-04,
9.5047270e-04,
9.4398709e-04,
9.3720521e-04,
9.3072179e-04,
9.2394080e-04,
9.1746032e-04,
9.1068084e-04,
9.0420390e-04,
8.9742662e-04,
8.9095392e-04,
8.8417952e-04,
8.7771179e-04,
8.7094085e-04,
8.6447864e-04,
8.5771195e-04,
8.5125605e-04,
8.4449417e-04,
8.3804537e-04,
8.3128905e-04,
8.2484801e-04,
8.1809804e-04,
8.1166541e-04,
8.0492231e-04,
7.9849871e-04,
7.9176314e-04,
7.8534925e-04,
7.7862190e-04,
7.7221840e-04,
7.6549986e-04,
7.5910745e-04,
7.5239860e-04,
7.4601795e-04,
7.3931930e-04,
7.3295115e-04,
7.2626338e-04,
7.1990837e-04,
7.1323216e-04,
7.0689083e-04,
7.0022676e-04,
6.9389973e-04,
6.8724848e-04,
6.8093646e-04,
6.7429875e-04,
6.6800232e-04,
6.6137885e-04,
6.5509885e-04,
6.4849027e-04,
6.4222706e-04,
6.3563400e-04,
6.2938834e-04,
6.2281137e-04,
6.1658393e-04,
6.1002360e-04,
6.0381488e-04,
5.9727190e-04,
5.9108258e-04,
5.8455767e-04,
5.7838840e-04,
5.7188220e-04,
5.6573352e-04,
5.5924676e-04,
5.5311939e-04,
5.4665242e-04,
5.4054667e-04,
5.3410035e-04,
5.2801697e-04,
5.2159182e-04,
5.1553144e-04,
5.0912794e-04,
5.0309114e-04,
4.9671003e-04,
4.9069750e-04,
4.8433942e-04,
4.7835170e-04,
4.7201715e-04,
4.6605481e-04,
4.5974444e-04,
4.5380791e-04,
4.4752205e-04,
4.4161204e-04,
4.3535160e-04,
4.2946865e-04,
4.2323414e-04,
4.1737874e-04,
4.1117063e-04,
4.0534349e-04,
3.9916241e-04,
3.9336413e-04,
3.8721059e-04,
3.8144161e-04,
3.7531606e-04,
3.6957697e-04,
3.6347996e-04,
3.5777110e-04,
3.5170323e-04,
3.4602561e-04,
3.3998746e-04,
3.3434132e-04,
3.2833337e-04,
3.2271916e-04,
3.1674194e-04,
3.1116039e-04,
3.0521440e-04,
2.9966598e-04,
2.9375163e-04,
2.8823676e-04,
2.8235462e-04,
2.7687386e-04,
2.7102435e-04,
2.6557819e-04,
2.5976213e-04,
2.5435130e-04,
2.4856886e-04,
2.4319354e-04,
2.3744522e-04,
2.3210595e-04,
2.2639238e-04,
2.2108972e-04,
2.1541130e-04,
2.1014564e-04,
2.0450275e-04,
1.9927461e-04,
1.9366784e-04,
1.8847767e-04,
1.8290738e-04,
1.7775579e-04,
1.7222272e-04,
1.6710987e-04,
1.6161408e-04,
1.5654042e-04,
1.5108251e-04,
1.4604858e-04,
1.4062917e-04,
1.3563531e-04,
1.3025472e-04,
1.2530132e-04,
1.1995994e-04,
1.1504755e-04,
1.0974584e-04,
1.0487478e-04,
9.9613151e-05,
9.4783972e-05,
8.9562783e-05,
8.4775451e-05,
7.9595042e-05,
7.4850290e-05,
6.9711246e-05,
6.5009419e-05,
5.9912150e-05,
5.5253479e-05,
5.0198282e-05,
4.5583132e-05,
4.0570450e-05,
3.5999259e-05,
3.1029370e-05,
2.6502448e-05,
2.1575821e-05,
1.7093545e-05,
1.2210225e-05,
7.7728388e-06,
2.9334904e-06,
-1.4583822e-06,
-6.2534467e-06,
-1.0599611e-05,
-1.5350224e-05,
-1.9650468e-05,
-2.4356253e-05,
-2.8610229e-05,
-3.3270757e-05,
-3.7478167e-05,
-4.2093251e-05,
-4.6253709e-05,
-5.0822955e-05,
-5.4936275e-05,
-5.9459603e-05,
-6.3525277e-05,
-6.8002059e-05,
-7.2019851e-05,
-7.6450076e-05,
-8.0419777e-05,
-8.4803293e-05,
-8.8724668e-05,
-9.3061040e-05,
-9.6933726e-05,
-1.0122266e-04,
-1.0504646e-04,
-1.0928774e-04,
-1.1306230e-04,
-1.1725573e-04,
-1.2098094e-04,
-1.2512624e-04,
-1.2880165e-04,
-1.3289854e-04,
-1.3652418e-04,
-1.4057263e-04,
-1.4414820e-04,
-1.4814795e-04,
-1.5167319e-04,
-1.5562390e-04,
-1.5909853e-04,
-1.6300004e-04,
-1.6642395e-04,
-1.7027599e-04,
-1.7364901e-04,
-1.7745155e-04,
-1.8077352e-04,
-1.8452622e-04,
-1.8779688e-04,
-1.9149975e-04,
-1.9471910e-04,
-1.9837186e-04,
-2.0153964e-04,
-2.0514206e-04,
-2.0825800e-04,
-2.1180995e-04,
-2.1487400e-04,
-2.1837533e-04,
-2.2138743e-04,
-2.2483794e-04,
-2.2779807e-04,
-2.3119771e-04,
-2.3410565e-04,
-2.3745420e-04,
-2.4030982e-04,
-2.4360730e-04,
-2.4641049e-04,
-2.4965656e-04,
-2.5240721e-04,
-2.5560184e-04,
-2.5829978e-04,
-2.6144296e-04,
-2.6408826e-04,
-2.6717989e-04,
-2.6977244e-04,
-2.7281242e-04,
-2.7535224e-04,
-2.7834040e-04,
-2.8082735e-04,
-2.8376358e-04,
-2.8619761e-04,
-2.8908190e-04,
-2.9146288e-04,
-2.9429514e-04,
-2.9662313e-04,
-2.9940336e-04,
-3.0167831e-04,
-3.0440657e-04,
-3.0662845e-04,
-3.0930472e-04,
-3.1147342e-04,
-3.1409765e-04,
-3.1621315e-04,
-3.1878521e-04,
-3.2084751e-04,
-3.2336737e-04,
-3.2537654e-04,
-3.2784426e-04,
-3.2980023e-04,
-3.3221595e-04,
-3.3411882e-04,
-3.3648241e-04,
-3.3833221e-04,
-3.4064378e-04,
-3.4244039e-04,
-3.4469998e-04,
-3.4644343e-04,
-3.4865102e-04,
-3.5034128e-04,
-3.5249692e-04,
-3.5413406e-04,
-3.5623784e-04,
-3.5782210e-04,
-3.5987412e-04,
-3.6140546e-04,
-3.6340582e-04,
-3.6488421e-04,
-3.6683285e-04,
-3.6825846e-04,
-3.7015548e-04,
-3.7152831e-04,
-3.7337379e-04,
-3.7469396e-04,
-3.7648800e-04,
-3.7775556e-04,
-3.7949829e-04,
-3.8071338e-04,
-3.8240501e-04,
-3.8356791e-04,
-3.8520840e-04,
-3.8631909e-04,
-3.8790847e-04,
-3.8896694e-04,
-3.9050543e-04,
-3.9151195e-04,
-3.9299963e-04,
-3.9395438e-04,
-3.9539138e-04,
-3.9629446e-04,
-3.9768098e-04,
-3.9853255e-04,
-3.9986869e-04,
-4.0066897e-04,
-4.0195503e-04,
-4.0270414e-04,
-4.0393995e-04,
-4.0463806e-04,
-4.0582375e-04,
-4.0647104e-04,
-4.0760705e-04,
-4.0820380e-04,
-4.0929017e-04,
-4.0983658e-04,
-4.1087350e-04,
-4.1136968e-04,
-4.1235738e-04,
-4.1280355e-04,
-4.1374219e-04,
-4.1413864e-04,
-4.1502846e-04,
-4.1537514e-04,
-4.1621619e-04,
-4.1651350e-04,
-4.1730610e-04,
-4.1755443e-04,
-4.1829887e-04,
-4.1849834e-04,
-4.1919475e-04,
-4.1934556e-04,
-4.1999414e-04,
-4.2009657e-04,
-4.2069756e-04,
-4.2075181e-04,
-4.2130558e-04,
-4.2131195e-04,
-4.2181859e-04,
-4.2177708e-04,
-4.2223705e-04,
-4.2214818e-04,
-4.2256181e-04,
-4.2242593e-04,
-4.2279328e-04,
-4.2261041e-04,
-4.2293180e-04,
-4.2270227e-04,
-4.2297797e-04,
-4.2270218e-04,
-4.2293246e-04,
-4.2261065e-04,
-4.2279597e-04,
-4.2242837e-04,
-4.2256876e-04,
-4.2215568e-04,
-4.2225181e-04,
-4.2179364e-04,
-4.2184572e-04,
-4.2134264e-04,
-4.2135076e-04,
-4.2080300e-04,
-4.2076763e-04,
-4.2017564e-04,
-4.2009716e-04,
-4.1946129e-04,
-4.1933998e-04,
-4.1866063e-04,
-4.1849679e-04,
-4.1777407e-04,
-4.1756797e-04,
-4.1680251e-04,
-4.1655470e-04,
-4.1574664e-04,
-4.1545728e-04,
-4.1460688e-04,
-4.1427627e-04,
-4.1338407e-04,
-4.1301270e-04,
-4.1207911e-04,
-4.1166738e-04,
-4.1069267e-04,
-4.1024086e-04,
-4.0922551e-04,
-4.0873377e-04,
-4.0767796e-04,
-4.0714694e-04,
-4.0605135e-04,
-4.0548128e-04,
-4.0434600e-04,
-4.0373723e-04,
-4.0256273e-04,
-4.0191575e-04,
-4.0070265e-04,
-4.0001781e-04,
-3.9876639e-04,
-3.9804400e-04,
-3.9675462e-04,
-3.9599508e-04,
-3.9466813e-04,
-3.9387165e-04,
-3.9250763e-04,
-3.9167496e-04,
-3.9027433e-04,
-3.8940555e-04,
-3.8796859e-04,
-3.8706430e-04,
-3.8559153e-04,
-3.8465223e-04,
-3.8314407e-04,
-3.8217002e-04,
-3.8062678e-04,
-3.7961833e-04,
-3.7804051e-04,
-3.7699817e-04,
-3.7538608e-04,
-3.7431029e-04,
-3.7266472e-04,
-3.7155594e-04,
-3.6987705e-04,
-3.6873554e-04,
-3.6702391e-04,
-3.6585029e-04,
-3.6410627e-04,
-3.6290100e-04,
-3.6112490e-04,
-3.5988832e-04,
-3.5808062e-04,
-3.5681322e-04,
-3.5497446e-04,
-3.5367676e-04,
-3.5180730e-04,
-3.5047988e-04,
-3.4858039e-04,
-3.4722346e-04,
-3.4529408e-04,
-3.4390816e-04,
-3.4194962e-04,
-3.4053515e-04,
-3.3854774e-04,
-3.3710521e-04,
-3.3508935e-04,
-3.3361924e-04,
-3.3157552e-04,
-3.3007841e-04,
-3.2800725e-04,
-3.2648351e-04,
-3.2438545e-04,
-3.2283572e-04,
-3.2071106e-04,
-3.1913553e-04,
-3.1698472e-04,
-3.1538404e-04,
-3.1320772e-04,
-3.1158221e-04,
-3.0938086e-04,
-3.0773108e-04,
-3.0550516e-04,
-3.0383168e-04,
-3.0158172e-04,
-2.9988496e-04,
-2.9761135e-04,
-2.9589171e-04,
-2.9359508e-04,
-2.9185308e-04,
-2.8953364e-04,
-2.8776966e-04,
-2.8542814e-04,
-2.8364281e-04,
-2.8127965e-04,
-2.7947336e-04,
-2.7708923e-04,
-2.7526249e-04,
-2.7285777e-04,
-2.7101109e-04,
-2.6858623e-04,
-2.6671996e-04,
-2.6427543e-04,
-2.6239015e-04,
-2.5992654e-04,
-2.5802263e-04,
-2.5554036e-04,
-2.5361839e-04,
-2.5111815e-04,
-2.4917864e-04,
-2.4666078e-04,
-2.4470424e-04,
-2.4216943e-04,
-2.4019618e-04,
-2.3764480e-04,
-2.3565533e-04,
-2.3308780e-04,
-2.3108262e-04,
-2.2849956e-04,
-2.2647934e-04,
-2.2388119e-04,
-2.2184624e-04,
-2.1923355e-04,
-2.1718455e-04,
-2.1455785e-04,
-2.1249519e-04,
-2.0985488e-04,
-2.0777912e-04,
-2.0512569e-04,
-2.0303710e-04,
-2.0037110e-04,
-1.9827023e-04,
-1.9559215e-04,
-1.9347966e-04,
-1.9079011e-04,
-1.8866648e-04,
-1.8596579e-04,
-1.8383145e-04,
-1.8112015e-04,
-1.7897574e-04,
-1.7625428e-04,
-1.7410016e-04,
-1.7136904e-04,
-1.6920573e-04,
-1.6646527e-04,
-1.6429327e-04,
-1.6154421e-04,
-1.5936406e-04,
-1.5660691e-04,
-1.5441917e-04,
-1.5165438e-04,
-1.4945934e-04,
-1.4668730e-04,
-1.4448554e-04,
-1.4170678e-04,
-1.3949884e-04,
-1.3671384e-04,
-1.3450012e-04,
-1.3170934e-04,
-1.2949035e-04,
-1.2669419e-04,
-1.2447061e-04,
-1.2166975e-04,
-1.1944196e-04,
-1.1663689e-04,
-1.1440526e-04,
-1.1159629e-04,
-1.0936121e-04,
-1.0654888e-04,
-1.0431096e-04,
-1.0149590e-04,
-9.9255574e-05,
-9.6438156e-05,
-9.4195879e-05,
-9.1376582e-05,
-8.9132793e-05,
-8.6312206e-05,
-8.4067572e-05,
-8.1246085e-05,
-7.9000865e-05,
-7.6178975e-05,
-7.3933491e-05,
-7.1111565e-05,
-6.8866411e-05,
-6.6045087e-05,
-6.3800809e-05,
-6.0980648e-05,
-5.8737580e-05,
-5.5918829e-05,
-5.3677438e-05,
-5.0860667e-05,
-4.8621403e-05,
-4.5807210e-05,
-4.3570544e-05,
-4.0759111e-05,
-3.8525413e-05,
-3.5717290e-05,
-3.3486955e-05,
-3.0682710e-05,
-2.8456315e-05,
-2.5656460e-05,
-2.3434477e-05,
-2.0639362e-05,
-1.8422025e-05,
-1.5631999e-05,
-1.3419841e-05,
-1.0635468e-05,
-8.4289994e-06,
-5.6506953e-06,
-3.4502089e-06,
-6.7823824e-07,
1.5156816e-06,
4.2808140e-06,
6.4677293e-06,
9.2254151e-06,
1.1404963e-05,
1.4154851e-05,
1.6326638e-05,
1.9068520e-05,
2.1232257e-05,
2.3965508e-05,
2.6120595e-05,
2.8844828e-05,
3.0990833e-05,
3.3705738e-05,
3.5842405e-05,
3.8547508e-05,
4.0674204e-05,
4.3369180e-05,
4.5485482e-05,
4.8169907e-05,
5.0275556e-05,
5.2949072e-05,
5.5043773e-05,
5.7706104e-05,
5.9789327e-05,
6.2439757e-05,
6.4511043e-05,
6.7149369e-05,
6.9208471e-05,
7.1834336e-05,
7.3880893e-05,
7.6493755e-05,
7.8527328e-05,
8.1126981e-05,
8.3147165e-05,
8.5733324e-05,
8.7739769e-05,
9.0311996e-05,
9.2304496e-05,
9.4862349e-05,
9.6840217e-05,
9.9383226e-05,
1.0134632e-04,
1.0387430e-04,
1.0582238e-04,
1.0833494e-04,
1.1026755e-04,
1.1276423e-04,
1.1468116e-04,
1.1716168e-04,
1.1906254e-04,
1.2152659e-04,
1.2341095e-04,
1.2585817e-04,
1.2772583e-04,
1.3015569e-04,
1.3200615e-04,
1.3441851e-04,
1.3625176e-04,
1.3864629e-04,
1.4046191e-04,
1.4283827e-04,
1.4463588e-04,
1.4699372e-04,
1.4877315e-04,
1.5111206e-04,
1.5287293e-04,
1.5519261e-04,
1.5693461e-04,
1.5923491e-04,
1.6095770e-04,
1.6323811e-04,
1.6494156e-04,
1.6720207e-04,
1.6888594e-04,
1.7112606e-04,
1.7278993e-04,
1.7500939e-04,
1.7665302e-04,
1.7885155e-04,
1.8047465e-04,
1.8265189e-04,
1.8425419e-04,
1.8640997e-04,
1.8799132e-04,
1.9012550e-04,
1.9168539e-04,
1.9379765e-04,
1.9533613e-04,
1.9742631e-04,
1.9894288e-04,
2.0101052e-04,
2.0250500e-04,
2.0454990e-04,
2.0602208e-04,
2.0804407e-04,
2.0949364e-04,
2.1149246e-04,
2.1291932e-04,
2.1489486e-04,
2.1629884e-04,
2.1825076e-04,
2.1963140e-04,
2.2155965e-04,
2.2291687e-04,
2.2482111e-04,
2.2615447e-04,
2.2803451e-04,
2.2934402e-04,
2.3119965e-04,
2.3248506e-04,
2.3431618e-04,
2.3557731e-04,
2.3738371e-04,
2.3862052e-04,
2.4040199e-04,
2.4161415e-04,
2.4337025e-04,
2.4455762e-04,
2.4628849e-04,
2.4745082e-04,
2.4915606e-04,
2.5029322e-04,
2.5197288e-04,
2.5308476e-04,
2.5473858e-04,
2.5582501e-04,
2.5745288e-04,
2.5851377e-04,
2.6011554e-04,
2.6115075e-04,
2.6272606e-04,
2.6373525e-04,
2.6528395e-04,
2.6626731e-04,
2.6778942e-04,
2.6874665e-04,
2.7024192e-04,
2.7117305e-04,
2.7264148e-04,
2.7354625e-04,
2.7498761e-04,
2.7586603e-04,
2.7728018e-04,
2.7813210e-04,
2.7951900e-04,
2.8034419e-04,
2.8170347e-04,
2.8250188e-04,
2.8383373e-04,
2.8460555e-04,
2.8590975e-04,
2.8665471e-04,
2.8793121e-04,
2.8864928e-04,
2.8989790e-04,
2.9058886e-04,
2.9180960e-04,
2.9247353e-04,
2.9366624e-04,
2.9430299e-04,
2.9546761e-04,
2.9607702e-04,
2.9721347e-04,
2.9779574e-04,
2.9890417e-04,
2.9945922e-04,
3.0053932e-04,
3.0106697e-04,
3.0211882e-04,
3.0261897e-04,
3.0364238e-04,
3.0411505e-04,
3.0511024e-04,
3.0555536e-04,
3.0652216e-04,
3.0693972e-04,
3.0787807e-04,
3.0826800e-04,
3.0917811e-04,
3.0954059e-04,
3.1042232e-04,
3.1075708e-04,
3.1161023e-04,
3.1191735e-04,
3.1274203e-04,
3.1302148e-04,
3.1381769e-04,
3.1406961e-04,
3.1483751e-04,
3.1506167e-04,
3.1580116e-04,
3.1599771e-04,
3.1670876e-04,
3.1687770e-04,
3.1756060e-04,
3.1770189e-04,
3.1835637e-04,
3.1846988e-04,
3.1909596e-04,
3.1918199e-04,
3.1977977e-04,
3.1983838e-04,
3.2040797e-04,
3.2043923e-04,
3.2098049e-04,
3.2098427e-04,
3.2149736e-04,
3.2147379e-04,
3.2195872e-04,
3.2190796e-04,
3.2236482e-04,
3.2228672e-04,
3.2271544e-04,
3.2261009e-04,
3.2301090e-04,
3.2287862e-04,
3.2325155e-04,
3.2309238e-04,
3.2343746e-04,
3.2325138e-04,
3.2356842e-04,
3.2335558e-04,
3.2364492e-04,
3.2340537e-04,
3.2366708e-04,
3.2340108e-04,
3.2363505e-04,
3.2334253e-04,
3.2354899e-04,
3.2323015e-04,
3.2340934e-04,
3.2306432e-04,
3.2321620e-04,
3.2284505e-04,
3.2296961e-04,
3.2257235e-04,
3.2266970e-04,
3.2224669e-04,
3.2231709e-04,
3.2186833e-04,
3.2191194e-04,
3.2143760e-04,
3.2145434e-04,
3.2095453e-04,
3.2094467e-04,
3.2041959e-04,
3.2038334e-04,
3.1983298e-04,
3.1977030e-04,
3.1919482e-04,
3.1910585e-04,
3.1850544e-04,
3.1839053e-04,
3.1776550e-04,
3.1762464e-04,
3.1697504e-04,
3.1680852e-04,
3.1613434e-04,
3.1594226e-04,
3.1524378e-04,
3.1502629e-04,
3.1430371e-04,
3.1406103e-04,
3.1331434e-04,
3.1304658e-04,
3.1227606e-04,
3.1198352e-04,
3.1118945e-04,
3.1087238e-04,
3.1005487e-04,
3.0971319e-04,
3.0887246e-04,
3.0850645e-04,
3.0764252e-04,
3.0725242e-04,
3.0636561e-04,
3.0595159e-04,
3.0504213e-04,
3.0460438e-04,
3.0367234e-04,
3.0321118e-04,
3.0225686e-04,
3.0177253e-04,
3.0079615e-04,
3.0028873e-04,
2.9929034e-04,
2.9875998e-04,
2.9773995e-04,
2.9718679e-04,
2.9614537e-04,
2.9556974e-04,
2.9450717e-04,
2.9390936e-04,
2.9282588e-04,
2.9220591e-04,
2.9110175e-04,
2.9046003e-04,
2.8933540e-04,
2.8867206e-04,
2.8752712e-04,
2.8684232e-04,
2.8567727e-04,
2.8497137e-04,
2.8378660e-04,
2.8305967e-04,
2.8185550e-04,
2.8110791e-04,
2.7988459e-04,
2.7911661e-04,
2.7787429e-04,
2.7708588e-04,
2.7582493e-04,
2.7501650e-04,
2.7373710e-04,
2.7290879e-04,
2.7161117e-04,
2.7076333e-04,
2.6944785e-04,
2.6858082e-04,
2.6724768e-04,
2.6636162e-04,
2.6501117e-04,
2.6410640e-04,
2.6273892e-04,
2.6181557e-04,
2.6043110e-04,
2.5948943e-04,
2.5808846e-04,
2.5712876e-04,
2.5571147e-04,
2.5473408e-04,
2.5330082e-04,
2.5230607e-04,
2.5085713e-04,
2.4984522e-04,
2.4838073e-04,
2.4735199e-04,
2.4587234e-04,
2.4482700e-04,
2.4333241e-04,
2.4227059e-04,
2.4076124e-04,
2.3968353e-04,
2.3815987e-04,
2.3706639e-04,
2.3552869e-04,
2.3441990e-04,
2.3286846e-04,
2.3174455e-04,
2.3017950e-04,
2.2904072e-04,
2.2746234e-04,
2.2630910e-04,
2.2471780e-04,
2.2355027e-04,
2.2194618e-04,
2.2076462e-04,
2.1914825e-04,
2.1795323e-04,
2.1632483e-04,
2.1511642e-04,
2.1347633e-04,
2.1225488e-04,
2.1060331e-04,
2.0936899e-04,
2.0770614e-04,
2.0645936e-04,
2.0478566e-04,
2.0352685e-04,
2.0184256e-04,
2.0057185e-04,
1.9887719e-04,
1.9759501e-04,
1.9589064e-04,
1.9459725e-04,
1.9288319e-04,
1.9157879e-04,
1.8985546e-04,
1.8854025e-04,
1.8680787e-04,
1.8548218e-04,
1.8374124e-04,
1.8240550e-04,
1.8065634e-04,
1.7931086e-04,
1.7755356e-04,
1.7619853e-04,
1.7443358e-04,
1.7306950e-04,
1.7129729e-04,
1.6992420e-04,
1.6814480e-04,
1.6676313e-04,
1.6497696e-04,
1.6358686e-04,
1.6179431e-04,
1.6039629e-04,
1.5859789e-04,
1.5719215e-04,
1.5538806e-04,
1.5397483e-04,
1.5216516e-04,
1.5074484e-04,
1.4893022e-04,
1.4750318e-04,
1.4568368e-04,
1.4425001e-04,
1.4242601e-04,
1.4098621e-04,
1.3915801e-04,
1.3771236e-04,
1.3588043e-04,
1.3442937e-04,
1.3259396e-04,
1.3113758e-04,
1.2929889e-04,
1.2783739e-04,
1.2599576e-04,
1.2452979e-04,
1.2268564e-04,
1.2121537e-04,
1.1936876e-04,
1.1789445e-04,
1.1604597e-04,
1.1456802e-04,
1.1271783e-04,
1.1123663e-04,
1.0938516e-04,
1.0790092e-04,
1.0604820e-04,
1.0456116e-04,
1.0270761e-04,
1.0121805e-04,
9.9364170e-05,
9.7872702e-05,
9.6018576e-05,
9.4525264e-05,
9.2671176e-05,
9.1176466e-05,
8.9322871e-05,
8.7826997e-05,
8.5974085e-05,
8.4477438e-05,
8.2625524e-05,
8.1128167e-05,
7.9277452e-05,
7.7779868e-05,
7.5930831e-05,
7.4433344e-05,
7.2586463e-05,
7.1089279e-05,
6.9244521e-05,
6.7747881e-05,
6.5905671e-05,
6.4410079e-05,
6.2570647e-05,
6.1076227e-05,
5.9239940e-05,
5.7747033e-05,
5.5914076e-05,
5.4422860e-05,
5.2593723e-05,
5.1104656e-05,
4.9279614e-05,
4.7793003e-05,
4.5972398e-05,
4.4488113e-05,
4.2672111e-05,
4.1190653e-05,
3.9379677e-05,
3.7901416e-05,
3.6095650e-05,
3.4620749e-05,
3.2820609e-05,
3.1349360e-05,
2.9554969e-05,
2.8087752e-05,
2.6299628e-05,
2.4836645e-05,
2.3054894e-05,
2.1596452e-05,
1.9821287e-05,
1.8367367e-05,
1.6599234e-05,
1.5150342e-05,
1.3389588e-05,
1.1945942e-05,
1.0192766e-05,
8.7546590e-06,
7.0093348e-06,
5.5769553e-06,
3.8397062e-06,
2.4134832e-06,
6.8458348e-07,
-7.3541924e-07,
-2.4557436e-06,
-3.8691961e-06,
-5.5808143e-06,
-6.9873915e-06,
-8.6897835e-06,
-1.0089236e-05,
-1.1782165e-05,
-1.3174292e-05,
-1.4857607e-05,
-1.6242067e-05,
-1.7915600e-05,
-1.9292291e-05,
-2.0955678e-05,
-2.2324249e-05,
-2.3977418e-05,
-2.5337660e-05,
-2.6980190e-05,
-2.8331856e-05,
-2.9963643e-05,
-3.1306286e-05,
-3.2926963e-05,
-3.4260534e-05,
-3.5869937e-05,
-3.7194235e-05,
-3.8792196e-05,
-4.0106934e-05,
-4.1693271e-05,
-4.2998260e-05,
-4.4572623e-05,
-4.5867614e-05,
-4.7429882e-05,
-4.8714489e-05,
-5.0264323e-05,
-5.1538480e-05,
-5.3075726e-05,
-5.4339113e-05,
-5.5863583e-05,
-5.7116105e-05,
-5.8627609e-05,
-5.9869158e-05,
-6.1367390e-05,
-6.2597712e-05,
-6.4082480e-05,
-6.5301247e-05,
-6.6772324e-05,
-6.7979416e-05,
-6.9436527e-05,
-7.0631631e-05,
-7.2074731e-05,
-7.3257835e-05,
-7.4686657e-05,
-7.5857524e-05,
-7.7271953e-05,
-7.8430366e-05,
-7.9830183e-05,
-8.0976083e-05,
-8.2360903e-05,
-8.3494063e-05,
-8.4863716e-05,
-8.5983858e-05,
-8.7338202e-05,
-8.8445293e-05,
-8.9784065e-05,
-9.0878011e-05,
-9.2201265e-05,
-9.3281871e-05,
-9.4589235e-05,
-9.5656261e-05,
-9.6947662e-05,
-9.8000930e-05,
-9.9276210e-05,
-1.0031553e-04,
-1.0157440e-04,
-1.0259969e-04,
-1.0384198e-04,
-1.0485310e-04,
-1.0607872e-04,
-1.0707559e-04,
-1.0828445e-04,
-1.0926705e-04,
-1.1045896e-04,
-1.1142690e-04,
-1.1260155e-04,
-1.1355472e-04,
-1.1471212e-04,
-1.1565045e-04,
-1.1679049e-04,
-1.1771364e-04,
-1.1883621e-04,
-1.1974424e-04,
-1.2084913e-04,
-1.2174195e-04,
-1.2282914e-04,
-1.2370657e-04,
-1.2477603e-04,
-1.2563794e-04,
-1.2668929e-04,
-1.2753534e-04,
-1.2856851e-04,
-1.2939884e-04,
-1.3041387e-04,
-1.3122837e-04,
-1.3222509e-04,
-1.3302347e-04,
-1.3400190e-04,
-1.3478418e-04,
-1.3574412e-04,
-1.3651020e-04,
-1.3745162e-04,
-1.3820138e-04,
-1.3912411e-04,
-1.3985731e-04,
-1.4076113e-04,
-1.4147772e-04,
-1.4236279e-04,
-1.4306295e-04,
-1.4392917e-04,
-1.4461261e-04,
-1.4545977e-04,
-1.4612645e-04,
-1.4695462e-04,
-1.4760437e-04,
-1.4841342e-04,
-1.4904627e-04,
-1.4983608e-04,
-1.5045197e-04,
-1.5122231e-04,
-1.5182111e-04,
-1.5257205e-04,
-1.5315392e-04,
-1.5388561e-04,
-1.5445047e-04,
-1.5516255e-04,
-1.5571014e-04,
-1.5640256e-04,
-1.5693306e-04,
-1.5760588e-04,
-1.5811901e-04,
-1.5877219e-04,
-1.5926808e-04,
-1.5990150e-04,
-1.6038004e-04,
-1.6099359e-04,
-1.6145487e-04,
-1.6204867e-04,
-1.6249285e-04,
-1.6306682e-04,
-1.6349353e-04,
-1.6404735e-04,
-1.6445668e-04,
-1.6499053e-04,
-1.6538263e-04,
-1.6589644e-04,
-1.6627117e-04,
-1.6676503e-04,
-1.6712237e-04,
-1.6759619e-04,
-1.6793605e-04,
-1.6838992e-04,
-1.6871247e-04,
-1.6914636e-04,
-1.6945160e-04,
-1.6986529e-04,
-1.7015291e-04,
-1.7054645e-04,
-1.7081681e-04,
-1.7119041e-04,
-1.7144347e-04,
-1.7179694e-04,
-1.7203276e-04,
-1.7236623e-04,
-1.7258458e-04,
-1.7289800e-04,
-1.7309901e-04,
-1.7339255e-04,
-1.7357626e-04,
-1.7384986e-04,
-1.7401616e-04,
-1.7426967e-04,
-1.7441861e-04,
-1.7465241e-04,
-1.7478422e-04,
-1.7499826e-04,
-1.7511279e-04,
-1.7530697e-04,
-1.7540437e-04,
-1.7557875e-04,
-1.7565883e-04,
-1.7581348e-04,
-1.7587651e-04,
-1.7601156e-04,
-1.7605746e-04,
-1.7617294e-04,
-1.7620165e-04,
-1.7629753e-04,
-1.7630941e-04,
-1.7638602e-04,
-1.7638094e-04,
-1.7643805e-04,
-1.7641598e-04,
-1.7645374e-04,
-1.7641486e-04,
-1.7643321e-04,
-1.7637756e-04,
-1.7637678e-04,
-1.7630449e-04,
-1.7628458e-04,
-1.7619567e-04,
-1.7615672e-04,
-1.7605120e-04,
-1.7599330e-04,
-1.7587153e-04,
-1.7579467e-04,
-1.7565643e-04,
-1.7556057e-04,
-1.7540607e-04,
-1.7529147e-04,
-1.7512084e-04,
-1.7498741e-04,
-1.7480092e-04,
-1.7464898e-04,
-1.7444651e-04,
-1.7427599e-04,
-1.7405775e-04,
-1.7386872e-04,
-1.7363472e-04,
-1.7342744e-04,
-1.7317784e-04,
-1.7295213e-04,
-1.7268697e-04,
-1.7244310e-04,
-1.7216266e-04,
-1.7190076e-04,
-1.7160511e-04,
-1.7132522e-04,
-1.7101467e-04,
-1.7071686e-04,
-1.7039129e-04,
-1.7007562e-04,
-1.6973521e-04,
-1.6940179e-04,
-1.6904668e-04,
-1.6869585e-04,
-1.6832603e-04,
-1.6795771e-04,
-1.6757349e-04,
-1.6718803e-04,
-1.6678953e-04,
-1.6638694e-04,
-1.6597417e-04,
-1.6555468e-04,
-1.6512790e-04,
-1.6469134e-04,
-1.6425062e-04,
-1.6379731e-04,
-1.6334272e-04,
-1.6287281e-04,
-1.6240470e-04,
-1.6191843e-04,
-1.6143676e-04,
-1.6093427e-04,
-1.6043933e-04,
-1.5992074e-04,
-1.5941252e-04,
-1.5887795e-04,
-1.5835656e-04,
-1.5780633e-04,
-1.5727188e-04,
-1.5670590e-04,
-1.5615871e-04,
-1.5557731e-04,
-1.5501743e-04,
-1.5442082e-04,
-1.5384864e-04,
-1.5323690e-04,
-1.5265236e-04,
-1.5202575e-04,
-1.5142899e-04,
-1.5078749e-04,
-1.5017867e-04,
-1.4952254e-04,
-1.4890193e-04,
-1.4823142e-04,
-1.4759902e-04,
-1.4691433e-04,
-1.4627051e-04,
-1.4557170e-04,
-1.4491667e-04,
-1.4420405e-04,
-1.4353793e-04,
-1.4281148e-04,
-1.4213435e-04,
-1.4139434e-04,
-1.4070632e-04,
-1.3995287e-04,
-1.3925432e-04,
-1.3848772e-04,
-1.3777892e-04,
-1.3699926e-04,
-1.3628023e-04,
-1.3548784e-04,
-1.3475884e-04,
-1.3395370e-04,
-1.3321508e-04,
-1.3239746e-04,
-1.3164914e-04,
-1.3081902e-04,
-1.3006137e-04,
-1.2921906e-04,
-1.2845228e-04,
-1.2759813e-04,
-1.2682247e-04,
-1.2595661e-04,
-1.2517226e-04,
-1.2429474e-04,
-1.2350185e-04,
-1.2261300e-04,
-1.2181171e-04,
-1.2091169e-04,
-1.2010234e-04,
-1.1919123e-04,
-1.1837386e-04,
-1.1745181e-04,
-1.1662693e-04,
-1.1569428e-04,
-1.1486209e-04,
-1.1391913e-04,
-1.1307966e-04,
-1.1212643e-04,
-1.1127982e-04,
-1.1031653e-04,
-1.0946311e-04,
-1.0849001e-04,
-1.0762990e-04,
-1.0664738e-04,
-1.0578075e-04,
-1.0478874e-04,
-1.0391586e-04,
-1.0291482e-04,
-1.0203606e-04,
-1.0102617e-04,
-1.0014168e-04,
-9.9122994e-05,
-9.8232793e-05,
-9.7205532e-05,
-9.6309837e-05,
-9.5274318e-05,
-9.4373534e-05,
-9.3330008e-05,
-9.2424279e-05,
-9.1373014e-05,
-9.0462353e-05,
-8.9403463e-05,
-8.8488264e-05,
-8.7422266e-05,
-8.6502709e-05,
-8.5429547e-05,
-8.4505743e-05,
-8.3425521e-05,
-8.2497734e-05,
-8.1410849e-05,
-8.0479340e-05,
-7.9386093e-05,
-7.8451173e-05,
-7.7351751e-05,
-7.6413576e-05,
-7.5308031e-05,
-7.4366653e-05,
-7.3255397e-05,
-7.2311240e-05,
-7.1194530e-05,
-7.0247662e-05,
-6.9125441e-05,
-6.8176074e-05,
-6.7048696e-05,
-6.6097253e-05,
-6.4965027e-05,
-6.4011692e-05,
-6.2874830e-05,
-6.1919776e-05,
-6.0778379e-05,
-5.9821775e-05,
-5.8675950e-05,
-5.7718057e-05,
-5.6568290e-05,
-5.5609458e-05,
-5.4455750e-05,
-5.3495996e-05,
-5.2338578e-05,
-5.1378303e-05,
-5.0217538e-05,
-4.9257066e-05,
-4.8093022e-05,
-4.7132491e-05,
-4.5965383e-05,
-4.5004913e-05,
-4.3834832e-05,
-4.2874683e-05,
-4.1701992e-05,
-4.0742553e-05,
-3.9567550e-05,
-3.8608935e-05,
-3.7431627e-05,
-3.6473921e-05,
-3.5294779e-05,
-3.4338444e-05,
-3.3157572e-05,
-3.2202709e-05,
-3.1020152e-05,
-3.0067014e-05,
-2.8883050e-05,
-2.7931766e-05,
-2.6746584e-05,
-2.5797543e-05,
-2.4611557e-05,
-2.3665044e-05,
-2.2478343e-05,
-2.1534274e-05,
-2.0347014e-05,
-1.9405703e-05,
-1.8218350e-05,
-1.7280028e-05,
-1.6092582e-05,
-1.5157301e-05,
-1.3970021e-05,
-1.3038162e-05,
-1.1851264e-05,
-1.0922955e-05,
-9.7367643e-06,
-8.8123709e-06,
-7.6270573e-06,
-6.7067090e-06,
-5.5223204e-06,
-4.6060023e-06,
-3.4228162e-06,
-2.5110220e-06,
-1.3293517e-06,
-4.2200946e-07,
7.5795888e-07,
1.6605890e-06,
2.8385188e-06,
3.7361454e-06,
4.9119956e-06,
5.8044527e-06,
6.9778213e-06,
7.8649358e-06,
9.0358378e-06,
9.9174824e-06,
1.1085778e-05,
1.1961714e-05,
1.3127053e-05,
1.3996902e-05,
1.5159166e-05,
1.6022976e-05,
1.7181813e-05,
1.8039196e-05,
1.9194443e-05,
2.0045370e-05,
2.1197034e-05,
2.2041161e-05,
2.3188916e-05,
2.4026231e-05,
2.5169983e-05,
2.6000254e-05,
2.7139820e-05,
2.7962655e-05,
2.9097679e-05,
2.9912991e-05,
3.1043523e-05,
3.1851059e-05,
3.2976758e-05,
3.3776354e-05,
3.4897188e-05,
3.5688895e-05,
3.6804670e-05,
3.7587989e-05,
3.8698598e-05,
3.9473482e-05,
4.0578638e-05,
4.1344865e-05,
4.2444436e-05,
4.3201653e-05,
4.4295415e-05,
4.5043695e-05,
4.6131679e-05,
4.6870616e-05,
4.7952592e-05,
4.8682287e-05,
4.9758130e-05,
5.0478398e-05,
5.1547679e-05,
5.2258199e-05,
5.3321005e-05,
5.4021588e-05,
5.5077627e-05,
5.5768200e-05,
5.6817327e-05,
5.7497643e-05,
5.8539837e-05,
5.9209893e-05,
6.0244956e-05,
6.0904490e-05,
6.1932348e-05,
6.2581333e-05,
6.3601703e-05,
6.4239758e-05,
6.5252254e-05,
6.5879513e-05,
6.6884209e-05,
6.7500397e-05,
6.8497158e-05,
6.9102246e-05,
7.0090787e-05,
7.0684642e-05,
7.1665048e-05,
7.2247508e-05,
7.3219415e-05,
7.3790400e-05,
7.4753845e-05,
7.5313006e-05,
7.6267671e-05,
7.6814797e-05,
7.7760620e-05,
7.8296039e-05,
7.9232883e-05,
7.9756333e-05,
8.0684158e-05,
8.1195424e-05,
8.2113887e-05,
8.2612991e-05,
8.3522141e-05,
8.4008838e-05,
8.4908409e-05,
8.5382657e-05,
8.6272604e-05,
8.6734054e-05,
8.7614206e-05,
8.8062917e-05,
8.8933478e-05,
8.9369466e-05,
9.0230083e-05,
9.0653189e-05,
9.1503758e-05,
9.1913612e-05,
9.2754021e-05,
9.3150775e-05,
9.3980939e-05,
9.4364357e-05,
9.5184231e-05,
9.5554222e-05,
9.6363586e-05,
9.6720070e-05,
9.7518999e-05,
9.7862025e-05,
9.8650612e-05,
9.8979824e-05,
9.9757707e-05,
1.0007310e-04,
1.0084011e-04,
1.0114148e-04,
1.0189782e-04,
1.0218526e-04,
1.0293068e-04,
1.0320403e-04,
1.0393859e-04,
1.0419772e-04,
1.0492121e-04,
1.0516617e-04,
1.0587877e-04,
1.0610943e-04,
1.0681090e-04,
1.0702692e-04,
1.0771716e-04,
1.0791862e-04,
1.0859750e-04,
1.0878454e-04,
1.0945228e-04,
1.0962472e-04,
1.1028099e-04,
1.1043878e-04,
1.1108359e-04,
1.1122667e-04,
1.1186001e-04,
1.1198836e-04,
1.1261021e-04,
1.1272367e-04,
1.1333375e-04,
1.1343223e-04,
1.1403070e-04,
1.1411423e-04,
1.1470109e-04,
1.1476993e-04,
1.1534507e-04,
1.1539881e-04,
1.1596202e-04,
1.1600078e-04,
1.1655205e-04,
1.1657587e-04,
1.1711526e-04,
1.1712408e-04,
1.1765136e-04,
1.1764511e-04,
1.1816028e-04,
1.1813900e-04,
1.1864225e-04,
1.1860590e-04,
1.1909726e-04,
1.1904587e-04,
1.1952497e-04,
1.1945829e-04,
1.1992523e-04,
1.1984345e-04,
1.2029827e-04,
1.2020141e-04,
1.2064408e-04,
1.2053214e-04,
1.2096247e-04,
1.2083542e-04,
1.2125359e-04,
1.2111148e-04,
1.2151749e-04,
1.2136026e-04,
1.2175413e-04,
1.2158159e-04,
1.2196307e-04,
1.2177533e-04,
1.2214473e-04,
1.2194189e-04,
1.2229920e-04,
1.2208118e-04,
1.2242634e-04,
1.2219314e-04,
1.2252620e-04,
1.2227782e-04,
1.2259873e-04,
1.2233525e-04,
1.2264405e-04,
1.2236542e-04,
1.2266212e-04,
1.2236814e-04,
1.2265281e-04,
1.2234385e-04,
1.2261670e-04,
1.2229264e-04,
1.2255351e-04,
1.2221430e-04,
1.2246325e-04,
1.2210890e-04,
1.2234603e-04,
1.2197658e-04,
1.2220175e-04,
1.2181733e-04,
1.2203072e-04,
1.2163127e-04,
1.2183288e-04,
1.2141840e-04,
1.2160848e-04,
1.2117921e-04,
1.2135767e-04,
1.2091341e-04,
1.2108015e-04,
1.2062103e-04,
1.2077619e-04,
1.2030226e-04,
1.2044587e-04,
1.1995722e-04,
1.2008936e-04,
1.1958613e-04,
1.1970684e-04,
1.1918905e-04,
1.1929833e-04,
1.1876608e-04,
1.1886416e-04,
1.1831742e-04,
1.1840411e-04,
1.1784282e-04,
1.1791829e-04,
1.1734273e-04,
1.1740702e-04,
1.1681723e-04,
1.1687041e-04,
1.1626652e-04,
1.1630883e-04,
1.1569090e-04,
1.1572207e-04,
1.1509024e-04,
1.1511038e-04,
1.1446476e-04,
1.1447402e-04,
1.1381450e-04,
1.1381283e-04,
1.1313960e-04,
1.1312736e-04,
1.1244055e-04,
1.1241766e-04,
1.1171733e-04,
1.1168396e-04,
1.1097028e-04,
1.1092655e-04,
1.1019940e-04,
1.1014511e-04,
1.0940470e-04,
1.0934006e-04,
1.0858661e-04,
1.0851172e-04,
1.0774518e-04,
1.0766017e-04,
1.0688083e-04,
1.0678592e-04,
1.0599366e-04,
1.0588883e-04,
1.0508384e-04,
1.0496930e-04,
1.0415168e-04,
1.0402734e-04,
1.0319692e-04,
1.0306309e-04,
1.0222014e-04,
1.0207694e-04,
1.0122175e-04,
1.0106924e-04,
1.0020180e-04,
1.0004017e-04,
9.9160657e-05,
9.8989889e-05,
9.8098174e-05,
9.7918499e-05,
9.7014866e-05,
9.6826408e-05,
9.5910844e-05,
9.5713495e-05,
9.4786124e-05,
9.4580463e-05,
9.3641499e-05,
9.3427498e-05,
9.2477116e-05,
9.2254770e-05,
9.1292992e-05,
9.1062519e-05,
9.0089412e-05,
8.9850730e-05,
8.8866448e-05,
8.8619985e-05,
8.7624724e-05,
8.7370464e-05,
8.6364169e-05,
8.6102238e-05,
8.5085400e-05,
8.4816094e-05,
8.3788720e-05,
8.3512064e-05,
8.2474125e-05,
8.2190052e-05,
8.1141836e-05,
8.0850584e-05,
7.9792145e-05,
7.9493813e-05,
7.8425534e-05,
7.8120398e-05,
7.7042253e-05,
7.6730280e-05,
7.5642373e-05,
7.5323942e-05,
7.4226714e-05,
7.3901728e-05,
7.2795042e-05,
7.2463716e-05,
7.1347553e-05,
7.1009908e-05,
6.9884818e-05,
6.9541073e-05,
6.8407140e-05,
6.8057529e-05,
6.6915044e-05,
6.6559514e-05,
6.5408376e-05,
6.5047082e-05,
6.3887645e-05,
6.3520989e-05,
6.2353289e-05,
6.1981101e-05,
6.0805183e-05,
6.0427725e-05,
5.9243740e-05,
5.8861350e-05,
5.7669697e-05,
5.7282435e-05,
5.6083234e-05,
5.5691302e-05,
5.4484583e-05,
5.4087836e-05,
5.2873719e-05,
5.2472557e-05,
5.1251467e-05,
5.0846058e-05,
4.9617796e-05,
4.9208183e-05,
4.7973096e-05,
4.7559448e-05,
4.6317760e-05,
4.5900645e-05,
4.4652344e-05,
4.4231626e-05,
4.2976922e-05,
4.2552753e-05,
4.1291608e-05,
4.0864083e-05,
3.9596888e-05,
3.9166438e-05,
3.7893390e-05,
3.7459954e-05,
3.6181017e-05,
3.5744872e-05,
3.4460386e-05,
3.4021648e-05,
3.2731846e-05,
3.2290878e-05,
3.0995566e-05,
3.0552306e-05,
2.9251789e-05,
2.8806417e-05,
2.7500851e-05,
2.7053690e-05,
2.5743478e-05,
2.5294697e-05,
2.3979767e-05,
2.3529348e-05,
2.2209954e-05,
2.1758236e-05,
2.0434494e-05,
1.9981626e-05,
1.8653831e-05,
1.8199756e-05,
1.6867859e-05,
1.6412923e-05,
1.5077198e-05,
1.4621557e-05,
1.3282323e-05,
1.2826259e-05,
1.1483702e-05,
1.1027094e-05,
9.6811799e-06,
9.2242361e-06,
7.8753410e-06,
7.4182983e-06,
6.0664861e-06,
5.6095462e-06,
4.2550727e-06,
3.7980617e-06,
2.4411086e-06,
1.9846365e-06,
6.2534710e-07,
1.6946370e-07,
-1.1919259e-06,
-1.6470717e-06,
-3.0105300e-06,
-3.4650410e-06,
-4.8303230e-06,
-5.2838715e-06,
-6.6506869e-06,
-7.1031275e-06,
-8.4712774e-06,
-8.9224160e-06,
-1.0291880e-05,
-1.0741680e-05,
-1.2112086e-05,
-1.2560184e-05,
-1.3931499e-05,
-1.4377829e-05,
-1.5749983e-05,
-1.6194384e-05,
-1.7567305e-05,
-1.8009600e-05,
-1.9382911e-05,
-1.9822877e-05,
-2.1196412e-05,
-2.1633882e-05,
-2.3007419e-05,
-2.3442289e-05,
-2.4815942e-05,
-2.5247979e-05,
-2.6621296e-05,
-2.7050526e-05,
-2.8423498e-05,
-2.8849645e-05,
-3.0222166e-05,
-3.0645037e-05,
-3.2016966e-05,
-3.2436212e-05,
-3.3807322e-05,
-3.4222872e-05,
-3.5593012e-05,
-3.6004694e-05,
-3.7373776e-05,
-3.7781568e-05,
-3.9149527e-05,
-3.9553111e-05,
-4.0919646e-05,
-4.1319095e-05,
-4.2684097e-05,
-4.3079012e-05,
-4.4442220e-05,
-4.4832523e-05,
-4.6193853e-05,
-4.6579287e-05,
-4.7938664e-05,
-4.8319127e-05,
-4.9676346e-05,
-5.0051790e-05,
-5.1406820e-05,
-5.1777150e-05,
-5.3129657e-05,
-5.3494542e-05,
-5.4844500e-05,
-5.5203862e-05,
-5.6550976e-05,
-5.6904588e-05,
-5.8248670e-05,
-5.8596550e-05,
-5.9937574e-05,
-6.0279595e-05,
-6.1617427e-05,
-6.1953324e-05,
-6.3287817e-05,
-6.3617654e-05,
-6.4948643e-05,
-6.5272162e-05,
-6.6599262e-05,
-6.6916258e-05,
-6.8239552e-05,
-6.8549939e-05,
-6.9869055e-05,
-7.0172828e-05,
-7.1487830e-05,
-7.1784899e-05,
-7.3095601e-05,
-7.3385810e-05,
-7.4691992e-05,
-7.4975081e-05,
-7.6276776e-05,
-7.6552699e-05,
-7.7849587e-05,
-7.8118076e-05,
-7.9409949e-05,
-7.9671020e-05,
-8.0957891e-05,
-8.1211457e-05,
-8.2493114e-05,
-8.2739208e-05,
-8.4015695e-05,
-8.4254041e-05,
-8.5524943e-05,
-8.5755341e-05,
-8.7020582e-05,
-8.7243030e-05,
-8.8502680e-05,
-8.8716848e-05,
-8.9970620e-05,
-9.0176406e-05,
-9.1424291e-05,
-9.1621792e-05,
-9.2863720e-05,
-9.3052678e-05,
-9.4288639e-05,
-9.4469036e-05,
-9.5698720e-05,
-9.5870231e-05,
-9.7093345e-05,
-9.7255907e-05,
-9.8472603e-05,
-9.8626386e-05,
-9.9836503e-05,
-9.9981024e-05,
-1.0118448e-04,
-1.0131976e-04,
-1.0251659e-04,
-1.0264265e-04,
-1.0383267e-04,
-1.0394915e-04,
-1.0513240e-04,
-1.0523922e-04,
-1.0641523e-04,
-1.0651220e-04,
-1.0768107e-04,
-1.0776834e-04,
-1.0893017e-04,
-1.0900768e-04,
-1.1016207e-04,
-1.1022955e-04,
-1.1137657e-04,
-1.1143400e-04,
-1.1257363e-04,
-1.1262092e-04,
-1.1375286e-04,
-1.1378987e-04,
-1.1491409e-04,
-1.1494063e-04,
-1.1605696e-04,
-1.1607308e-04,
-1.1718175e-04,
-1.1718761e-04,
-1.1828836e-04,
-1.1828354e-04,
-1.1937607e-04,
-1.1936072e-04,
-1.2044511e-04,
-1.2041906e-04,
-1.2149524e-04,
-1.2145835e-04,
-1.2252606e-04,
-1.2247847e-04,
-1.2353767e-04,
-1.2347910e-04,
-1.2452990e-04,
-1.2446059e-04,
-1.2550295e-04,
-1.2542275e-04,
-1.2645621e-04,
-1.2636478e-04,
-1.2738941e-04,
-1.2728710e-04,
-1.2830285e-04,
-1.2818946e-04,
-1.2919628e-04,
-1.2907158e-04,
-1.3006944e-04,
-1.2993362e-04,
-1.3092240e-04,
-1.3077530e-04,
-1.3175514e-04,
-1.3159687e-04,
-1.3256749e-04,
-1.3239762e-04,
-1.3335880e-04,
-1.3317750e-04,
-1.3412949e-04,
-1.3393689e-04,
-1.3487950e-04,
-1.3467552e-04,
-1.3560863e-04,
-1.3539300e-04,
-1.3631683e-04,
-1.3608961e-04,
-1.3700394e-04,
-1.3676518e-04,
-1.3767007e-04,
-1.3741955e-04,
-1.3831479e-04,
-1.3805225e-04,
-1.3893801e-04,
-1.3866387e-04,
-1.3954024e-04,
-1.3925425e-04,
-1.4012101e-04,
-1.3982323e-04,
-1.4068018e-04,
-1.4037053e-04,
-1.4121789e-04,
-1.4089624e-04,
-1.4173390e-04,
-1.4140042e-04,
-1.4222838e-04,
-1.4188272e-04,
-1.4270091e-04,
-1.4234324e-04,
-1.4315196e-04,
-1.4278237e-04,
-1.4358142e-04,
-1.4319961e-04,
-1.4398893e-04,
-1.4359495e-04,
-1.4437442e-04,
-1.4396849e-04,
-1.4473815e-04,
-1.4432003e-04,
-1.4508001e-04,
-1.4464985e-04,
-1.4540002e-04,
-1.4495770e-04,
-1.4569804e-04,
-1.4524383e-04,
-1.4597448e-04,
-1.4550808e-04,
-1.4622879e-04,
-1.4575023e-04,
-1.4646110e-04,
-1.4597033e-04,
-1.4667144e-04,
-1.4616879e-04,
-1.4685995e-04,
-1.4634526e-04,
-1.4702669e-04,
-1.4650000e-04,
-1.4717148e-04,
-1.4663285e-04,
-1.4729446e-04,
-1.4674395e-04,
-1.4739561e-04,
-1.4683302e-04,
-1.4747472e-04,
-1.4690029e-04,
-1.4753213e-04,
-1.4694583e-04,
-1.4756796e-04,
-1.4696992e-04,
-1.4758206e-04,
-1.4697234e-04,
-1.4757459e-04,
-1.4695307e-04,
-1.4754534e-04,
-1.4691215e-04,
-1.4749466e-04,
-1.4684968e-04,
-1.4742226e-04,
-1.4676567e-04,
-1.4732853e-04,
-1.4666036e-04,
-1.4721351e-04,
-1.4653385e-04,
-1.4707738e-04,
-1.4638605e-04,
-1.4691983e-04,
-1.4621711e-04,
-1.4674109e-04,
-1.4602678e-04,
-1.4654111e-04,
-1.4581546e-04,
-1.4632029e-04,
-1.4558325e-04,
-1.4607853e-04,
-1.4533028e-04,
-1.4581615e-04,
-1.4505651e-04,
-1.4553303e-04,
-1.4476214e-04,
-1.4522931e-04,
-1.4444694e-04,
-1.4490488e-04,
-1.4411131e-04,
-1.4455992e-04,
-1.4375512e-04,
-1.4419467e-04,
-1.4337892e-04,
-1.4380937e-04,
-1.4298266e-04,
-1.4340410e-04,
-1.4256634e-04,
-1.4297867e-04,
-1.4212994e-04,
-1.4253337e-04,
-1.4167373e-04,
-1.4206826e-04,
-1.4119768e-04,
-1.4158356e-04,
-1.4070215e-04,
-1.4107931e-04,
-1.4018728e-04,
-1.4055593e-04,
-1.3965336e-04,
-1.4001339e-04,
-1.3910024e-04,
-1.3945171e-04,
-1.3852790e-04,
-1.3887075e-04,
-1.3793669e-04,
-1.3827116e-04,
-1.3732679e-04,
-1.3765287e-04,
-1.3669839e-04,
-1.3701629e-04,
-1.3605159e-04,
-1.3636129e-04,
-1.3538673e-04,
-1.3568835e-04,
-1.3470375e-04,
-1.3499720e-04,
-1.3400263e-04,
-1.3428794e-04,
-1.3328364e-04,
-1.3356108e-04,
-1.3254730e-04,
-1.3281687e-04,
-1.3179358e-04,
-1.3205531e-04,
-1.3102272e-04,
-1.3127671e-04,
-1.3023467e-04,
-1.3048105e-04,
-1.2942987e-04,
-1.2966859e-04,
-1.2860815e-04,
-1.2883923e-04,
-1.2776976e-04,
-1.2799338e-04,
-1.2691525e-04,
-1.2713176e-04,
-1.2604480e-04,
-1.2625393e-04,
-1.2515829e-04,
-1.2536022e-04,
-1.2425602e-04,
-1.2445081e-04,
-1.2333804e-04,
-1.2352599e-04,
-1.2240476e-04,
-1.2258572e-04,
-1.2145615e-04,
-1.2163034e-04,
-1.2049269e-04,
-1.2066042e-04,
-1.1951487e-04,
-1.1967606e-04,
-1.1852233e-04,
-1.1867689e-04,
-1.1751527e-04,
-1.1766350e-04,
-1.1649414e-04,
-1.1663606e-04,
-1.1545911e-04,
-1.1559507e-04,
-1.1441035e-04,
-1.1454027e-04,
-1.1334808e-04,
-1.1347235e-04,
-1.1227285e-04,
-1.1239157e-04,
-1.1118468e-04,
-1.1129751e-04,
-1.1008330e-04,
-1.1019066e-04,
-1.0896938e-04,
-1.0907145e-04,
-1.0784331e-04,
-1.0794006e-04,
-1.0670527e-04,
-1.0679685e-04,
-1.0555515e-04,
-1.0564172e-04,
-1.0439347e-04,
-1.0447531e-04,
-1.0322060e-04,
-1.0329756e-04,
-1.0203621e-04,
-1.0210831e-04,
-1.0084078e-04,
-1.0090854e-04,
-9.9634924e-05,
-9.9698303e-05,
-9.8418691e-05,
-9.8477743e-05,
-9.7192383e-05,
-9.7246948e-05,
-9.5955829e-05,
-9.6006325e-05,
-9.4709717e-05,
-9.4756200e-05,
-9.3454144e-05,
-9.3496495e-05,
-9.2189014e-05,
-9.2227706e-05,
-9.0915266e-05,
-9.0950349e-05,
-8.9632965e-05,
-8.9664389e-05,
-8.8341999e-05,
-8.8369987e-05,
-8.7042849e-05,
-8.7067210e-05,
-8.5735635e-05,
-8.5756826e-05,
-8.4420880e-05,
-8.4438827e-05,
-8.3098473e-05,
-8.3113335e-05,
-8.1768938e-05,
-8.1781103e-05,
-8.0432760e-05,
-8.0441947e-05,
-7.9089726e-05,
-7.9096105e-05,
-7.7740055e-05,
-7.7743900e-05,
-7.6384179e-05,
-7.6385565e-05,
-7.5022621e-05,
-7.5021727e-05,
-7.3655423e-05,
-7.3652221e-05,
-7.2282584e-05,
-7.2277442e-05,
-7.0904919e-05,
-7.0897863e-05,
-6.9522210e-05,
-6.9513243e-05,
-6.8134656e-05,
-6.8124027e-05,
-6.6742827e-05,
-6.6730695e-05,
-6.5346834e-05,
-6.5333503e-05,
-6.3947356e-05,
-6.3932631e-05,
-6.2544052e-05,
-6.2528063e-05,
-6.1137297e-05,
-6.1120422e-05,
-5.9727628e-05,
-5.9709797e-05,
-5.8314853e-05,
-5.8296430e-05,
-5.6899569e-05,
-5.6880673e-05,
-5.5482232e-05,
-5.5462912e-05,
-5.4062779e-05,
-5.4043350e-05,
-5.2641703e-05,
-5.2621891e-05,
-5.1218806e-05,
-5.1198885e-05,
-4.9794749e-05,
-4.9774933e-05,
-4.8369634e-05,
-4.8349998e-05,
-4.6943835e-05,
-4.6924616e-05,
-4.5517648e-05,
-4.5498913e-05,
-4.4091230e-05,
-4.4073031e-05,
-4.2664787e-05,
-4.2647336e-05,
-4.1238462e-05,
-4.1221744e-05,
-3.9812610e-05,
-3.9796902e-05,
-3.8387785e-05,
-3.8373073e-05,
-3.6963941e-05,
-3.6950412e-05,
-3.5541634e-05,
-3.5529234e-05,
-3.4120732e-05,
-3.4109630e-05,
-3.2701426e-05,
-3.2691731e-05,
-3.1284280e-05,
-3.1276109e-05,
-2.9869186e-05,
-2.9862769e-05,
-2.8456861e-05,
-2.8452331e-05,
-2.7047463e-05,
-2.7044711e-05,
-2.5641070e-05,
-2.5640283e-05,
-2.4237885e-05,
-2.4239057e-05,
-2.2838029e-05,
-2.2841366e-05,
-2.1441864e-05,
-2.1447547e-05,
-2.0049926e-05,
-2.0057972e-05,
-1.8662098e-05,
-1.8672798e-05,
-1.7278975e-05,
-1.7292329e-05,
-1.5900479e-05,
-1.5916484e-05,
-1.4526860e-05,
-1.4545621e-05,
-1.3158113e-05,
-1.3179895e-05,
-1.1794892e-05,
-1.1819799e-05,
-1.0437405e-05,
-1.0465652e-05,
-9.0858392e-06,
-9.1173364e-06,
-7.7402888e-06,
-7.7753573e-06,
-6.4010337e-06,
-6.4396470e-06,
-5.0681263e-06,
-5.1104306e-06,
-3.7418797e-06,
-3.7878980e-06,
-2.4225610e-06,
-2.4726382e-06,
-1.1107236e-06,
-1.1648632e-06,
1.9369879e-07,
1.3538100e-07,
1.4907438e-06,
1.4282857e-06,
2.7799692e-06,
2.7130313e-06,
4.0612848e-06,
3.9898714e-06,
5.3344697e-06,
5.2583840e-06,
6.5992396e-06,
6.5183983e-06,
7.8550439e-06,
7.7693530e-06,
9.1019785e-06,
9.0114749e-06,
1.0340044e-05,
1.0244735e-05,
1.1569194e-05,
1.1468841e-05,
1.2788708e-05,
1.2683198e-05,
1.3998647e-05,
1.3887933e-05,
1.5198838e-05,
1.5082748e-05,
1.6388975e-05,
1.6267285e-05,
1.7568663e-05,
1.7441604e-05,
1.8738287e-05,
1.8605835e-05,
1.9897527e-05,
1.9759625e-05,
2.1046142e-05,
2.0902357e-05,
2.2183522e-05,
2.2034159e-05,
2.3309922e-05,
2.3154818e-05,
2.4425209e-05,
2.4264241e-05,
2.5529019e-05,
2.5362000e-05,
2.6621389e-05,
2.6448557e-05,
2.7702288e-05,
2.7523459e-05,
2.8771273e-05,
2.8586284e-05,
2.9828045e-05,
2.9636643e-05,
3.0872537e-05,
3.0675124e-05,
3.1904901e-05,
3.1701197e-05,
3.2924830e-05,
3.2714746e-05,
3.3932084e-05,
3.3715651e-05,
3.4926909e-05,
3.4703989e-05,
3.5908752e-05,
3.5679136e-05,
3.6877410e-05,
3.6641075e-05,
3.7832718e-05,
3.7589752e-05,
3.8775116e-05,
3.8525477e-05,
3.9704201e-05,
3.9447668e-05,
4.0619587e-05,
4.0356095e-05,
4.1521397e-05,
4.1251080e-05,
4.2409620e-05,
4.2132017e-05,
4.3283696e-05,
4.2998831e-05,
4.4143731e-05,
4.3851733e-05,
4.4989674e-05,
4.4690732e-05,
4.5821892e-05,
4.5515500e-05,
4.6639550e-05,
4.6325758e-05,
4.7442590e-05,
4.7121421e-05,
4.8231382e-05,
4.7902826e-05,
4.9005515e-05,
4.8669252e-05,
4.9764870e-05,
4.9421078e-05,
5.0509596e-05,
5.0158228e-05,
5.1239577e-05,
5.0880738e-05,
5.1954686e-05,
5.1587929e-05,
5.2654397e-05,
5.2279974e-05,
5.3339029e-05,
5.2957090e-05,
5.4008726e-05,
5.3619040e-05,
5.4663032e-05,
5.4265677e-05,
5.5302148e-05,
5.4897107e-05,
5.5925991e-05,
5.5513045e-05,
5.6534448e-05,
5.6113686e-05,
5.7127121e-05,
5.6698479e-05,
5.7704226e-05,
5.7267866e-05,
5.8265999e-05,
5.7822041e-05,
5.8812119e-05,
5.8360305e-05,
5.9342506e-05,
5.8883086e-05,
5.9857273e-05,
5.9390001e-05,
6.0356159e-05,
5.9881043e-05,
6.0839335e-05,
6.0356369e-05,
6.1306458e-05,
6.0815820e-05,
6.1758055e-05,
6.1259668e-05,
6.2193963e-05,
6.1687726e-05,
6.2613712e-05,
6.2099645e-05,
6.3017621e-05,
6.2495809e-05,
6.3405593e-05,
6.2875894e-05,
6.3777522e-05,
6.3240195e-05,
6.4133768e-05,
6.3588526e-05,
6.4474041e-05,
6.3921137e-05,
6.4798601e-05,
6.4237854e-05,
6.5107130e-05,
6.4538317e-05,
6.5399496e-05,
6.4822952e-05,
6.5676028e-05,
6.5091625e-05,
6.5936640e-05,
6.5344483e-05,
6.6181490e-05,
6.5581633e-05,
6.6410589e-05,
6.5802674e-05,
6.6623793e-05,
6.6008104e-05,
6.6821104e-05,
6.6197515e-05,
6.7002412e-05,
6.6370880e-05,
6.7168019e-05,
6.6528826e-05,
6.7317931e-05,
6.6670977e-05,
6.7452226e-05,
6.6797676e-05,
6.7571060e-05,
6.6908643e-05,
6.7673951e-05,
6.7003704e-05,
6.7761301e-05,
6.7083350e-05,
6.7833033e-05,
6.7147340e-05,
6.7889209e-05,
6.7195972e-05,
6.7930206e-05,
6.7229334e-05,
6.7955796e-05,
6.7247420e-05,
6.7966104e-05,
6.7250277e-05,
6.7961079e-05,
6.7237628e-05,
6.7940401e-05,
6.7209776e-05,
6.7904956e-05,
6.7166959e-05,
6.7854489e-05,
6.7109201e-05,
6.7789072e-05,
6.7036672e-05,
6.7708826e-05,
6.6949076e-05,
6.7613713e-05,
6.6846850e-05,
6.7503749e-05,
6.6729718e-05,
6.7378819e-05,
6.6597812e-05,
6.7239292e-05,
6.6451705e-05,
6.7085698e-05,
6.6291186e-05,
6.6917649e-05,
6.6116381e-05,
6.6735269e-05,
6.5927235e-05,
6.6538502e-05,
6.5723781e-05,
6.6327709e-05,
6.5506329e-05,
6.6102822e-05,
6.5274753e-05,
6.5863902e-05,
6.5029487e-05,
6.5611644e-05,
6.4770937e-05,
6.5345682e-05,
6.4498499e-05,
6.5065990e-05,
6.4212458e-05,
6.4772800e-05,
6.3912881e-05,
6.4466156e-05,
6.3600181e-05,
6.4146440e-05,
6.3274209e-05,
6.3813640e-05,
6.2935279e-05,
6.3467995e-05,
6.2583680e-05,
6.3109831e-05,
6.2219404e-05,
6.2738450e-05,
6.1842028e-05,
6.2354478e-05,
6.1452131e-05,
6.1958148e-05,
6.1049980e-05,
6.1549657e-05,
6.0635752e-05,
6.1128953e-05,
6.0209276e-05,
6.0696285e-05,
5.9771041e-05,
6.0251893e-05,
5.9321037e-05,
5.9795608e-05,
5.8859010e-05,
5.9327271e-05,
5.8385367e-05,
5.8847725e-05,
5.7900578e-05,
5.8357061e-05,
5.7404782e-05,
5.7855373e-05,
5.6897712e-05,
5.7342412e-05,
5.6379725e-05,
5.6818715e-05,
5.5851064e-05,
5.6284398e-05,
5.5311707e-05,
5.5739229e-05,
5.4761517e-05,
5.5183715e-05,
5.4201580e-05,
5.4618277e-05,
5.3631697e-05,
5.4042893e-05,
5.3051824e-05,
5.3457434e-05,
5.2461827e-05,
5.2862126e-05,
5.1862385e-05,
5.2257419e-05,
5.1253465e-05,
5.1643339e-05,
5.0635220e-05,
5.1019899e-05,
5.0007756e-05,
5.0387762e-05,
4.9371935e-05,
4.9746796e-05,
4.8727174e-05,
4.9097012e-05,
4.8073566e-05,
4.8438421e-05,
4.7411585e-05,
4.7771763e-05,
4.6741548e-05,
4.7097009e-05,
4.6063467e-05,
4.6414297e-05,
4.5377521e-05,
4.5723813e-05,
4.4684007e-05,
4.5026018e-05,
4.3982995e-05,
4.4320576e-05,
4.3274454e-05,
4.3607702e-05,
4.2558645e-05,
4.2887861e-05,
4.1836283e-05,
4.2161488e-05,
4.1107117e-05,
4.1428308e-05,
4.0371401e-05,
4.0688662e-05,
3.9629215e-05,
3.9942759e-05,
3.8881036e-05,
3.9190861e-05,
3.8126446e-05,
3.8432838e-05,
3.7366222e-05,
3.7669125e-05,
3.6600519e-05,
3.6900317e-05,
3.5829636e-05,
3.6126108e-05,
3.5053324e-05,
3.5346633e-05,
3.4271956e-05,
3.4562222e-05,
3.3485673e-05,
3.3773165e-05,
3.2694988e-05,
3.2979531e-05,
3.1899545e-05,
3.2181552e-05,
3.1100158e-05,
3.1379612e-05,
3.0296940e-05,
3.0573874e-05,
2.9489651e-05,
2.9764003e-05,
2.8678627e-05,
2.8950655e-05,
2.7864140e-05,
2.8133915e-05,
2.7046534e-05,
2.7314284e-05,
2.6226048e-05,
2.6491557e-05,
2.5402635e-05,
2.5666241e-05,
2.4576555e-05,
2.4838458e-05,
2.3748204e-05,
2.4008031e-05,
2.2917259e-05,
2.3175351e-05,
2.2084450e-05,
2.2340895e-05,
2.1249845e-05,
2.1504667e-05,
2.0413863e-05,
2.0667157e-05,
1.9576330e-05,
1.9828003e-05,
1.8737557e-05,
1.8987809e-05,
1.7897500e-05,
1.8146639e-05,
1.7056709e-05,
1.7304519e-05,
1.6215285e-05,
1.6462154e-05,
1.5373610e-05,
1.5619430e-05,
1.4531726e-05,
1.4776657e-05,
1.3689959e-05,
1.3933889e-05,
1.2847997e-05,
1.3091098e-05,
1.2006483e-05,
1.2248778e-05,
1.1165474e-05,
1.1407191e-05,
1.0325172e-05,
1.0566620e-05,
9.4861304e-06,
9.7272055e-06,
8.6480575e-06,
8.8887874e-06,
7.8112861e-06,
8.0519737e-06,
6.9759926e-06,
7.2163979e-06,
6.1421318e-06,
6.3827288e-06,
5.3104436e-06,
5.5511215e-06,
4.4810510e-06,
4.7218851e-06,
3.6537694e-06,
3.8952262e-06,
2.8291581e-06,
3.0708404e-06,
2.0068671e-06,
2.2491176e-06,
1.1875144e-06,
1.4304655e-06,
3.7106823e-07,
6.1464187e-07,
-4.4209678e-07,
-1.9739359e-07,
-1.2515857e-06,
-1.0059193e-06,
-2.0574114e-06,
-1.8108372e-06,
-2.8596526e-06,
-2.6119809e-06,
-3.6581253e-06,
-3.4092810e-06,
-4.4524472e-06,
-4.2021429e-06,
-5.2422643e-06,
-4.9906817e-06,
-6.0277186e-06,
-5.7745591e-06,
-6.8081235e-06,
-6.5533203e-06,
-7.5837002e-06,
-7.3273516e-06,
-8.3544480e-06,
-8.0965934e-06,
-9.1200343e-06,
-8.8606513e-06,
-9.8804877e-06,
-9.6189655e-06,
-1.0634946e-05,
-1.0371467e-05,
-1.1383762e-05,
-1.1118488e-05,
-1.2126809e-05,
-1.1859242e-05,
-1.2863418e-05,
-1.2594008e-05,
-1.3594192e-05,
-1.3322774e-05,
-1.4318769e-05,
-1.4045229e-05,
-1.5036633e-05,
-1.4760900e-05,
-1.5747650e-05,
-1.5469387e-05,
-1.6451526e-05,
-1.6171081e-05,
-1.7148794e-05,
-1.6865959e-05,
-1.7838830e-05,
-1.7553293e-05,
-1.8521404e-05,
-1.8233599e-05,
-1.9196884e-05,
-1.8906396e-05,
-1.9864625e-05,
-1.9571302e-05,
-2.0524350e-05,
-2.0228167e-05,
-2.1175830e-05,
-2.0877074e-05,
-2.1819623e-05,
-2.1518173e-05,
-2.2455485e-05,
-2.2150948e-05,
-2.3082737e-05,
-2.2775157e-05,
-2.3701688e-05,
-2.3391017e-05,
-2.4312016e-05,
-2.3997956e-05,
-2.4913367e-05,
-2.4595877e-05,
-2.5505741e-05,
-2.5184901e-05,
-2.6089109e-05,
-2.5765207e-05,
-2.6663878e-05,
-2.6336398e-05,
-2.7229119e-05,
-2.6897898e-05,
-2.7784758e-05,
-2.7450055e-05,
-2.8331230e-05,
-2.7992642e-05,
-2.8867714e-05,
-2.8525329e-05,
-2.9394499e-05,
-2.9048284e-05,
-2.9911531e-05,
-2.9561533e-05,
-3.0418884e-05,
-3.0064939e-05,
-3.0916074e-05,
-3.0557939e-05,
-3.1402628e-05,
-3.1040378e-05,
-3.1878977e-05,
-3.1512809e-05,
-3.2345155e-05,
-3.1974649e-05,
-3.2800449e-05,
-3.2425887e-05,
-3.3245377e-05,
-3.2866628e-05,
-3.3679715e-05,
-3.3296742e-05,
-3.4103311e-05,
-3.3715849e-05,
-3.4515457e-05,
-3.4123745e-05,
-3.4916627e-05,
-3.4520722e-05,
-3.5307041e-05,
-3.4906910e-05,
-3.5686280e-05,
-3.5281635e-05,
-3.6054151e-05,
-3.5645164e-05,
-3.6410745e-05,
-3.5997339e-05,
-3.6756010e-05,
-3.6338063e-05,
-3.7089514e-05,
-3.6667024e-05,
-3.7411215e-05,
-3.6984442e-05,
-3.7721615e-05,
-3.7290586e-05,
-3.8020505e-05,
-3.7584961e-05,
-3.8307356e-05,
-3.7867257e-05,
-3.8582490e-05,
-3.8137828e-05,
-3.8845528e-05,
-3.8396419e-05,
-3.9096797e-05,
-3.8642989e-05,
-3.9335885e-05,
-3.8877484e-05,
-3.9563144e-05,
-3.9100135e-05,
-3.9778349e-05,
-3.9310839e-05,
-3.9981353e-05,
-3.9509005e-05,
-4.0171788e-05,
-3.9694886e-05,
-4.0350211e-05,
-3.9868559e-05,
-4.0516195e-05,
-4.0029970e-05,
-4.0669996e-05,
-4.0178962e-05,
-4.0811542e-05,
-4.0315653e-05,
-4.0940794e-05,
-4.0439944e-05,
-4.1057373e-05,
-4.0551666e-05,
-4.1161428e-05,
-4.0650700e-05,
-4.1252868e-05,
-4.0737496e-05,
-4.1332065e-05,
-4.0811720e-05,
-4.1398838e-05,
-4.0873563e-05,
-4.1452891e-05,
-4.0922752e-05,
-4.1494561e-05,
-4.0959365e-05,
-4.1523422e-05,
-4.0983167e-05,
-4.1539656e-05,
-4.0994328e-05,
-4.1543245e-05,
-4.0992991e-05,
-4.1534432e-05,
-4.0979341e-05,
-4.1513033e-05,
-4.0952926e-05,
-4.1479155e-05,
-4.0913890e-05,
-4.1432261e-05,
-4.0862185e-05,
-4.1372830e-05,
-4.0797798e-05,
-4.1300712e-05,
-4.0720795e-05,
-4.1216265e-05,
-4.0631366e-05,
-4.1119071e-05,
-4.0529505e-05,
-4.1009637e-05,
-4.0415168e-05,
-4.0887310e-05,
-4.0288144e-05,
-4.0752418e-05,
-4.0148326e-05,
-4.0604863e-05,
-3.9996119e-05,
-4.0444790e-05,
-3.9831483e-05,
-4.0272576e-05,
-3.9654669e-05,
-4.0087978e-05,
-3.9465314e-05,
-3.9890737e-05,
-3.9263585e-05,
-3.9681312e-05,
-3.9049441e-05,
-3.9459204e-05,
-3.8822978e-05,
-3.9224701e-05,
-3.8584065e-05,
-3.8978279e-05,
-3.8333222e-05,
-3.8719625e-05,
-3.8070417e-05,
-3.8449164e-05,
-3.7795553e-05,
-3.8166214e-05,
-3.7508064e-05,
-3.7871201e-05,
-3.7208716e-05,
-3.7564200e-05,
-3.6897305e-05,
-3.7245275e-05,
-3.6574083e-05,
-3.6914352e-05,
-3.6239132e-05,
-3.6571995e-05,
-3.5892535e-05,
-3.6217866e-05,
-3.5534314e-05,
-3.5851909e-05,
-3.5164031e-05,
-3.5473937e-05,
-3.4781915e-05,
-3.5084782e-05,
-3.4388678e-05,
-3.4684088e-05,
-3.3983881e-05,
-3.4272268e-05,
-3.3567819e-05,
-3.3849006e-05,
-3.3140668e-05,
-3.3414526e-05,
-3.2702111e-05,
-3.2968905e-05,
-3.2252298e-05,
-3.2511683e-05,
-3.1791053e-05,
-3.2043569e-05,
-3.1319388e-05,
-3.1564982e-05,
-3.0836899e-05,
-3.1075387e-05,
-3.0343502e-05,
-3.0575177e-05,
-2.9839435e-05,
-3.0064300e-05,
-2.9324784e-05,
-2.9542611e-05,
-2.8799418e-05,
-2.9010552e-05,
-2.8263535e-05,
-2.8467875e-05,
-2.7717544e-05,
-2.7915501e-05,
-2.7162048e-05,
-2.7353026e-05,
-2.6595859e-05,
-2.6780208e-05,
-2.6019754e-05,
-2.6197423e-05,
-2.5433752e-05,
-2.5604937e-05,
-2.4837901e-05,
-2.5002465e-05,
-2.4232435e-05,
-2.4390416e-05,
-2.3617431e-05,
-2.3769143e-05,
-2.2993522e-05,
-2.3138815e-05,
-2.2360224e-05,
-2.2498712e-05,
-2.1717125e-05,
-2.1849372e-05,
-2.1065268e-05,
-2.1191160e-05,
-2.0404486e-05,
-2.0524210e-05,
-1.9734904e-05,
-1.9848421e-05,
-1.9056738e-05,
-1.9163930e-05,
-1.8370083e-05,
-1.8471419e-05,
-1.7675281e-05,
-1.7770234e-05,
-1.6971483e-05,
-1.7060222e-05,
-1.6259656e-05,
-1.6342515e-05,
-1.5540041e-05,
-1.5617044e-05,
-1.4812543e-05,
-1.4883725e-05,
-1.4077263e-05,
-1.4142755e-05,
-1.3334273e-05,
-1.3394127e-05,
-1.2583937e-05,
-1.2638339e-05,
-1.1826158e-05,
-1.1874805e-05,
-1.1060760e-05,
-1.1104147e-05,
-1.0289014e-05,
-1.0327099e-05,
-9.5103145e-06,
-9.5430822e-06,
-8.7248148e-06,
-8.7523051e-06,
-7.9325356e-06,
-7.9550247e-06,
-7.1337175e-06,
-7.1513565e-06,
-6.3287614e-06,
-6.3413846e-06,
-5.5173712e-06,
-5.5252474e-06,
-4.7001498e-06,
-4.7036259e-06,
-3.8774564e-06,
-3.8762299e-06,
-3.0488014e-06,
-3.0428167e-06,
-2.2145486e-06,
-2.2039974e-06,
-1.3748004e-06,
-1.3599206e-06,
-5.3000719e-07,
-5.1086231e-07,
3.1978225e-07,
3.4338394e-07,
1.1746251e-06,
1.2021807e-06,
2.0338020e-06,
2.0653663e-06,
2.8978034e-06,
2.9336377e-06,
3.7663868e-06,
3.8062466e-06,
4.6391147e-06,
4.6829192e-06,
5.5157969e-06,
5.5634352e-06,
6.3962303e-06,
6.4476784e-06,
7.2805701e-06,
7.3358337e-06,
8.1684522e-06,
8.2273009e-06,
9.0595272e-06,
9.1221595e-06,
9.9542338e-06,
1.0020584e-05,
1.0851914e-05,
1.0921497e-05,
1.1752214e-05,
1.1825307e-05,
1.2655029e-05,
1.2731568e-05,
1.3560565e-05,
1.3640453e-05,
1.4468705e-05,
1.4551782e-05,
1.5378933e-05,
1.5465193e-05,
1.6291174e-05,
1.6380601e-05,
1.7205380e-05,
1.7297702e-05,
1.8121099e-05,
1.8216165e-05,
1.9038242e-05,
1.9136255e-05,
1.9956683e-05,
2.0057661e-05,
2.0876615e-05,
2.0980384e-05,
2.1797623e-05,
2.1903876e-05,
2.2719305e-05,
2.2827961e-05,
2.3641582e-05,
2.3752525e-05,
2.4564253e-05,
2.4677292e-05,
2.5487205e-05,
2.5602613e-05,
2.6410524e-05,
2.6527842e-05,
2.7333762e-05,
2.7453252e-05,
2.8256842e-05,
2.8378311e-05,
2.9179473e-05,
2.9302586e-05,
3.0101362e-05,
3.0226059e-05,
3.1022495e-05,
3.1148849e-05,
3.1942826e-05,
3.2070646e-05,
3.2862224e-05,
3.2991649e-05,
3.3780543e-05,
3.3910854e-05,
3.4697254e-05,
3.4828771e-05,
3.5612161e-05,
3.5744781e-05,
3.6525222e-05,
3.6658792e-05,
3.7436213e-05,
3.7570846e-05,
3.8345320e-05,
3.8481073e-05,
3.9252304e-05,
3.9388858e-05,
4.0156859e-05,
4.0294029e-05,
4.1058549e-05,
4.1196306e-05,
4.1957420e-05,
4.2095617e-05,
4.2853185e-05,
4.2991959e-05,
4.3745800e-05,
4.3885161e-05,
4.4635229e-05,
4.4775229e-05,
4.5521521e-05,
4.5661837e-05,
4.6403901e-05,
4.6544415e-05,
4.7282283e-05,
4.7422794e-05,
4.8156380e-05,
4.8297476e-05,
4.9026657e-05,
4.9167627e-05,
4.9892560e-05,
5.0033704e-05,
5.0753945e-05,
5.0895286e-05,
5.1610991e-05,
5.1752341e-05,
5.2463208e-05,
5.2604062e-05,
5.3309876e-05,
5.3450610e-05,
5.4151447e-05,
5.4291919e-05,
5.4987951e-05,
5.5128359e-05,
5.5819272e-05,
5.5959042e-05,
5.6644867e-05,
5.6784050e-05,
5.7464610e-05,
5.7603329e-05,
5.8278639e-05,
5.8416594e-05,
5.9086115e-05,
5.9223082e-05,
5.9887195e-05,
6.0023642e-05,
6.0682283e-05,
6.0817996e-05,
6.1471315e-05,
6.1605781e-05,
6.2253375e-05,
6.2386743e-05,
6.3028544e-05,
6.3160394e-05,
6.3796750e-05,
6.3927404e-05,
6.4557865e-05,
6.4686996e-05,
6.5311283e-05,
6.5439094e-05,
6.6057828e-05,
6.6184263e-05,
6.6797030e-05,
6.6921961e-05,
6.7528553e-05,
6.7651431e-05,
6.8251703e-05,
6.8373011e-05,
6.8967032e-05,
6.9086299e-05,
6.9674437e-05,
6.9791792e-05,
7.0373481e-05,
7.0488666e-05,
7.1064108e-05,
7.1177554e-05,
7.1746792e-05,
7.1857915e-05,
7.2420510e-05,
7.2529461e-05,
7.3085103e-05,
7.3191806e-05,
7.3740773e-05,
7.3845362e-05,
7.4387654e-05,
7.4490164e-05,
7.5025649e-05,
7.5125559e-05,
7.5654111e-05,
7.5751588e-05,
7.6273469e-05,
7.6368747e-05,
7.6883253e-05,
7.6975755e-05,
7.7483000e-05,
7.7572964e-05,
7.8072910e-05,
7.8160547e-05,
7.8653412e-05,
7.8738461e-05,
7.9223943e-05,
7.9306747e-05,
7.9784444e-05,
7.9864203e-05,
8.0334466e-05,
8.0411816e-05,
8.0874644e-05,
8.0949244e-05,
8.1403899e-05,
8.1475710e-05,
8.1922715e-05,
8.1991803e-05,
8.2431207e-05,
8.2497703e-05,
8.2929276e-05,
8.2992868e-05,
8.3416331e-05,
8.3477036e-05,
8.3892210e-05,
8.3949850e-05,
8.4357174e-05,
8.4412087e-05,
8.4811421e-05,
8.4862915e-05,
8.5254056e-05,
8.5302629e-05,
8.5685758e-05,
8.5731262e-05,
8.6106275e-05,
8.6148670e-05,
8.6515289e-05,
8.6554402e-05,
8.6912675e-05,
8.6948200e-05,
8.7298161e-05,
8.7330762e-05,
8.7672632e-05,
8.7701743e-05,
8.8035331e-05,
8.8060652e-05,
8.8386214e-05,
8.8408141e-05,
8.8725313e-05,
8.8743575e-05,
8.9052318e-05,
8.9066897e-05,
8.9367259e-05,
8.9378119e-05,
8.9670135e-05,
8.9677143e-05,
8.9960818e-05,
8.9964639e-05,
9.0239886e-05,
9.0239646e-05,
9.0506197e-05,
9.0502237e-05,
9.0760538e-05,
9.0752653e-05,
9.1002303e-05,
9.0990367e-05,
9.1231561e-05,
9.1215753e-05,
9.1448533e-05,
9.1428851e-05,
9.1653012e-05,
9.1629538e-05,
9.1845124e-05,
9.1817812e-05,
9.2024466e-05,
9.1993166e-05,
9.2190780e-05,
9.2155831e-05,
9.2344741e-05,
9.2305719e-05,
9.2485781e-05,
9.2442880e-05,
9.2614255e-05,
9.2567583e-05,
9.2730006e-05,
9.2679515e-05,
9.2832924e-05,
9.2778627e-05,
9.2923155e-05,
9.2864648e-05,
9.2999877e-05,
9.2937765e-05,
9.3064013e-05,
9.2998166e-05,
9.3115331e-05,
9.3045688e-05,
9.3153833e-05,
9.3080601e-05,
9.3179621e-05,
9.3102473e-05,
9.3192142e-05,
9.3111191e-05,
9.3191747e-05,
9.3106974e-05,
9.3178380e-05,
9.3089587e-05,
9.3151687e-05,
9.3059375e-05,
9.3112584e-05,
9.3016535e-05,
9.3060401e-05,
9.2960675e-05,
9.2995453e-05,
9.2891942e-05,
9.2917370e-05,
9.2809644e-05,
9.2825979e-05,
9.2714492e-05,
9.2721872e-05,
9.2606667e-05,
9.2604848e-05,
9.2485801e-05,
9.2475006e-05,
9.2352143e-05,
9.2332351e-05,
9.2205838e-05,
9.2176773e-05,
9.2046317e-05,
9.2008241e-05,
9.1873674e-05,
9.1826393e-05,
9.1687826e-05,
9.1631934e-05,
9.1489654e-05,
9.1424956e-05,
9.1278874e-05,
9.1205195e-05,
9.1055068e-05,
9.0972743e-05,
9.0818560e-05,
9.0727262e-05,
9.0569342e-05,
9.0469168e-05,
9.0307262e-05,
9.0198133e-05,
9.0032298e-05,
8.9914348e-05,
8.9745162e-05,
8.9618722e-05,
8.9445740e-05,
8.9310530e-05,
8.9133604e-05,
8.8989565e-05,
8.8808718e-05,
8.8656056e-05,
8.8471413e-05,
8.8310039e-05,
8.8121568e-05,
8.7951556e-05,
8.7759658e-05,
8.7580695e-05,
8.7385330e-05,
8.7197879e-05,
8.6999453e-05,
8.6803182e-05,
8.6600998e-05,
8.6395872e-05,
8.6190074e-05,
8.5976106e-05,
8.5767214e-05,
8.5544542e-05,
8.5332419e-05,
8.5101210e-05,
8.4885567e-05,
8.4645738e-05,
8.4427293e-05,
8.4178747e-05,
8.3957218e-05,
8.3700225e-05,
8.3475544e-05,
8.3209490e-05,
8.2981623e-05,
8.2706846e-05,
8.2476033e-05,
8.2192754e-05,
8.1959482e-05,
8.1667678e-05,
8.1431478e-05,
8.1131109e-05,
8.0891964e-05,
8.0583239e-05,
8.0341359e-05,
8.0024359e-05,
7.9779805e-05,
7.9454233e-05,
7.9206606e-05,
7.8872418e-05,
7.8622395e-05,
7.8280068e-05,
7.8027536e-05,
7.7677439e-05,
7.7422354e-05,
7.7064047e-05,
7.6806153e-05,
7.6439503e-05,
7.6179165e-05,
7.5804786e-05,
7.5541584e-05,
7.5159381e-05,
7.4893898e-05,
7.4503432e-05,
7.4235227e-05,
7.3837086e-05,
7.3566841e-05,
7.3161130e-05,
7.2888625e-05,
7.2475480e-05,
7.2200256e-05,
7.1779169e-05,
7.1501526e-05,
7.1072870e-05,
7.0793173e-05,
7.0357266e-05,
7.0075066e-05,
6.9631854e-05,
6.9347511e-05,
6.8896950e-05,
6.8610447e-05,
6.8152877e-05,
6.7864386e-05,
6.7399518e-05,
6.7109253e-05,
6.6636907e-05,
6.6344349e-05,
6.5864643e-05,
6.5570395e-05,
6.5083972e-05,
6.4788086e-05,
6.4294440e-05,
6.3996879e-05,
6.3496427e-05,
6.3196882e-05,
6.2689766e-05,
6.2388841e-05,
6.1874819e-05,
6.1572181e-05,
6.1051189e-05,
6.0747373e-05,
6.0219221e-05,
5.9914028e-05,
5.9379375e-05,
5.9073281e-05,
5.8532135e-05,
5.8224904e-05,
5.7676631e-05,
5.7368600e-05,
5.6813784e-05,
5.6504533e-05,
5.5943177e-05,
5.5633309e-05,
5.5065267e-05,
5.4754389e-05,
5.4179944e-05,
5.3868339e-05,
5.3287400e-05,
5.2975471e-05,
5.2388504e-05,
5.2076191e-05,
5.1482568e-05,
5.1169596e-05,
5.0569595e-05,
5.0256448e-05,
4.9650120e-05,
4.9336813e-05,
4.8724333e-05,
4.8410805e-05,
4.7792426e-05,
4.7478786e-05,
4.6854529e-05,
4.6540568e-05,
4.5910493e-05,
4.5597014e-05,
4.4961153e-05,
4.4647497e-05,
4.4005478e-05,
4.3691723e-05,
4.3044457e-05,
4.2730945e-05,
4.2077955e-05,
4.1764894e-05,
4.1106709e-05,
4.0793668e-05,
4.0130355e-05,
3.9817766e-05,
3.9149058e-05,
3.8836537e-05,
3.8162812e-05,
3.7851003e-05,
3.7172105e-05,
3.6860444e-05,
3.6176593e-05,
3.5865616e-05,
3.5177190e-05,
3.4866860e-05,
3.4173612e-05,
3.3864080e-05,
3.3166377e-05,
3.2857495e-05,
3.2155050e-05,
3.1846820e-05,
3.1139773e-05,
3.0832263e-05,
3.0121122e-05,
2.9814439e-05,
2.9098908e-05,
2.8793289e-05,
2.8073737e-05,
2.7769518e-05,
2.7045521e-05,
2.6742416e-05,
2.6014363e-05,
2.5712715e-05,
2.4980370e-05,
2.4680255e-05,
2.3943587e-05,
2.3644645e-05,
2.2904239e-05,
2.2607207e-05,
2.1863241e-05,
2.1567663e-05,
2.0819805e-05,
2.0526425e-05,
1.9774939e-05,
1.9483390e-05,
1.8727848e-05,
1.8438129e-05,
1.7679095e-05,
1.7391735e-05,
1.6628730e-05,
1.6343628e-05,
1.5577025e-05,
1.5294340e-05,
1.4524430e-05,
1.4244712e-05,
1.3471340e-05,
1.3193882e-05,
1.2417052e-05,
1.2142381e-05,
1.1362230e-05,
1.1089969e-05,
1.0306579e-05,
1.0037144e-05,
9.2507374e-06,
8.9842765e-06,
8.1947379e-06,
7.9312261e-06,
7.1388620e-06,
6.8789663e-06,
6.0836124e-06,
5.8268660e-06,
5.0283863e-06,
4.7745841e-06,
3.9734809e-06,
3.7227602e-06,
2.9189341e-06,
2.6716836e-06,
1.8654739e-06,
1.6218618e-06,
8.1315100e-07,
5.7273108e-07,
-2.3818570e-07,
-4.7485926e-07,
-1.2879197e-06,
-1.5207505e-06,
-2.3361468e-06,
-2.5658958e-06,
-3.3836681e-06,
-3.6096304e-06,
-4.4289845e-06,
-4.6513034e-06,
-5.4724334e-06,
-5.6905222e-06,
-6.5132407e-06,
-6.7274410e-06,
-7.5519848e-06,
-7.7626473e-06,
-8.5885872e-06,
-8.7948516e-06,
-9.6221959e-06,
-9.8247614e-06,
-1.0653684e-05,
-1.0852379e-05,
-1.1682816e-05,
-1.1876788e-05,
-1.2708100e-05,
-1.2897736e-05,
-1.3729947e-05,
-1.3915377e-05,
-1.4748626e-05,
-1.4929925e-05,
-1.5764364e-05,
-1.5941243e-05,
-1.6776395e-05,
-1.6948547e-05,
-1.7784527e-05,
-1.7952324e-05,
-1.8789318e-05,
-1.8951989e-05,
-1.9789636e-05,
-1.9946991e-05,
-2.0785363e-05,
-2.0937651e-05,
-2.1776609e-05,
-2.1924371e-05,
-2.2764118e-05,
-2.2906775e-05,
-2.3747136e-05,
-2.3884174e-05,
-2.4724800e-05,
-2.4856780e-05,
-2.5697959e-05,
-2.5824310e-05,
-2.6665897e-05,
-2.6786490e-05,
-2.7628084e-05,
-2.7743058e-05,
-2.8585473e-05,
-2.8694594e-05,
-2.9537543e-05,
-2.9640904e-05,
-3.0484176e-05,
-3.0581406e-05,
-3.1424587e-05,
-3.1515418e-05,
-3.2358693e-05,
-3.2443855e-05,
-3.3287267e-05,
-3.3365975e-05,
-3.4209031e-05,
-3.4281716e-05,
-3.5124522e-05,
-3.5191183e-05,
-3.6034320e-05,
-3.6094570e-05,
-3.6937586e-05,
-3.6990984e-05,
-3.7833414e-05,
-3.7880461e-05,
-3.8722194e-05,
-3.8762757e-05,
-3.9604391e-05,
-3.9638588e-05,
-4.0479547e-05,
-4.0507195e-05,
-4.1347239e-05,
-4.1368269e-05,
-4.2207808e-05,
-4.2222462e-05,
-4.3061094e-05,
-4.3069066e-05,
-4.3906529e-05,
-4.3907351e-05,
-4.4743588e-05,
-4.4737935e-05,
-4.5573158e-05,
-4.5561040e-05,
-4.6395418e-05,
-4.6376280e-05,
-4.7209088e-05,
-4.7183293e-05,
-4.8014768e-05,
-4.7981769e-05,
-4.8812013e-05,
-4.8772423e-05,
-4.9600802e-05,
-4.9554133e-05,
-5.0380883e-05,
-5.0327081e-05,
-5.1152543e-05,
-5.1091537e-05,
-5.1915646e-05,
-5.1847863e-05,
-5.2670106e-05,
-5.2594734e-05,
-5.3415031e-05,
-5.3332434e-05,
-5.4150916e-05,
-5.4061022e-05,
-5.4877576e-05,
-5.4780466e-05,
-5.5595046e-05,
-5.5490312e-05,
-5.6303190e-05,
-5.6191112e-05,
-5.7002379e-05,
-5.6882238e-05,
-5.7691646e-05,
-5.7564006e-05,
-5.8371181e-05,
-5.8235527e-05,
-5.9040927e-05,
-5.8897277e-05,
-5.9700645e-05,
-5.9549584e-05,
-6.0351015e-05,
-6.0191700e-05,
-6.0991163e-05,
-6.0823995e-05,
-6.1621416e-05,
-6.1446329e-05,
-6.2241433e-05,
-6.2057887e-05,
-6.2850921e-05,
-6.2658964e-05,
-6.3449993e-05,
-6.3249837e-05,
-6.4038993e-05,
-6.3830265e-05,
-6.4617397e-05,
-6.4400564e-05,
-6.5185703e-05,
-6.4959981e-05,
-6.5742668e-05,
-6.5508955e-05,
-6.6289203e-05,
-6.6046887e-05,
-6.6824495e-05,
-6.6573835e-05,
-6.7349360e-05,
-6.7089849e-05,
-6.7863175e-05,
-6.7595541e-05,
-6.8366344e-05,
-6.8090089e-05,
-6.8858581e-05,
-6.8573537e-05,
-6.9339235e-05,
-6.9045530e-05,
-6.9808449e-05,
-6.9506319e-05,
-7.0266560e-05,
-6.9955737e-05,
-7.0713280e-05,
-7.0394356e-05,
-7.1149193e-05,
-7.0821528e-05,
-7.1573397e-05,
-7.1237429e-05,
-7.1986288e-05,
-7.1641932e-05,
-7.2387815e-05,
-7.2034487e-05,
-7.2776867e-05,
-7.2415321e-05,
-7.3155167e-05,
-7.2784857e-05,
-7.3521626e-05,
-7.3143158e-05,
-7.3876946e-05,
-7.3490145e-05,
-7.4220310e-05,
-7.3824797e-05,
-7.4551724e-05,
-7.4147360e-05,
-7.4871373e-05,
-7.4458590e-05,
-7.5179163e-05,
-7.4757333e-05,
-7.5474624e-05,
-7.5044453e-05,
-7.5759178e-05,
-7.5320174e-05,
-7.6031488e-05,
-7.5584153e-05,
-7.6292206e-05,
-7.5835735e-05,
-7.6540000e-05,
-7.6074713e-05,
-7.6776145e-05,
-7.6301863e-05,
-7.7000462e-05,
-7.6517373e-05,
-7.7212667e-05,
-7.6720470e-05,
-7.7413110e-05,
-7.6912064e-05,
-7.7601535e-05,
-7.7091493e-05,
-7.7777999e-05,
-7.7258528e-05,
-7.7941783e-05,
-7.7412820e-05,
-7.8092906e-05,
-7.7555314e-05,
-7.8232660e-05,
-7.7686298e-05,
-7.8360469e-05,
-7.7804757e-05,
-7.8475971e-05,
-7.7911312e-05,
-7.8579907e-05,
-7.8005923e-05,
-7.8670936e-05,
-7.8087734e-05,
-7.8750368e-05,
-7.8157381e-05,
-7.8816837e-05,
-7.8214896e-05,
-7.8871816e-05,
-7.8261375e-05,
-7.8915135e-05,
-7.8295710e-05,
-7.8946088e-05,
-7.8317208e-05,
-7.8964718e-05,
-7.8327240e-05,
-7.8971649e-05,
-7.8324685e-05,
-7.8965684e-05,
-7.8309758e-05,
-7.8947928e-05,
-7.8283149e-05,
-7.8918255e-05,
-7.8244872e-05,
-7.8877408e-05,
-7.8195122e-05,
-7.8824024e-05,
-7.8132456e-05,
-7.8758097e-05,
-7.8057865e-05,
-7.8680467e-05,
-7.7971843e-05,
-7.8591040e-05,
-7.7873351e-05,
-7.8489527e-05,
-7.7763460e-05,
-7.8376043e-05,
-7.7641910e-05,
-7.8251823e-05,
-7.7508911e-05,
-7.8115596e-05,
-7.7363529e-05,
-7.7966511e-05,
-7.7205782e-05,
-7.7805714e-05,
-7.7037177e-05,
-7.7634014e-05,
-7.6856862e-05,
-7.7450311e-05,
-7.6664745e-05,
-7.7255386e-05,
-7.6461396e-05,
-7.7048651e-05,
-7.6246347e-05,
-7.6831102e-05,
-7.6020102e-05,
-7.6601278e-05,
-7.5781323e-05,
-7.6359555e-05,
-7.5531764e-05,
-7.6107387e-05,
-7.5271532e-05,
-7.5844357e-05,
-7.4999633e-05,
-7.5569349e-05,
-7.4716538e-05,
-7.5283636e-05,
-7.4421977e-05,
-7.4986559e-05,
-7.4116286e-05,
-7.4678088e-05,
-7.3799518e-05,
-7.4358533e-05,
-7.3471128e-05,
-7.4028291e-05,
-7.3132834e-05,
-7.3687769e-05,
-7.2783488e-05,
-7.3335881e-05,
-7.2423011e-05,
-7.2972870e-05,
-7.2051886e-05,
-7.2599366e-05,
-7.1669871e-05,
-7.2215309e-05,
-7.1277585e-05,
-7.1820321e-05,
-7.0874507e-05,
-7.1415549e-05,
-7.0461392e-05,
-7.1000768e-05,
-7.0038052e-05,
-7.0575139e-05,
-6.9603949e-05,
-7.0138652e-05,
-6.9159572e-05,
-6.9692465e-05,
-6.8705339e-05,
-6.9235935e-05,
-6.8241179e-05,
-6.8769687e-05,
-6.7767102e-05,
-6.8293453e-05,
-6.7282937e-05,
-6.7807441e-05,
-6.6789758e-05,
-6.7311941e-05,
-6.6285915e-05,
-6.6805817e-05,
-6.5772445e-05,
-6.6290422e-05,
-6.5249757e-05,
-6.5766260e-05,
-6.4718055e-05,
-6.5232292e-05,
-6.4176930e-05,
-6.4689122e-05,
-6.3626244e-05,
-6.4136634e-05,
-6.3066456e-05,
-6.3574456e-05,
-6.2497861e-05,
-6.3003639e-05,
-6.1919461e-05,
-6.2423512e-05,
-6.1332437e-05,
-6.1835071e-05,
-6.0737134e-05,
-6.1238039e-05,
-6.0133149e-05,
-6.0632085e-05,
-5.9520083e-05,
-6.0017244e-05,
-5.8898392e-05,
-5.9393944e-05,
-5.8268516e-05,
-5.8762497e-05,
-5.7630379e-05,
-5.8123052e-05,
-5.6984376e-05,
-5.7475849e-05,
-5.6330097e-05,
-5.6820458e-05,
-5.5668267e-05,
-5.6157105e-05,
-5.4998142e-05,
-5.5485855e-05,
-5.4320067e-05,
-5.4806508e-05,
-5.3634440e-05,
-5.4119932e-05,
-5.2941540e-05,
-5.3426778e-05,
-5.2241409e-05,
-5.2725602e-05,
-5.1534392e-05,
-5.2017569e-05,
-5.0819355e-05,
-5.1301897e-05,
-5.0097346e-05,
-5.0579510e-05,
-4.9368244e-05,
-4.9850274e-05,
-4.8632777e-05,
-4.9114316e-05,
-4.7890580e-05,
-4.8372104e-05,
-4.7142311e-05,
-4.7623784e-05,
-4.6387520e-05,
-4.6868064e-05,
-4.5625924e-05,
-4.6106049e-05,
-4.4857890e-05,
-4.5338057e-05,
-4.4083665e-05,
-4.4564189e-05,
-4.3303883e-05,
-4.3784662e-05,
-4.2518591e-05,
-4.2999432e-05,
-4.1727328e-05,
-4.2208394e-05,
-4.0930828e-05,
-4.1411430e-05,
-4.0128168e-05,
-4.0608929e-05,
-3.9319849e-05,
-3.9800951e-05,
-3.8507477e-05,
-3.8988769e-05,
-3.7689779e-05,
-3.8171070e-05,
-3.6867502e-05,
-3.7349104e-05,
-3.6040173e-05,
-3.6522014e-05,
-3.5208074e-05,
-3.5689923e-05,
-3.4371010e-05,
-3.4852786e-05,
-3.3529074e-05,
-3.4012046e-05,
-3.2683605e-05,
-3.3166997e-05,
-3.1834762e-05,
-3.2318112e-05,
-3.0981142e-05,
-3.1464448e-05,
-3.0123406e-05,
-3.0607376e-05,
-2.9261614e-05,
-2.9746205e-05,
-2.8396374e-05,
-2.8881340e-05,
-2.7526810e-05,
-2.8012806e-05,
-2.6654700e-05,
-2.7142004e-05,
-2.5780248e-05,
-2.6268192e-05,
-2.4901710e-05,
-2.5390152e-05,
-2.4019646e-05,
-2.4509338e-05,
-2.3134655e-05,
-2.3625535e-05,
-2.2247134e-05,
-2.2739571e-05,
-2.1357079e-05,
-2.1850824e-05,
-2.0464130e-05,
-2.0959776e-05,
-1.9570064e-05,
-2.0067152e-05,
-1.8673505e-05,
-1.9172164e-05,
-1.7773405e-05,
-1.8273656e-05,
-1.6871930e-05,
-1.7374723e-05,
-1.5969235e-05,
-1.6473605e-05,
-1.5065183e-05,
-1.5571995e-05,
-1.4159349e-05,
-1.4668278e-05,
-1.3252300e-05,
-1.3763736e-05,
-1.2344256e-05,
-1.2857882e-05,
-1.1434233e-05,
-1.1950211e-05,
-1.0523061e-05,
-1.1041562e-05,
-9.6120038e-06,
-1.0133180e-05,
-8.7005231e-06,
-9.2238737e-06,
-7.7882939e-06,
-8.3140690e-06,
-6.8751606e-06,
-7.4035374e-06,
-5.9620409e-06,
-6.4932664e-06,
-5.0482045e-06,
-5.5820311e-06,
-4.1342168e-06,
-4.6708282e-06,
-3.2212961e-06,
-3.7612887e-06,
-2.3088621e-06,
-2.8511872e-06,
-1.3963640e-06,
-1.9418418e-06,
-4.8434608e-07,
-1.0318629e-06,
4.2724169e-07,
-1.2344475e-07,
1.3377152e-06,
7.8430335e-07,
2.2478266e-06,
1.6918336e-06,
3.1564538e-06,
2.5970747e-06,
4.0632240e-06,
3.5003006e-06,
4.9693912e-06,
4.4042062e-06,
5.8746899e-06,
5.3057945e-06,
6.7778666e-06,
6.2064152e-06,
7.6791241e-06,
7.1040185e-06,
8.5782748e-06,
8.0004530e-06,
9.4759240e-06,
8.8947277e-06,
1.0371014e-05,
9.7859098e-06,
1.1263760e-05,
1.0675667e-05,
1.2155139e-05,
1.1563777e-05,
1.3044034e-05,
1.2448843e-05,
1.3929883e-05,
1.3330513e-05,
1.4812591e-05,
1.4209675e-05,
1.5692936e-05,
1.5086500e-05,
1.6570870e-05,
1.5959723e-05,
1.7445247e-05,
1.6830140e-05,
1.8316515e-05,
1.7697948e-05,
1.9184956e-05,
1.8561159e-05,
2.0049319e-05,
1.9421355e-05,
2.0910192e-05,
2.0276651e-05,
2.1766731e-05,
2.1129685e-05,
2.2620560e-05,
2.1978695e-05,
2.3470274e-05,
2.2823182e-05,
2.4315587e-05,
2.3664145e-05,
2.5156955e-05,
2.4500390e-05,
2.5993801e-05,
2.5331430e-05,
2.6826100e-05,
2.6159207e-05,
2.7654345e-05,
2.6981928e-05,
2.8477536e-05,
2.7800687e-05,
2.9296666e-05,
2.8614458e-05,
3.0109868e-05,
2.9422817e-05,
3.0918385e-05,
3.0226054e-05,
3.1721860e-05,
3.1023902e-05,
3.2519506e-05,
3.1817057e-05,
3.3312745e-05,
3.2605163e-05,
3.4100633e-05,
3.3388086e-05,
3.4883135e-05,
3.4164653e-05,
3.5659499e-05,
3.4936042e-05,
3.6429689e-05,
3.5701577e-05,
3.7194852e-05,
3.6461206e-05,
3.7953996e-05,
3.7215602e-05,
3.8706976e-05,
3.7964363e-05,
3.9454664e-05,
3.8706030e-05,
4.0195434e-05,
3.9441819e-05,
4.0930272e-05,
4.0170153e-05,
4.1657418e-05,
4.0892322e-05,
4.2378930e-05,
4.1608694e-05,
4.3093981e-05,
4.2318469e-05,
4.3802343e-05,
4.3021571e-05,
4.4504028e-05,
4.3717302e-05,
4.5198287e-05,
4.4405596e-05,
4.5885319e-05,
4.5087304e-05,
4.6565778e-05,
4.5761852e-05,
4.7238522e-05,
4.6428660e-05,
4.7905244e-05,
4.7089743e-05,
4.8564226e-05,
4.7742936e-05,
4.9215755e-05,
4.8387433e-05,
4.9859272e-05,
4.9024193e-05,
5.0494319e-05,
4.9653911e-05,
5.1122826e-05,
5.0276187e-05,
5.1743636e-05,
5.0890904e-05,
5.2356825e-05,
5.1496994e-05,
5.2962228e-05,
5.2096187e-05,
5.3559736e-05,
5.2686737e-05,
5.4148444e-05,
5.3268222e-05,
5.4728613e-05,
5.3842517e-05,
5.5301436e-05,
5.4409385e-05,
5.5867250e-05,
5.4967912e-05,
5.6423881e-05,
5.5517485e-05,
5.6972203e-05,
5.6058743e-05,
5.7511392e-05,
5.6590856e-05,
5.8042552e-05,
5.7114750e-05,
5.8563903e-05,
5.7629975e-05,
5.9077313e-05,
5.8137329e-05,
5.9583404e-05,
5.8636223e-05,
6.0080142e-05,
5.9125895e-05,
6.0567026e-05,
5.9605935e-05,
6.1045256e-05,
6.0078191e-05,
6.1514678e-05,
6.0540198e-05,
6.1975293e-05,
6.0994543e-05,
6.2427096e-05,
6.1440352e-05,
6.2870780e-05,
6.1876773e-05,
6.3304831e-05,
6.2304041e-05,
6.3728387e-05,
6.2721383e-05,
6.4142741e-05,
6.3129795e-05,
6.4548467e-05,
6.3529506e-05,
6.4945313e-05,
6.3919327e-05,
6.5332250e-05,
6.4300349e-05,
6.5711559e-05,
6.4672722e-05,
6.6080637e-05,
6.5034313e-05,
6.6438674e-05,
6.5386385e-05,
6.6787730e-05,
6.5728866e-05,
6.7128263e-05,
6.6063284e-05,
6.7459535e-05,
6.6387169e-05,
6.7780590e-05,
6.6701779e-05,
6.8091801e-05,
6.7005981e-05,
6.8394441e-05,
6.7301211e-05,
6.8685380e-05,
6.7584921e-05,
6.8966325e-05,
6.7858897e-05,
6.9238935e-05,
6.8124297e-05,
6.9502424e-05,
6.8380519e-05,
6.9755136e-05,
6.8625164e-05,
6.9996922e-05,
6.8860628e-05,
7.0230102e-05,
6.9085949e-05,
7.0452887e-05,
6.9301421e-05,
7.0665467e-05,
6.9506664e-05,
7.0869073e-05,
6.9702340e-05,
7.1062843e-05,
6.9889672e-05,
7.1247064e-05,
7.0065644e-05,
7.1419664e-05,
7.0230858e-05,
7.1582265e-05,
7.0386663e-05,
7.1736264e-05,
7.0533249e-05,
7.1879150e-05,
7.0668425e-05,
7.2013267e-05,
7.0795394e-05,
7.2137224e-05,
7.0911846e-05,
7.2250328e-05,
7.1018700e-05,
7.2354027e-05,
7.1114112e-05,
7.2446858e-05,
7.1200503e-05,
7.2530767e-05,
7.1277387e-05,
7.2604456e-05,
7.1344794e-05,
7.2668046e-05,
7.1400766e-05,
7.2722170e-05,
7.1448501e-05,
7.2765305e-05,
7.1484561e-05,
7.2798425e-05,
7.1510175e-05,
7.2821382e-05,
7.1526451e-05,
7.2835021e-05,
7.1533724e-05,
7.2839032e-05,
7.1530742e-05,
7.2831535e-05,
7.1517285e-05,
7.2815055e-05,
7.1494597e-05,
7.2788336e-05,
7.1461357e-05,
7.2751860e-05,
7.1418214e-05,
7.2706245e-05,
7.1365070e-05,
7.2649791e-05,
7.1303850e-05,
7.2585533e-05,
7.1232387e-05,
7.2510267e-05,
7.1149794e-05,
7.2424192e-05,
7.1057081e-05,
7.2329112e-05,
7.0956766e-05,
7.2224394e-05,
7.0844179e-05,
7.2110339e-05,
7.0724176e-05,
7.1986444e-05,
7.0593349e-05,
7.1852307e-05,
7.0453398e-05,
7.1710028e-05,
7.0303286e-05,
7.1556638e-05,
7.0142325e-05,
7.1394138e-05,
6.9972903e-05,
7.1222459e-05,
6.9795334e-05,
7.1042161e-05,
6.9607215e-05,
7.0851844e-05,
6.9410657e-05,
7.0651362e-05,
6.9202831e-05,
7.0442115e-05,
6.8985641e-05,
7.0223057e-05,
6.8759902e-05,
6.9994578e-05,
6.8524432e-05,
6.9757396e-05,
6.8280609e-05,
6.9511109e-05,
6.8027216e-05,
6.9255806e-05,
6.7766106e-05,
6.8990732e-05,
6.7493111e-05,
6.8715968e-05,
6.7212001e-05,
6.8433675e-05,
6.6922799e-05,
6.8141560e-05,
6.6625504e-05,
6.7842005e-05,
6.6319771e-05,
6.7533065e-05,
6.6004248e-05,
6.7215245e-05,
6.5679450e-05,
6.6887846e-05,
6.5347539e-05,
6.6551868e-05,
6.5003527e-05,
6.6206310e-05,
6.4654328e-05,
6.5853637e-05,
6.4296375e-05,
6.5492137e-05,
6.3928686e-05,
6.5122529e-05,
6.3553235e-05,
6.4743287e-05,
6.3167445e-05,
6.4355276e-05,
6.2773843e-05,
6.3958987e-05,
6.2372291e-05,
6.3554877e-05,
6.1963092e-05,
6.3142474e-05,
6.1546801e-05,
6.2722627e-05,
6.1121028e-05,
6.2294298e-05,
6.0685854e-05,
6.1856683e-05,
6.0243269e-05,
6.1411215e-05,
5.9792103e-05,
6.0957274e-05,
5.9333195e-05,
6.0496814e-05,
5.8866306e-05,
6.0028345e-05,
5.8394686e-05,
5.9553407e-05,
5.7911848e-05,
5.9068732e-05,
5.7421404e-05,
5.8576328e-05,
5.6924376e-05,
5.8076515e-05,
5.6419886e-05,
5.7570887e-05,
5.5909288e-05,
5.7057533e-05,
5.5390212e-05,
5.6537928e-05,
5.4864675e-05,
5.6011144e-05,
5.4333117e-05,
5.5477700e-05,
5.3790774e-05,
5.4933114e-05,
5.3242466e-05,
5.4383632e-05,
5.2689299e-05,
5.3829047e-05,
5.2128168e-05,
5.3267211e-05,
5.1560914e-05,
5.2698042e-05,
5.0985600e-05,
5.2121634e-05,
5.0404596e-05,
5.1539928e-05,
4.9816060e-05,
5.0949895e-05,
4.9221381e-05,
5.0352528e-05,
4.8620289e-05,
4.9752310e-05,
4.8015201e-05,
4.9146672e-05,
4.7403050e-05,
4.8532042e-05,
4.6783411e-05,
4.7911425e-05,
4.6158490e-05,
4.7284188e-05,
4.5527132e-05,
4.6652403e-05,
4.4889402e-05,
4.6012549e-05,
4.4247565e-05,
4.5369900e-05,
4.3599143e-05,
4.4720980e-05,
4.2946129e-05,
4.4065484e-05,
4.2287308e-05,
4.3403060e-05,
4.1620937e-05,
4.2736097e-05,
4.0951762e-05,
4.2064560e-05,
4.0275971e-05,
4.1387611e-05,
3.9595810e-05,
4.0706101e-05,
3.8910599e-05,
4.0021267e-05,
3.8220873e-05,
3.9326881e-05,
3.7523888e-05,
3.8627737e-05,
3.6822652e-05,
3.7925386e-05,
3.6116069e-05,
3.7217527e-05,
3.5405240e-05,
3.6505287e-05,
3.4689194e-05,
3.5786907e-05,
3.3968410e-05,
3.5066475e-05,
3.3242847e-05,
3.4339729e-05,
3.2513371e-05,
3.3606902e-05,
3.1776086e-05,
3.2871595e-05,
3.1037862e-05,
3.2134889e-05,
3.0297192e-05,
3.1391937e-05,
2.9549356e-05,
3.0644213e-05,
2.8799480e-05,
2.9892864e-05,
2.8044557e-05,
2.9138932e-05,
2.7287180e-05,
2.8379553e-05,
2.6523638e-05,
2.7618335e-05,
2.5756739e-05,
2.6853011e-05,
2.4989215e-05,
2.6085609e-05,
2.4218850e-05,
2.5313289e-05,
2.3440861e-05,
2.4536354e-05,
2.2662424e-05,
2.3758937e-05,
2.1881261e-05,
2.2977814e-05,
2.1098283e-05,
2.2194650e-05,
2.0309438e-05,
2.1408684e-05,
1.9521733e-05,
2.0618636e-05,
1.8729512e-05,
1.9826110e-05,
1.7934114e-05,
1.9031224e-05,
1.7135218e-05,
1.8232449e-05,
1.6333661e-05,
1.7432676e-05,
1.5532430e-05,
1.6630042e-05,
1.4726658e-05,
1.5824794e-05,
1.3918709e-05,
1.5015280e-05,
1.3109504e-05,
1.4205717e-05,
1.2295196e-05,
1.3393108e-05,
1.1480920e-05,
1.2580490e-05,
1.0668059e-05,
1.1765798e-05,
9.8498125e-06,
1.0948224e-05,
9.0337465e-06,
1.0131264e-05,
8.2129703e-06,
9.3097692e-06,
7.3929788e-06,
8.4879929e-06,
6.5682104e-06,
7.6669472e-06,
5.7440164e-06,
6.8426839e-06,
4.9202234e-06,
6.0189344e-06,
4.0969429e-06,
5.1936395e-06,
3.2680707e-06,
4.3646473e-06,
2.4395267e-06,
3.5372801e-06,
1.6108134e-06,
2.7077838e-06,
7.8203613e-07,
1.8809428e-06,
-4.8334197e-08,
1.0512499e-06,
-8.7550631e-07,
2.2276490e-07,
-1.7055410e-06,
-6.0604069e-07,
-2.5369632e-06,
-1.4360208e-06,
-3.3689832e-06,
-2.2672616e-06,
-4.2039246e-06,
-3.0992246e-06,
-5.0336636e-06,
-3.9270621e-06,
-5.8653830e-06,
-4.7571214e-06,
-6.6954987e-06,
-5.5902621e-06,
-7.5281327e-06,
-6.4189236e-06,
-8.3620792e-06,
-7.2497252e-06,
-9.1957158e-06,
-8.0796492e-06,
-1.0024612e-05,
-8.9073043e-06,
-1.0857331e-05,
-9.7365164e-06,
-1.1683135e-05,
-1.0557569e-05,
-1.2508924e-05,
-1.1385080e-05,
-1.3336272e-05,
-1.2210597e-05,
-1.4166821e-05,
-1.3035108e-05,
-1.4991443e-05,
-1.3859287e-05,
-1.5815249e-05,
-1.4681064e-05,
-1.6636240e-05,
-1.5502106e-05,
-1.7459738e-05,
-1.6323450e-05,
-1.8281166e-05,
-1.7141709e-05,
-1.9099872e-05,
-1.7962811e-05,
-1.9924355e-05,
-1.8781995e-05,
-2.0743640e-05,
-1.9602296e-05,
-2.1556686e-05,
-2.0414802e-05,
-2.2369743e-05,
-2.1226687e-05,
-2.3184294e-05,
-2.2040793e-05,
-2.3995714e-05,
-2.2850841e-05,
-2.4809359e-05,
-2.3664472e-05,
-2.5618644e-05,
-2.4471578e-05,
-2.6428554e-05,
-2.5281903e-05,
-2.7231724e-05,
-2.6087473e-05,
-2.8035954e-05,
-2.6886197e-05,
-2.8837431e-05,
-2.7688003e-05,
-2.9641959e-05,
-2.8490381e-05,
-3.0438469e-05,
-2.9285577e-05,
-3.1236491e-05,
-3.0083208e-05,
-3.2029858e-05,
-3.0868817e-05,
-3.2812985e-05,
-3.1655869e-05,
-3.3599037e-05,
-3.2435441e-05,
-3.4383511e-05,
-3.3216858e-05,
-3.5160780e-05,
-3.3996071e-05,
-3.5938971e-05,
-3.4770626e-05,
-3.6711453e-05,
-3.5542043e-05,
-3.7482418e-05,
-3.6309763e-05,
-3.8251089e-05,
-3.7071665e-05,
-3.9009114e-05,
-3.7828688e-05,
-3.9776100e-05,
-3.8590361e-05,
-4.0534325e-05,
-3.9351892e-05,
-4.1288018e-05,
-4.0100665e-05,
-4.2036072e-05,
-4.0844239e-05,
-4.2784722e-05,
-4.1590594e-05,
-4.3526292e-05,
-4.2325787e-05,
-4.4263896e-05,
-4.3061791e-05,
-4.4998650e-05,
-4.3788906e-05,
-4.5731414e-05,
-4.4526946e-05,
-4.6461609e-05,
-4.5250924e-05,
-4.7187732e-05,
-4.5965251e-05,
-4.7901952e-05,
-4.6680341e-05,
-4.8623507e-05,
-4.7399253e-05,
-4.9335911e-05,
-4.8105587e-05,
-5.0040449e-05,
-4.8811754e-05,
-5.0748311e-05,
-4.9512508e-05,
-5.1441662e-05,
-5.0208915e-05,
-5.2143019e-05,
-5.0893393e-05,
-5.2828302e-05,
-5.1577256e-05,
-5.3504719e-05,
-5.2259637e-05,
-5.4187088e-05,
-5.2939353e-05,
-5.4863460e-05,
-5.3612409e-05,
-5.5530658e-05,
-5.4279589e-05,
-5.6200342e-05,
-5.4949895e-05,
-5.6861646e-05,
-5.5602788e-05,
-5.7524855e-05,
-5.6267675e-05,
-5.8174698e-05,
-5.6924826e-05,
-5.8826476e-05,
-5.7573908e-05,
-5.9471074e-05,
-5.8216137e-05,
-6.0115307e-05,
-5.8858680e-05,
-6.0750281e-05,
-5.9487095e-05,
-6.1375613e-05,
-6.0116183e-05,
-6.2003190e-05,
-6.0732265e-05,
-6.2611362e-05,
-6.1357784e-05,
-6.3230417e-05,
-6.1962026e-05,
-6.3840955e-05,
-6.2559967e-05,
-6.4432321e-05,
-6.3154212e-05,
-6.5031468e-05,
-6.3753891e-05,
-6.5625919e-05,
-6.4337568e-05,
-6.6200549e-05,
-6.4911178e-05,
-6.6780482e-05,
-6.5489152e-05,
-6.7342008e-05,
-6.6043909e-05,
-6.7919373e-05,
-6.6596798e-05,
-6.8464831e-05,
-6.7139161e-05,
-6.8999221e-05,
-6.7677717e-05,
-6.9541129e-05,
-6.8218697e-05,
-7.0080171e-05,
-6.8745823e-05,
-7.0596144e-05,
-6.9252187e-05,
-7.1109166e-05,
-6.9774339e-05,
-7.1633177e-05,
-7.0269766e-05,
-7.2137726e-05,
-7.0792612e-05,
-7.2634637e-05,
-7.1287282e-05,
-7.3136423e-05,
-7.1782022e-05,
-7.3633276e-05,
-7.2274574e-05,
-7.4125949e-05,
-7.2757866e-05,
-7.4597922e-05,
-7.3214474e-05,
-7.5048908e-05,
-7.3675058e-05,
-7.5524209e-05,
-7.4143081e-05,
-7.5949237e-05,
-7.4592949e-05,
-7.6408310e-05,
-7.5016652e-05,
-7.6841388e-05,
-7.5437464e-05,
-7.7254079e-05,
-7.5849188e-05,
-7.7664931e-05,
-7.6260426e-05,
-7.8068171e-05,
-7.6652472e-05,
-7.8444223e-05,
-7.7029953e-05,
-7.8834901e-05,
-7.7447527e-05,
-7.9219838e-05,
-7.7779545e-05,
-7.9595592e-05,
-7.8143707e-05,
-7.9918247e-05,
-7.8470847e-05,
-8.0228176e-05,
-7.8784212e-05,
-8.0539969e-05,
-7.9101842e-05,
-8.0852096e-05,
-7.9403419e-05,
-8.1131448e-05,
-7.9664053e-05,
-8.1391304e-05,
-7.9948454e-05,
-8.1711571e-05,
-8.0192915e-05,
-8.1898032e-05,
-8.0464845e-05,
-8.2123319e-05,
-8.0674935e-05,
-8.2347859e-05,
-8.0900202e-05,
-8.2584033e-05,
-8.1140777e-05,
-8.2822996e-05,
-8.1364058e-05,
-8.3016848e-05,
-8.1518937e-05,
-8.3139843e-05,
-8.1641049e-05,
-8.3307477e-05,
-8.1861297e-05,
-8.3338424e-05,
-8.1872369e-05,
-8.3465103e-05,
-8.1917644e-05,
-8.3519772e-05,
-8.1969287e-05,
-8.3569990e-05,
-8.2010098e-05,
-8.3597202e-05,
-8.2009317e-05,
-8.3558005e-05,
-8.1921990e-05,
-8.3420935e-05,
-8.1769647e-05,
-8.3293519e-05,
-8.1821812e-05,
-8.3418104e-05,
-8.1507153e-05,
-8.3255531e-05,
-8.1495185e-05,
-8.3119961e-05,
-8.1372797e-05,
-8.2969287e-05,
-8.1185934e-05,
-8.2747052e-05,
-8.0893543e-05,
-8.2405598e-05,
-8.0451000e-05,
-8.1921510e-05,
-7.9861450e-05,
-8.1407417e-05,
-7.9348578e-05,
-8.1816816e-05,
-7.9418760e-05,
-8.0401456e-05,
-7.8877499e-05,
-8.0034419e-05,
-7.8254896e-05,
-7.9462951e-05,
-7.7595289e-05,
-7.8806089e-05,
-7.6878719e-05,
-7.8076167e-05,
-7.6098940e-05,
-7.7274005e-05,
-7.5250027e-05,
-7.6394920e-05,
-7.4324210e-05,
-7.5431544e-05,
-5.6931991e-04
};
